      subroutine lhaReadEvAsc(lun,*)
      implicit none

C  Read back one event record of the LHA external event interface format 
C  written by "lhaWritEvAsc", from an ASCII-formatted file (LUN = "lun").
C  "Return 1" if the end-of-file is detected.
C  May 6, 2004, S. Odaka (KEK)
C
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      integer lun,i,j

C  Event header
      read(lun,*,end=9) NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP

C  Particle data
      do i=1,NUP
         read(lun,*) IDUP(i),ISTUP(i),(MOTHUP(j,i),j=1,2),
     >        (ICOLUP(j,i),j=1,2),(PUP(j,i),j=1,5),VTIMUP(i),SPINUP(i)
      enddo

      return

 9    continue
      NUP= 0
      return 1

 100  format(2I6,4E15.7)
 101  format(6I6,7E15.7)
      end
      subroutine lhaReadEvBin(lun,*)
      implicit none

C  Read back one event record of the LHA external event interface format 
C  written by "lhaWritEvBin", from an unformatted binary file (LUN = "lun").
C  "Return 1" if the end-of-file is detected.
C  May 6, 2004, S. Odaka (KEK)
C
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      integer lun,i,j

      read(lun,end=9) NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     >     (IDUP(i),ISTUP(i),(MOTHUP(j,i),j=1,2),(ICOLUP(j,i),j=1,2),
     >     (PUP(j,i),j=1,5),VTIMUP(i),SPINUP(i),i=1,NUP)

      return

 9    continue
      NUP= 0
      return 1
      end
      subroutine lhaReadHdAsc(lun,mode)
      implicit none

C  Read back the "header" (run information) records of the LHA external 
C  event interface written by "lhaWritHdAsc", from an ASCII-formatted 
C  file (LUN = "lun").
C   mode = 0  replace the whole record
C          1  replace the header part, and add the processes
C          2  keep the header part, and add the processes
C          3  dummy read
C  May 6, 2004, S. Odaka (KEK)
C
C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER IDBM(2),PDFG(2),PDFS(2),IDWT,NPR,LPR
      DOUBLE PRECISION EBM(2),XSEC,XERR,XMAX
      integer lun,mode,i,j

C  Header data
      read(lun,*) IDBM,EBM,PDFG,PDFS,IDWT,NPR
      if( mode.le.1 ) then
         do i=1,2
            IDBMUP(i)= IDBM(i)
            EBMUP(i)= EBM(i)
            PDFGUP(i)= PDFG(i)
            PDFSUP(i)= PDFS(i)
         enddo
         IDWTUP= IDWT
      endif

C  Process data
      if( mode.eq.0 ) then
         j= 0
      else
         j= NPRUP
      endif
      do i=1,NPR
         read(lun,*) XSEC,XERR,XMAX,LPR
         if( mode.le.2 ) then
            j= j +1
            if( j.le.MAXPUP ) then
               XSECUP(j)= XSEC
               XERRUP(j)= XERR
               XMAXUP(j)= XMAX
               LPRUP(j)= LPR
            else
               print *,'lhaReadHd (warning):',
     >              ' num. of processes exceeded the maximum.',
     >              ' -> ignored'
            endif
         endif
      enddo
      if( mode.le.2 ) NPRUP= j

      return
      end
      subroutine lhaReadHdBin(lun,mode)
      implicit none

C  Read back the "header" (run information) records of the LHA external 
C  event interface written by "lhaWritHdBin", from an unformatted banary 
C  file (LUN = "lun").
C   mode = 0  replace the whole record
C          1  replace the header part, and add the processes
C          2  keep the header part, and add the processes
C          3  dummy read
C  May 6, 2004, S. Odaka (KEK)
C
C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER IDBM(2),PDFG(2),PDFS(2),IDWT,NPR,LPR
      DOUBLE PRECISION EBM(2),XSEC,XERR,XMAX
      integer lun,mode,i,j

C  Header data
      read(lun) IDBM,EBM,PDFG,PDFS,IDWT,NPR
      if( mode.le.1 ) then
         do i=1,2
            IDBMUP(i)= IDBM(i)
            EBMUP(i)= EBM(i)
            PDFGUP(i)= PDFG(i)
            PDFSUP(i)= PDFS(i)
         enddo
         IDWTUP= IDWT
      endif

C  Process data
      if( mode.eq.0 ) then
         j= 0
      else
         j= NPRUP
      endif
      do i=1,NPR
         read(lun) XSEC,XERR,XMAX,LPR
         if( mode.le.2 ) then
            j= j +1
            if( j.le.MAXPUP ) then
               XSECUP(j)= XSEC
               XERRUP(j)= XERR
               XMAXUP(j)= XMAX
               LPRUP(j)= LPR
            else
               print *,'lhaReadHd (warning):',
     >              ' num. of processes exceeded the maximum.',
     >              ' -> ignored'
            endif
         endif
      enddo
      if( mode.le.2 ) NPRUP= j

      return
      end
