      SUBROUTINE UPINIT

C  An "LhaExt" version of UPINIT.
C
C  This routine opens a data file, reads in the "run information" from the 
C  header records, and skips siecified number of events.
C  LUNs for the control file and the data file, and other control parameters 
C  must be given in a read-control file named "lharead_control.txt".
C
C  June 8, 2004, S. Odaka (KEK)

      IMPLICIT NONE

      integer lunctl,lundat,ifmt
      common /lharead/lunctl,lundat,ifmt

      character*80 lhactl
      parameter ( lhactl= 'lharead_control.txt' )
      integer lun,idwt,nSkip,iev
      integer lun1,lun2

C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      print *,' '
      print *,'============================== UPINIT =====',
     >     '=================================='

C  Open the control file.
      lun= 10
      call lunfind( lun )
      lunctl= lun
      open(lunctl,file=lhactl,status='OLD')
      print '('' LHA control file opened: LUN = '',I3)',lunctl
      print '(''   file name = '',A80)',lhactl

C  Read the 1st line: LUNs for the control file and the data file.
      read(lunctl,*) lun1,lun2
      if( lun1.ge.1 ) then
         close( lunctl )
         lunctl= lun1
         print '('' LUN for the control file reassigned: LUN = '',
     >        I3)',lunctl
         open(lunctl,file=lhactl,status='OLD')
         read(lunctl,*) lun1
      endif

C  Read the 2nd line: the parameter for the weighting strategy.
      read(lunctl,*) idwt
      print '('' Specified weighting strategy = '',I3)',idwt
      if( idwt.eq.0 ) then
         print *,'   -> no overwrite'
      else
         print *,'   -> overwrite'
      endif

C  Read the 3rd line: number of events to skip.
      read(lunctl,*) nSkip
      print '('' First '',I3,'' events will be skipped.'')',nSkip

C  Open the 1st data file.
      if( lun2.ge.1 ) then
         lundat= lun2
      else
         call lunfind( lun )
         lundat= lun
      endif
      print '('' Input data file: LUN = '',I3)',lundat

      call lhaopen( lunctl, lundat, ifmt, *9 )

C  Update the run information.
      print *,'Read the header record'
      if( ifmt.eq.0 ) then
         call lhaReadHdAsc( lundat, 0 )
      elseif( ifmt.eq.1 ) then
         call lhaReadHdBin( lundat, 0 )
      endif

C  Overwrite the weighting strategy parameter if specified.
      if( idwt.ne.0 ) IDWTUP= idwt
      print '('' -> weighting strategy = '',I3)',IDWTUP

C  Skip specified number of events.
      if( nSkip.gt.0 ) then
         do iev=1,nSkip
            call lhaReadEvnt(lunctl,lundat,ifmt,*8)
         enddo
         print '('' First '',I3,'' events skipped.'')',nSkip
      endif

      print *,'===========================================',
     >     '=================================='

      return

 8    continue
      print *,'End-of-data during the event skip -> stop'
      stop

 9    continue
      print *,'(error) no input-data file given'
      stop

      end

      subroutine lunfind( lun )
      integer lun
      logical lopen

 1    inquire( lun, opened=lopen )
      if( lopen ) then
         lun= lun +1
         goto 1
      endif

      return
      end

      subroutine lhaopen( lunctl, lundat, ifmt, * )

      character*10 filefmt
      character*80 filename

C  Read the information for the 1st data file.
      call lhactlrd( lunctl, filefmt, filename, *9 )
      print '('' New input file: format = '',A10)',filefmt
      print '(''   file name = '',A80)',filename

C  Open the 1st data file.
      if( filefmt(1:3).eq.'asc' ) then
         ifmt= 0
         open(lundat,file=filename,form='FORMATTED',status='OLD')
      elseif( filefmt(1:3).eq.'bin' ) then
         ifmt= 1
         open(lundat,file=filename,form='UNFORMATTED',status='OLD')
      endif

      print *,'   -> successfully opened'

      return

 9    return 1

      end

      subroutine lhactlrd( lunctl, filefmt, filename, * )

      character*10 filefmt
      character*80 filename
      character*80 string

 1    read(lunctl,'(A80)',end=9) string
C      print *,'Input string:'
C      print '(A80)',string

      if( string(1:1).eq.'*' ) goto 1
      if( string(1:4).eq.'end ' .or. string(1:4).eq.'End '
     >     .or. string(1:4).eq.'END ' ) goto 9

      isep= 0
      do i=1,80
         isep= isep +1
         if( string(isep:isep).eq.':' ) goto 2
      enddo
      print *,'LHACTLRD (error): no separator in the control list'
      stop

 2    continue
      filefmt= string(1:isep-1)
      filename= string(isep+1:80)

      return

 9    continue
      print *,'LHACTLRD: end of the control list'
      return 1

      end
