*****************************************************************************
* QCDPS program library V1.4.4
* Auther : 
*  V1.0: Based on the Kato-note and QEDPS made by Munehisa (Yamanashi Univ.)
*                                         Y. Kurihara (KEK)   03/Aug./2001
*  V1.3: Tanaka-Munehisa algorithm  for singlet-part
*    Mod. Phys. Lett. A13 (1998) 1085,    Y. Kurihara (KEK)   10/Dec./2001
*  V1.3 completed                         Y. Kurihara (KEK)   26/Dec./2001
*  V1.4: X-deterministic PS               Y. Kurihara (KEK)   27/Dec./2001
*  V1.4.2:x-determination improved        Y. Kurihara (KEK)   14/Jan./2002
*  V1.4.3:Fix bug in first x fixing.      Y. Kurihara (KEK)   06/Mar./2002
*  V1.4.4:Fix bug in QPXfix.              Y. Kurihara (KEK)   11/Mar./2002
*  V1.4.5:Boost up non-diagonal branch    Y. Kurihara (KEK)   20/Oct./2003
*  V1.4.6:fix bug in non-siglet treatment Y. Kurihara (KEK)   22/Oct./2003
*  V1.4.7:fix the particle replacement       S. Odaka (KEK)   24/Jul./2007
*  V1.4.8:migration to GR@PPA                S. Odaka (KEK)   25/Feb./2008
*     New commons QPinfo, QPlock, PSKfail were added. The min. energy 
*     scale "Qps" is now able to be refered externally. A new steering 
*     routine "runQCDPS" was made to implement additional functions.
*     "grcpdf1" is used to refer the starting PDF.
*  V1.4.9:QPsetLd added in QPINIT            S. Odaka (KEK)   10/Mar./2008
*     This subroutine import the Lambda value from PDF if possible.
*     Users can choose another value using the same routine later.
*  V1.4.10:further bug fix for non-diagonals S. Odaka (KEK)   18/Mar./2008
*****************************************************************************
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPINIT(Eb)
      IMPLICIT REAL*8 (A-H,O-Z)
C
      character Version*80
      parameter(version='V1.4.10 (18/Mar./2008)')
C
      include 'qpincl.h'
      COMMON /QPRND / NSEED
      common/QPinfo/Qps,ipps(3,2)
      common/QPlock/xpdf_lock(2),iqplock
      common/PSKfail/ ifail_pskini,nfail_pskini
C
      write(6,'('' QCDPS '',A80)') Version
*
      Ebeam=Eb
C Minimum energy scale (GeV) of PS
      Qps= 4.6d0
C For PSKINI
      ifail_pskini= 0
      nfail_pskini= 0
C Make sure that QCDPS is unlocked at the start.
      iqplock= 0

* Random number seed
      NSEED = 1805113725
* Maximum Q2 for Sudakov
      Q2MAX = 4*Ebeam**2
* Minimum Q2 for Sudakov
      Q2MIN = 4.5d0**2
c     Q2MIN = 1.5d0**2
* Initial Q2 for PDF 
c     Q2PDF(0) = 1.6d0**2
c     Q2PDF(1) = 1.6d0**2
c     Q2PDF(2) = 1.6d0**2
c     Q2PDF(3) = 1.6d0**2
c     Q2PDF(4) = 3.4d0**2
c     Q2PDF(5) = 4.6d0**2
      Q2PDF(0) = Qps**2
      Q2PDF(1) = Qps**2
      Q2PDF(2) = Qps**2
      Q2PDF(3) = Qps**2
      Q2PDF(4) = Qps**2
      Q2PDF(5) = Qps**2
C--- PDF initialization is done by GR@PPA.
C* set PDF parameters (CTEQ5L)
C      Iset=3
C      call SetCtq5(Iset)
* QCD Lambda
C  CTEQ5L
C      QCDLd = 0.1460d0
C  CTEQ6L1
      QCDLd = 0.1652d0
C  Import the value in PDF if possible.
      call QPsetLd( -1.d0 )

      beta = 11-2.d0/3.d0*Nf

      aLmd2=1.d-6
      aLmd2=1.d-12

C      eps=1.d-10
      eps=1.d-6
C      eps=1.d-3

      Tmax = LOG(Q2max/QCDLd**2)
      Tmin = LOG(Q2min/QCDLd**2)
      DT   = (Tmax-Tmin)/NSDK
c
      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      FUNCTION alphaQCD(Q2)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
*
      alphaQCD = 4*pi/beta/log(Q2/QCDLd**2)
*
      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      FUNCTION sudakov(ID1,q22,q21)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      common/qptest/CggF
c     CggF=1.0d0
      CggF=2.0d0
*

      T1=log(q21/QCDLd**2)
      T2=log(q22/QCDLd**2)

      T0=log(aLmd2/QCDLd**2)
*
      if     (ID1.eq.21) then
c      P2=2.d0/beta*2*Cg*( (T2-T1)-(T0+11.d0/12.d0)*log(T2/T1) )
c      P2=2.d0/beta  *Cg*( (T2-T1)-(T0+11.d0/12.d0)*log(T2/T1) )
c    .+   2.d0/beta*2*Nf*Tr/3.d0*log(T2/T1)
*Constant upper bound ***********
       P2=2.d0/beta*(log(T2/T1))*
     .(CggF*Cg*(log(1/eps)-11.d0/12.d0)+2*Nf*Tr/3.d0)
*********************************
      else if(ID1.ne.21) then
c      P2=2*Cf/beta*(2*(T2-T1)-log(T2/T1)*(2*T0+1.5d0))
*Constant upper bound ***********
       P2=2*Cf/beta*(log(T2/T1))*(2*log(1/eps)-3.d0/2.d0)
*********************************
      end if

      sudakov=exp(-p2)

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPGEN1(NAD)
C                                                                               
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      common/qptest/CggF
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
*
      ID  = NLPTN(1,NAD)
      X   =    PLPTN(1,NAD)
      Q21 =abs(PLPTN(2,NAD))
      Q22 = Es2
*********************** 
* Start Parton Shower *
*********************** 
1     continue
      SDKV21=sudakov(ID,Q22,Q21)
      eta=QPRAND(Idum)
      if(SDKV21.gt.eta) goto 2
*
* Branch!
*
*New Q2 determination
*
      Q21o=Q21
      eps1=aLmd2/Q21
      eps1=eps
      T1=log(Q21/QCDLd**2)
      if     (ID.eq.21) then
c      Ceps=2*Cg*(log(1/eps1)-11.d0/12.d0)+Tr*2*Nf/3.d0
       Ceps=CggF*Cg*(log(1/eps1)-11.d0/12.d0)+Tr*2*Nf/3.d0
      else if(ID.ne.21) then
       Ceps=Cf*(2*log(1/eps1)- 3.d0/2.d0) 
      end if

      T2=exp(beta/2.d0/Ceps*(-log(eta)))*T1
      N0=T2/DT
      N00=N0
   
      Ntry=0
      Im  =0
3     continue
      Ntry=Ntry+1
      T2L=DT* N0
      T2U=DT*(N0+1)
      Q21L=exp(T2L)*QCDLd**2
      Q21U=exp(T2U)*QCDLd**2
      if(Q21L.lt.Q21o) then
       if(Im.eq.1) then
        Q21=Q21o
        goto 5
       else
        N0=N0+1
        goto 3
       end if
      end if
      sdkvU=sudakov(ID,Q21L,Q21o)
      sdkvL=sudakov(ID,Q21U,Q21o)
      if     (eta.lt.sdkvL) then
       N0=N0+1
       Im  =0
       goto 3
      else if(eta.gt.sdkvU) then
       N0=N0-1
       Im  =1
       goto 3
      end if
4     continue

c     Q21=Q21U-(Q21U-Q21L)*(eta-sdkvL)*(sdkvU-sdkvL)
c     Q21=Q21L+(Q21U-Q21L)*(-log(eta)-T2L)*(T2U-T2L)
      Q21=(Q21U+Q21L)/2
c     write(6,*)ID
c     write(6,*)N00,N0,Ntry
c     write(6,*)(eta-sudakov(ID,Q21,Q21o))/eta
5     continue
      Q2out=Q21

*
* Possible number of flavors at Q21
*
c     if     (Q21.gt.Q2PDF(5)) then
c      Nfq2=5
c     else if(Q21.gt.Q2PDF(4)) then
c      Nfq2=4
c     else 
c      Nfq2=3
c     end if
*
* Compornent determination
*
      eps1=aLmd2/Q21
      eps1=eps
      Y2max=1
      Y2min=eps1
      if     (ID.ne.21) then
*
* Quark branch part
*
* determination of diagnal branch or not
       fqq=Cf*(2*log(1/eps1)-17.d0/6.d0)
       fgq=Cf*                4.d0/3.d0
       if(QPRAND(Idum).lt.fqq/(fqq+fgq)) then
        Idiag=1
       else
        Idiag=0
       end if

       if     (Idiag.eq.1) then
*  Pqq compornent
        LID=ID
        NID=21
       else if(Idiag.eq.0) then
*  Pgq compornent
        LID=21
        NID=ID
       end if
      else if(ID.eq.21) then
*
* Gluon branch part
*
* determination of diagnal branch or not
       fqg=2*Nf*Tr/3.d0
       fgg=CggF*Cg*(log(1/eps1)-11.d0/12.d0)
       if(QPRAND(Idum).lt.fgg/(fgg+fqg)) then
        Idiag=1
       else
        Idiag=0
       end if

       if(Idiag.eq.0) then
*  Pqg compornent
        LID=MIN(QPRAND(Idum)*Nf+1,Nf*1.01d0)
        if(QPRAND(Idum).gt.0.5d0) LID=-LID
        NID=-LID
       else
*  Pgg compornent
        LID=21
        NID=21
       end if
      end if
      
*Double cascade
c     if(Y2 .LT. Q2out/Es2) then
c      goto 1
c     end 

*
* fill information array
* Leading particle
*
      L = NTOP+1
      PLPTN( 2,L) = -Q2out

      NLPTN(1,L) = LID
      NLPTN(2,L) = L - NAD
      NLPTN(5,L) = 0
      NLPTN(6,L) = -1
      NLPTN(8,L) = Idiag

*
* Non-Leading particle
*
      L = NTOP+2
      PLPTN( 2,L) = 0

      NLPTN(1,L) = NID
      NLPTN(2,L) = L - NAD
      NLPTN(3,L) = 0
      NLPTN(5,L) = 1
      NLPTN(6,L) = 1
      NLPTN(8,L) = Idiag

*
* Parent particle
*
      NLPTN(3,NAD) = 2
      NLPTN(4,NAD) = (L-1) - NAD
      NLPTN(5,NAD) = 1

      NTOP = NTOP + 2

      return
*
* end of "Branch" part
***********************************
* not branch
*
2     continue
      NLPTN(3,NAD) = 0
      NLPTN(4,NAD) = 0
      NLPTN(5,NAD) = 1

      return 
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPEVINI
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPlock/xpdf_lock(2),iqplock

      if( iqplock.eq.1 ) return

      NTOP=0
      do 1 l=1,1000
      do 1 i =1,10
       plptn(i,l) = 0.0d0
       nlptn(i,l) = 0
1     continue

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine runQCDPS( iproton, iparton, idir, scale, xxx, xpdf )
C  Interafce to QCDPS.
C  (input) iproton: =1 for proton, and -1 for anti-proton
C          iparton: PDG code of the final parton
C          idir: =1 for +z direction, and -1 for -z direction
C          scale: PS energy scale (GeV)
C          xxx: x value
C  (output) xpdf = x * pdf(x,Q)
C  Feb. 22, 2008, S. Odaka
      IMPLICIT REAL*8(A-H,O-Z)

      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
      common/QPlock/xpdf_lock(2),iqplock
C  Qps: minimum energy scale (GeV) of PS given in QPINIT
C  ipps(j,i): j'th pointer for the i'th beam (i = 1,2)
C       j = 1 starting point (beam particle)
C           2 final colliding parton
C           3 end point (usually a radiation)
C  xpdf_lock(i): x*pdf value previously evaluated
C                i = 1 for idir = 1, and 2 for idir = -1
C  iqplock: flag to lock the QPLIST contents
C           = 1 to lock the contents and return the previous evaluation.

      if( idir.gt.0 ) then
         ibeam= 1
      else
         ibeam= 2
      endif

      if( iqplock.eq.1 ) then
         xpdf= xpdf_lock(ibeam)
         return
      endif

      npre= NTOP
      call QCDPS(  iproton, iparton, idir, scale, xxx, pdf )

C  Finally remaining parton of this PS.
C  The last one is the final parton if no radiation.
      ipfin= NTOP -1
      if( ( NTOP -npre ).lt.4 ) ipfin= NTOP
C  Check the flavor of the final parton.
      if( NLPTN(1,ipfin).ne.iparton ) pdf= 0.d0

C  Set the pointers.
      ipps(1,ibeam)= npre +1
      ipps(2,ibeam)= ipfin
      ipps(3,ibeam)= NTOP

      xpdf= xxx*pdf
      xpdf_lock(ibeam)= xpdf

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QCDPS(Ipro,Ipar,Ib,Escl,Xin ,Weight)
C Input
C Iprot : Beam particle 
C         Ipro=+1  :      protpn
C              -1  : anti-protpn
C Ipatn : Target parton
C         Ipar=(PDG particle ID)
C Ibeam : Beam diretion
C         Ib   =+1  : +Z direction
C              =-1  : -Z direction
C Escl  : Energy scale of PS evolution
C
C Input
C Weight: Weight of this event
C
C         PLPTN( 1, ):  The lightcone fraction of this branch
C         PLPTN( 2, ):  virtual mass squared
C         PLPTN( 3, ):  Pt**2
C         PLPTN( 4, ):  x-component of the four momentum
C         PLPTN( 5, ):  y-component of the four momentum
C         PLPTN( 6, ):  z-component of the four momentum
C         PLPTN( 7, ):  E-component of the four momentum
C         PLPTN( 8, ):  + component of the lightcone momentum
C         PLPTN( 9, ):  - component of the lightcone momentum
C         PLPTN(10, ):  Weight of this branch
C
C         NLPTN( 1, ): particle ID
C         NLPTN( 2, ): relative address of the parent
C         NLPTN( 3, ): number of the children
C         NLPTN( 4, ): relative address of the first child
C         NLPTN( 5, ): status of the process
C         NLPTN( 6, ): spacelike(-1) or timelike(+1)
C         NLPTN( 7, ): non-singlet: 1 , singlet: 0
C         NLPTN( 8, ): diagonal branch: 1, non-diagonal branch 0
*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      data Pmass/0.93827231d0/
*
      Pbeam=sqrt( (Ebeam+Pmass)*(Ebeam-Pmass) )
      Es2=Escl**2
      Ibeam=Ib
      Iproton = Ipro

*check particle ID
      if(Ipar.le.-6 .or. Ipar.ge.22) then
       write(6,*)' wrong particle ID ',Ipar
       stop
      else
       Iparton=Ipar
       if(Ipar.ne.21)Iparton = Iproton*Ipar
       Ifo=IDconv2(Iparton)
       if(Ifo.eq.999) then
        write(6,*)' wrong partiocle ID ',Ipar
        stop
       end if
      end if
*
      if(Iparton.eq.1 .or. Iparton.eq.2) then
       Gfrac=0.2d0
       Dfrac=0.8d0
       Nwgt= 9
      else
       if(Iparton.eq.21) then
        Gfrac=0.9d0
        Dfrac=0
        Nwgt= 10
       else
        Gfrac=0.3d0
        Dfrac=0.7d0
        Nwgt= 9
       end if
      end if

       if(QPRAND(Idum).lt.Gfrac) then
        Wgt = 1.d0/Gfrac
        ID  = 21
       else
        if(QPRAND(Idum).le.Dfrac) then
         ID = Iparton
         Wgt = 1.d0/(1-Gfrac)/Dfrac
        else
1818     Ifl  = min(QPRAND(Idum)*5+1,5.1d0)
         if(QPRAND(Idum).gt.0.5d0)Ifl=-Ifl
         ID  = IDconv1(Ifl)
         if(ID.eq.Iparton) goto 1818
         Wgt = 1.d0/(1-Gfrac)/(1-Dfrac)*Nwgt
        end if
       end if

      NTOP = NTOP + 1
      L = NTOP
      L0  = L
      PLPTN(1,L) = 1.d0
      PLPTN(2,L) = 0.d0
      PLPTN(3,L) = 0.0D0
      PLPTN(4,L) = 0.0D0
      PLPTN(5,L) = 0.0D0

      PLPTN(8,L) = (Ebeam+Pbeam)/sqrt(2.d0) 
      PLPTN(9,L) = PLPTN(2,L)/2.d0/PLPTN(8,L)
      PLPTN(7,L) = (PLPTN(8,L)+PLPTN(9,L))/sqrt(2.d0)
      PLPTN(6,L) = (PLPTN(8,L)-PLPTN(9,L))/sqrt(2.d0)
      PLPTN(2,L) = Pmass**2
      PLPTN(10,L) = 1.d0

      NLPTN(1,L) = 2212
      NLPTN(2,L) = 0
      NLPTN(3,L) = 1
      NLPTN(4,L) = 1
      NLPTN(5,L) = 3
      NLPTN(6,L) = 1

      NTOP = NTOP + 1
      L = NTOP
      L1  = L
      PLPTN(1,L) = 1.d0
      PLPTN(2,L) = 0.d0
      PLPTN(3,L) = 0.0D0
      PLPTN(4,L) = 0.0D0
      PLPTN(5,L) = 0.0D0

      PLPTN(8,L) = -999
      PLPTN(9,L) = -999
      PLPTN(7,L) = -999
      PLPTN(6,L) = -999
      Ifl  = IDconv2(ID)
      IDF=abs(Ifl)
      PLPTN(2,L) =-Q2PDF(IDF)
      PLPTN(10,L) = -999
      NLPTN(1,L) = ID
      NLPTN(2,L) =  1
      NLPTN(5,L) =  0
      NLPTN(6,L) = -1

      NAD = NTOP
2     CONTINUE
      IF( NLPTN(5,NAD) .EQ. 0) THEN
       CALL QPGEN1(NAD)
      ENDIF
      NAD = NAD + 1
      IF( NAD .LE. NTOP) GOTO 2

      X0 = Xin
      CALL QPXfix(L1,X0, WgtBr,Idiag)
      Weight=Wgt*WgtBr
* 20/OCt./2003
      IparPS=NLPTN(1,NTOP-1)
c     if(IparPS.eq.21 .and. Idiag.eq.1)Weight=Weight*2
C      if(Ipar.ne.21) then
      if(Iparton.ne.21) then
       if(IparPS.ne.21 .and. Idiag.ne.1) then
C        NLPTN(1,NTOP-1)= Ipar
        NLPTN(1,NTOP-1)= Iparton
        Weight=Weight/10.d0
        if(NLPTN(1,NTOP).eq.-IparPS) then
C         NLPTN(1,NTOP)  =-Ipar
         NLPTN(1,NTOP)  =-Iparton
        end if
C        do 1616 I=NTOP-2,1,-1
C         if(NLPTN(3,I).eq.0 .and. NLPTN(6,I).eq. 1
C     .      .and. NLPTN(1,I).eq.-IparPS            ) then
C          NLPTN(1,I)  =-Ipar
C         end if
C1616    continue
C  2007/7/24 S. Odaka
        i= NTOP -1
 1616   continue
        i= i -NLPTN(2,i)
        if( NLPTN(1,i).eq.IparPS ) then
C           NLPTN(1,i)= Ipar
           NLPTN(1,i)= Iparton
           if( NLPTN(1,i+1).eq.-IparPS ) then
C              NLPTN(1,i+1)= -Ipar
              NLPTN(1,i+1)= -Iparton
           endif
           goto 1616
        endif
       end if
      end if
* 20/OCt./2003
      if(WgtBr.eq.0) return

* skip this to determine the kinematics in PSKINI; S.O.
CCC      CALL QPKINM(L1)

      do 1717 I=L0,NTOP
*Beam direction treatment
       IF(Ibeam .eq. -1) PLPTN(6,I) = - PLPTN(6,I)
*anti-proton treatment
       IF(Iproton.eq.-1.and.NLPTN(1,I).ne.21)NLPTN(1,I)=-NLPTN(1,I)
1717  continue

      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPXfix(L0,X0, WgtTOT,Idiag)
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
*
      WgtTOT = 1
      IF(NTOP.eq.L0) then
       x1 =x0
       Wgt=1
       goto 888
      END IF
*
* Case of at least one branch
*

* check upper-bound of X
      OMeps=1
      do 1 NAD=L0+1,NTOP
       if(NLPTN(6,NAD).eq.-1) then
        eps1=aLmd2/abs(PLPTN(2,NAD))
        eps1=eps
        OMeps = OMeps*(1-eps1)
       end if
1     continue
* Event loop start
      Ibr    = 0
      Xtot   = 1
      Idiag  = 1
      do 2 NAD=L0+1,NTOP
       IF( NLPTN(5,NAD) .EQ. 2)GOTO 2
*
       NPR = NAD - NLPTN(2,NAD)
       L1  = NAD
       L2  = L1 + 1
       ID1 = NLPTN(1,NPR)
       ID2 = NLPTN(1,L1)
       Ins = NLPTN(7,NPR)
*
       Idiag  = Idiag*NLPTN(8,L1)
*
       Ibr = Ibr + 1
*
*New X and Compornent determination
*
       eps1=aLmd2/abs(PLPTN(2,L1))
       eps1=eps
       Y1max=1-eps1
       Y1min=x0/Xtot/OMeps
       if(Y1max.le.Y1min) then
C        write(6,*)' Y1max is less than Y1min ',Y1min,Y1max
        WgtTOT=0
        return
       end if
       Y2max=1-Y1min
       Y2min=eps1
       if     (ID1.ne.21) then
*
* Quark branch part
*
* determination of diagnal branch or not
        if(ID1.eq.ID2) then
*  Pqq compornent
         Y2=Y2min*(Y2max/Y2min)**QPRAND(Idum)
         Y1=1-Y2
         YL =Y1
         YN =Y2
         Ceps=Cf*(2*log(1/eps1)-17.d0/6.d0)
         Wgt=Cf*Y1*(1+Y1**2)*log(Y2max/Y2min)/Ceps 
         Wgt=Wgt/Y1
        else 
*  Pgq compornent
         Y2=Y2min+(Y2max-Y2min)*QPRAND(Idum)
         Y1=1-Y2
         YL =Y1
         YN =Y2
         Ceps=Cf*4.d0/3.d0
         Wgt=Cf*(1+Y2**2)*(Y2max-Y2min)/Ceps 
         Wgt=Wgt/Y1
        end if
       else if(ID1.eq.21) then
*
* Gluon branch part
*
* determination of diagnal branch or not
        if(ID1.ne.ID2) then
*  Pqg compornent
         Y2=Y2min+(Y2max-Y2min)*QPRAND(Idum)
         Y1=1-Y2
         Ceps=Tr/3.d0 
         Wgt=Tr*Y1*(Y1**2+Y2**2)*(Y2max-Y2min)/Ceps
         Wgt=Wgt/Y1
         YL=Y1
         YN=Y2
        else
*  Pgg compornent
         Y2=Y2min*(Y2max/Y2min)**QPRAND(Idum)
         Y1=1-Y2
         Ceps=2*Cg*(log(1/eps1)-11.d0/12.d0)
         Wgt=2*Cg*(Y1**2+Y2**2+(Y1*Y2)**2)*log(Y2max/Y2min)/Ceps
         Wgt=Wgt/Y1
         YL=Y1
         YN=Y2
        end if
       end if
*
       PLPTN( 1,L1) = YL
       PLPTN(10,L1) = Wgt

       PLPTN( 1,L2) = YN
       PLPTN(10,L2) = Wgt
*
       WgtTOT=WgtTOT*Wgt
       Xtot  =Xtot  *YL
       OMeps=OMeps/(1-eps1)
*
       NLPTN(5,L1) = 2
       NLPTN(5,L2) = 2
2     continue
*
* set initial X
*
      X1 =min(1.d0-1.d-10,X0/Xtot)
      Wgt=1.d0/Xtot
888   continue 
      ID =NLPTN(1,L0)
      Ifl = IDconv2(ID)
      IDF=abs(Ifl)
      Qpdf=sqrt(Q2PDF(IDF))
      if(X1.gt.1) then
        write(6,*)X1
        stop
      end if
C--- Use a GR@PPA utility to set the initial PDF.
C      fPDF=  Ctq5Pdf(Ifl,X1,Qpdf)
      call grcpdf1( ID, X1, Qpdf, fPDF )
      fPDF= fPDF/X1
C---
      Wgt=Wgt*fPDF
      PLPTN( 1,L0) = X1
      PRT0=L0-NLPTN(2,L0)
      PLPTN( 8,L0) = X1*PLPTN(8,PRT0)
      PLPTN(10,L0) = Wgt
      NLPTN( 5,L0) = 2
      WgtTOT=WgtTOT*Wgt
 
      if(L0.eq.NTOP) return
      do 3 NAD=L0+1,NTOP
       NPR = NAD - NLPTN(2,NAD)
       X   = PLPTN(1,NAD)
       PLPTN( 8,NAD) = X*PLPTN(8,NPR)
3     continue

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE QPDUMP

      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      DIMENSION PSUM(3)

      WRITE(6,101)NTOP
 101  FORMAT(' LIST OF OUTPUT, # OF LISTS=',I5)
      WRITE(6,102)
 102  FORMAT('     ID',6X,' X ',6X,'Q2',8X,'PT2',8X,'PX',8X,'PY',8X,
     &  'PZ',8X,'E',8X,'Wgt',7X,'NLPTN(I,*)=')


      DO 110 I =1,NTOP
       WRITE(6,103)I,NLPTN(1,I),(PLPTN(K,I),K=1,7),PLPTN(10,I)
     .,(NLPTN(K,I),K=2,8)
 110  CONTINUE

 103  FORMAT(' ',I3,I4,2X,8(G9.2,1X),2X,7I3)
 
      RETURN
      END
*----+---1---------2---------3---------4---------5---------6---------7-+
C------------------------------------------------------------------
C         RANDOM
C------------------------------------------------------------------
C         RANDOM NUMBER  ( 0- 1.0 , UNIFORM )
C------------------------------------------------------------------
C
      FUNCTION QPRAND( IDUM  )

      IMPLICIT REAL*8(A-H,O-Z)
 
      COMMON /QPRND / NSEED
      NSEED = NSEED *48828125
      IF(NSEED.LT.0) NSEED=(NSEED+2147483647) + 1
      QPRAND=NSEED*0.4656613D-9
      RETURN
      END
