      subroutine QPsetLd( vlambda )
C  Set the QCD Lambda value to be used in QCDPS.
C  (input) vlambda: Lmbda value in GeV.
C  If "vlambda" > 0, this value is forced to be used.
C  If "vlambda" < 0, the internal value of chosen PDF for 5 flavors is 
C                    used if available.
C  This routine has to be called "after" calling the QCDPS initialization 
C  routine QPINIT.
C  If calling with vlambda < 0, this routine has to be called "after" 
C  completing the initialization of PDF.
C  Note that in QCDPS everything is done with the leading-order evaluation of 
C  alpha_s for 5 quark flavors.
C  The default value set in QPINIT is used if the chosen PDF is incompatible.
C  Mar 6, 2008 S.Odaka
      implicit real*8 (a-h,o-z)
      include 'qpincl.h'
#include "define.h"
#if PDFLIB == 1
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#elif PDFLIB == 2
      COMMON/W50511/ NPTYPE,NGROUP,NSET,MODE,NFL,LO,TMAS
      COMMON/W50512/ QCDL4,QCDL5
#elif PDFLIB == 4
      common/QCDtable/ Alambda, Nfl, Iorder
#endif

      print *,'QPsetLd: set the QCD Lambda value for QCDPS.'

      if( vlambda.gt.0.d0 ) then
         QCDLd= vlambda
         print '(5X,''Forced to be '',F7.5,'' GeV.'')',QCDLd

      else
         print *,'    Try to import the PDF value.'
#if PDFLIB == 1
C  PYTHIA built-in
         IORDPY= MSTP(2)
         NFPY= NFIN( MSTP(51) )
         ALAMPY= ALAMIN( MSTP(51) )
         if( IORDPY.eq.1 .and. NFPY.eq.Nf ) then
            print '(5X,F7.5,'' GeV for the chosen PYTHIA built-in PDF'',
     >           '' will be used.'')',ALAMPY
            QCDLd= ALAMPY
         else
            if( IORDPY.ne.1 ) then
               print '(5X,''Alpha_s order is incompatible.'')'
               print '(9X,''PYTHIA MSTP(2) ='',I3)',IORDPY
            endif
            if( NFPY.ne.Nf ) then
               print '(5X,''Number of flavors is incompatible.'')'
               print '(9X,''(QCDPS)'',I3,'' (PYTHIA)'',I3)',Nf,NFPY
            endif
            QCDLd= 0.192d0
            print '(5X,F7.5,'' GeV for CTEQ5L (PYTHIA default) '',
     >           ''will be used (the value for NF = 5).'')',QCDLd
         endif

#elif PDFLIB == 2
C  PDFLIB
         if( LO.eq.1 .and. iabs( NFL ).eq.Nf ) then
            print '(5X,''PDFLIB internal value (QCDL5 ='',F7.5,
     >           '' GeV) will be used.'')',QCDL5
            QCDLd= QCDL5
         else
            if( LO.ne.1 ) then
               print '(5X,''Alpha_s order is incompatible.'')'
               print '(9X,''PDFLIB LO ='',I3)',LO
            endif
            if( iabs( NFL ).ne.Nf ) then
               print '(5X,''Number of flavors is incompatible.'')'
               print '(9X,''(QCDPS)'',I3,'' (PDFLIB)'',I3)',Nf,NFL
            endif
            print '(5X,''QCDPS default value ('',F7.5,
     >           '' GeV) will be used.'')',QCDLd
         endif

#elif PDFLIB == 3
C  LHAPDF
         call GetOrderAs(iorder)
         if( iorder.eq.0 ) then
            call getnset(iset)
            call getnmem(iset,imem)
            call GetLam5(imem,alam5)
            print '(5X,''LHAPDF internal value ('',F7.5,
     >           '' GeV by "GetLam5") will be used.'')',alam5
            QCDLd= alam5
         else
            print '(5X,''Alpha_s order is incompatible.'')'
            print '(9X,''LHAPDF order ='',I3,'' by "GetOrderAs"'')',
     >           iorder
            print '(5X,''QCDPS default value ('',F7.5,
     >           '' GeV) will be used.'')',QCDLd
         endif

#elif PDFLIB == 4
C  CTEQ6
         if( Iorder.eq.1 .and. Nfl.eq.Nf ) then
            print '(5X,''CTEQ6 internal value (Alambda ='',F7.5,
     >           '' GeV) will be used.'')',Alambda
            QCDLd= Alambda
         else
            if( Iorder.ne.1 ) then
               print '(5X,''Alpha_s order is incompatible.'')'
               print '(9X,''CTEQ6 Iorder ='',I3)',Iorder
            endif
            if( Nfl.ne.Nf ) then
               print '(5X,''Number of flavors is incompatible.'')'
               print '(9X,''(QCDPS)'',I3,'' (CTEQ6)'',I3)',Nf,Nfl
            endif
            print '(5X,''QCDPS default value ('',F7.5,
     >           '' GeV) will be used.'')',QCDLd
         endif

#endif
      endif
      print *,''

      return
      end
