C Tue May 31 11:44:21 CDT 2005, Soushi Tsuno
C
C This file contains a bunch of functions to output the kinematical 
C configurations in the process.
C
C (F) GRCKFCD     Output KF code.
C (F) GRCIVMS     Output invariant mass of i and j particles.
C (F) GRCDPHI     Output Delta Phi of i and j particles.
C (F) GRCDRCN     Output Delta R of i and j particles.
C (S) GRCW2LB     Rotation and boost with respect to the input momenta.
C (S) GSKPDCY     Skip unstable particles in the event history.
C (F) GRCCEWK     Calculate partial width of W or Z.
C (F) WWIDTH      Calculate total width of W.
C (F) ZWIDTH      Calculate total width of Z.
C (F) TWIDTH      Calculate total width of top.
C (F) GRCALPS     Calculate alpha strong.
C (S) GRCSENE     Set energy scale and coupling constant.
C (S) GRCEXKN     Exchange particle IDs.
C (S) GRCMCDBSRT  Initialize to make the output unweighted file.
C (S) GRCMCDBEVT  Write unweighted event in the file.
C
C---------------------
C Return KF code (I1)
C---------------------
      INTEGER FUNCTION GRCKFCD(I1)
      IMPLICIT NONE
      INCLUDE 'inclk.h'
      INTEGER I1
      GRCKFCD = KFCODE(I1)
      RETURN
      END
C--------------------------
C Return Delta Phi (I1,I2)
C--------------------------
      DOUBLE PRECISION FUNCTION GRCDPHI(I1,I2)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INTEGER I1,I2
      IF (ABS(PLGRC(2,I1)-PLGRC(2,I2)).GT.ACOS(-1.0)) THEN
         GRCDPHI = 2.0*ACOS(-1.0) - ABS(PLGRC(2,I1)-PLGRC(2,I2))
      ELSE
         GRCDPHI = ABS(PLGRC(2,I1)-PLGRC(2,I2))
      ENDIF
      RETURN
      END
C------------------------
C Return Delta R (I1,I2)
C------------------------
      DOUBLE PRECISION FUNCTION GRCDRCN(I1,I2)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INTEGER I1,I2
      DOUBLE PRECISION GRCDPHI
      EXTERNAL GRCDPHI
      GRCDRCN = SQRT(GRCDPHI(I1,I2)**2+(PLGRC(3,I1)-PLGRC(3,I2))**2)
      RETURN
      END
C-------------------------------
C Return Invariant Mass (I1,I2)
C-------------------------------
      DOUBLE PRECISION FUNCTION GRCIVMS(I1,I2)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INTEGER I1,I2
      GRCIVMS = SQRT((PGRC(4,I1)+PGRC(4,I2))**2
     $     -(PGRC(1,I1)+PGRC(1,I2))**2
     $     -(PGRC(2,I1)+PGRC(2,I2))**2
     $     -(PGRC(3,I1)+PGRC(3,I2))**2)
      RETURN
      END
C-------------------------------------------------------
C Rotation and boost with respect to the input momenta.
C-------------------------------------------------------
      SUBROUTINE GRCW2LB(DIR,P4IN,P4OUT)
      IMPLICIT NONE
      INTEGER I,J,K
      DOUBLE PRECISION DEN,COSA,SINA,COSB,SINB,PCM2,FORM,OGAM,PROD
      DOUBLE PRECISION DIR(4),P4IN(4),P4OUT(4),PTMP(4),P4ON(4)
      DOUBLE PRECISION ROT1(3,3),ROT2(3,3),ROT3(3,3)
C...Initialization.
      DO I = 1,3
         DO J = 1,3
            ROT1(I,J) = 0.0D0
            ROT2(I,J) = 0.0D0
            ROT3(I,J) = 0.0D0
         ENDDO
      ENDDO
      DEN = SQRT(DIR(1)*DIR(1)+DIR(2)*DIR(2))
      IF (DEN.LE.1.0D-10) THEN
         ROT1(1,1) = SIGN(1.0D0,DIR(3))
         ROT1(2,2) = SIGN(1.0D0,DIR(3))
         ROT1(3,3) = SIGN(1.0D0,DIR(3))
         GO TO 71
      ENDIF
C...Rotation in phi.
      COSA = DIR(1)/DEN
      SINA = DIR(2)/DEN
      ROT2(1,1) = COSA
      ROT2(1,2) = SINA
      ROT2(2,1) = -SINA
      ROT2(2,2) = COSA
      ROT2(3,3) = 1.0D0
      PTMP(1) = DIR(1)*ROT2(1,1)+DIR(2)*ROT2(1,2)+DIR(3)*ROT2(1,3)
      PTMP(2) = DIR(1)*ROT2(2,1)+DIR(2)*ROT2(2,2)+DIR(3)*ROT2(2,3)
      PTMP(3) = DIR(1)*ROT2(3,1)+DIR(2)*ROT2(3,2)+DIR(3)*ROT2(3,3)
      PTMP(4) = DIR(4)
C...Rotation in theta.
      COSB = PTMP(3)/SQRT(PTMP(1)*PTMP(1)+PTMP(3)*PTMP(3))
      SINB = PTMP(1)/SQRT(PTMP(1)*PTMP(1)+PTMP(3)*PTMP(3))
      ROT3(1,1) = COSB
      ROT3(1,3) = -SINB
      ROT3(2,2) = SIGN(1.0D0,PTMP(3))
      ROT3(3,1) = SINB
      ROT3(3,3) = COSB
C...Reversed ratation matrix.
      DO I = 1,3
         DO J = 1,3
            DO K = 1,3
               ROT1(J,I) = ROT1(J,I)+ROT3(I,K)*ROT2(K,J)
            ENDDO
         ENDDO
      ENDDO
 71   CONTINUE
C...Ratation of input p with rotation matrix.
      P4ON(1) = P4IN(1)*ROT1(1,1)+P4IN(2)*ROT1(1,2)+P4IN(3)*ROT1(1,3)
      P4ON(2) = P4IN(1)*ROT1(2,1)+P4IN(2)*ROT1(2,2)+P4IN(3)*ROT1(2,3)
      P4ON(3) = P4IN(1)*ROT1(3,1)+P4IN(2)*ROT1(3,2)+P4IN(3)*ROT1(3,3)
      P4ON(4) = P4IN(4)
C...Backward boost to laboratory frame.
      PCM2 = DIR(1)*DIR(1)+DIR(2)*DIR(2)+DIR(3)*DIR(3)
      OGAM = 1.0D0/SQRT(DIR(4)*DIR(4)-PCM2)
      FORM = P4ON(1)*DIR(1)+P4ON(2)*DIR(2)+P4ON(3)*DIR(3)
      PROD = (DIR(4)*OGAM-1.0D0)*FORM/PCM2+OGAM*P4ON(4)
      P4OUT(4) = OGAM*(P4ON(4)*DIR(4)+FORM)
      DO I = 1,3
         P4OUT(I) = P4ON(I)+PROD*DIR(I)
      ENDDO
      RETURN
      END
C-----------------------------------------------
C Skip unstable particles in the event history.
C-----------------------------------------------
      SUBROUTINE GSKPDCY(ID)
      IMPLICIT NONE
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)
      SAVE /HEPEUP/
      INTEGER ID
      INTEGER I,J,INP,IDCNT(500),IDCY,IMOM1,IMOM2
      INP = 0
      DO 200 I = 1,NUP
         IF (IDUP(I).EQ.ID) THEN
            IDCY = I
            IMOM1 = MOTHUP(1,I)
            IMOM2 = MOTHUP(2,I)
            GO TO 200
         ENDIF
         INP = INP + 1
         IDCNT(INP) = I
 200  CONTINUE
      NUP = INP
      DO I = 1,NUP
         IDUP(I) = IDUP(IDCNT(I))
         ISTUP(I) = ISTUP(IDCNT(I))
         IF (MOTHUP(1,IDCNT(I)).EQ.IDCY) THEN
            MOTHUP(1,I) = IMOM1
            MOTHUP(2,I) = IMOM2
         ELSE
            IF (MOTHUP(1,IDCNT(I)).LT.IDCY) THEN
               MOTHUP(1,I) = MOTHUP(1,IDCNT(I))
               MOTHUP(2,I) = MOTHUP(2,IDCNT(I))
            ELSE
               MOTHUP(1,I) = MOTHUP(1,IDCNT(I))-1
               MOTHUP(2,I) = MAX(MOTHUP(2,IDCNT(I))-1,0)
            ENDIF
         ENDIF
         PUP(1,I) = PUP(1,IDCNT(I))
         PUP(2,I) = PUP(2,IDCNT(I))
         PUP(3,I) = PUP(3,IDCNT(I))
         PUP(4,I) = PUP(4,IDCNT(I))
         PUP(5,I) = PUP(5,IDCNT(I))
         ICOLUP(1,I) = ICOLUP(1,IDCNT(I))
         ICOLUP(2,I) = ICOLUP(2,IDCNT(I))
      ENDDO
      RETURN
      END
C------------------------------------------
C Return the factor GX for ELWK couplings.
C     IMODE = 1 : W decay.
C     IMODE = 2 : Z decay.
C------------------------------------------
      DOUBLE PRECISION FUNCTION GRCCEWK(IMODE,IFLV)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER IMODE,IFLV
      DOUBLE PRECISION SINW,GFRM,GX,CA,CV
C...Partial width of W -> e nu
      IF (IGAUGE.EQ.1) THEN
C...gauge invariant sets : (gf, mw, mz)
         SINW = 1 - (AMW/AMZ)**2
         GFRM = GF
      ELSE IF (IGAUGE.EQ.2) THEN
C...gauge invariant sets : (alpha, mw, mz)
         SINW = 1 - (AMW/AMZ)**2
         GFRM = PI*ALPHA/SQRT(2.0D0)/AMW**2/SINW
      ELSE IF (IGAUGE.EQ.3) THEN
C...gauge invariant sets : (alpha0, gf, mw)
         SINW = PI*ALPHA0/SQRT(2.0D0)/GF/AMW**2
         GFRM = GF
      ELSE IF (IGAUGE.EQ.4) THEN
C...gauge invariant sets : (sin2W, gf, mw)
         GFRM = GF
         SINW = STH2
      ENDIF
      IF (IMODE.EQ.1) THEN
         CV = 1.0D0
         CA = 1.0D0
         GX = 8.0D0*GFRM/SQRT(2.0D0)/2.0D0*AMW**3
      ELSE IF (IMODE.EQ.2) THEN
         IF (IABS(IFLV).LT.7) THEN
            IF (MOD(IABS(IFLV),2).EQ.0) THEN
               CV = 0.5D0-2.0D0*SINW*2.0D0/3.0D0
               CA = 0.5D0
            ELSE
               CV = -0.5D0+2.0D0*SINW*1.0D0/3.0D0
               CA = -0.5D0
            ENDIF
         ELSE
            IF (MOD(IABS(IFLV),2).EQ.0) THEN
               CV = 0.5D0
               CA = 0.5D0
            ELSE
               CV = -0.5D0+2.0D0*SINW
               CA = -0.5D0
            ENDIF
         ENDIF
         GX = 8.0D0*GFRM/SQRT(2.0D0)*AMZ**3
      ENDIF
      GRCCEWK = GX/48.0D0/PI*(CV**2+CA**2)
      RETURN
      END
C----------------------------------------
C Calculate total width of W boson.
C IMODE = 1 : LO calculation.
C         2 : with radiative correction.
C----------------------------------------
      DOUBLE PRECISION FUNCTION WWIDTH(IMODE)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER I,J,IMODE
      DOUBLE PRECISION E1,P1,G0
      DOUBLE PRECISION GRCCEWK,GRCALPS,COL,CMIX
      EXTERNAL GRCCEWK,GRCALPS
      WWIDTH = 0.0D0
      IF (IMODE.LT.1.OR.IMODE.GT.2) THEN
         WRITE(*,*) 'GRC::Wrong IMODE in WWIDTH, ',IMODE,'.'
         STOP
      ENDIF
      DO I = 1,2
         DO J = 1,3
            E1 = (AMW**2-AMDQ(J)**2+AMUQ(I)**2)/2.0D0/AMW
            P1 = SQRT(E1**2-AMDQ(J)**2)
            G0 = 1.0D0/3.0D0*(2.0D0*AMW**2-AMUQ(I)**2-AMDQ(J)**2
     $           -(AMDQ(J)**2-AMUQ(I)**2)**2/AMW**2)
            IF (IMODE.EQ.1) COL = 3.0D0
            IF (IMODE.EQ.2) COL = 3.0D0*(1.0D0+GRCALPS(AMW)/PI)
            CMIX = GRCCKM(I,J)
            WWIDTH = COL*CMIX*GRCCEWK(1,I)/AMW**3*P1*3.0D0*G0+WWIDTH
         ENDDO
      ENDDO
      DO I = 1,3
         E1 = (AMW**2-AMLP(I)**2+AMNU(I)**2)/2.0D0/AMW
         P1 = SQRT(E1**2-AMLP(I)**2)
         G0 = 1.0D0/3.0D0*(2.0D0*AMW**2-AMNU(I)**2-AMLP(I)**2
     $        -(AMLP(I)**2-AMNU(I)**2)**2/AMW**2)
         WWIDTH = GRCCEWK(1,2*I+9)/AMW**3*P1*3.0D0*G0+WWIDTH
      ENDDO
      RETURN
      END
C----------------------------------------
C Calculate total width of Z boson.
C IMODE = 1 : LO calculation.
C         2 : with radiative correction.
C----------------------------------------
      DOUBLE PRECISION FUNCTION ZWIDTH(IMODE)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER I,IMODE
      DOUBLE PRECISION E1,P1,G0
      DOUBLE PRECISION GRCCEWK,GRCALPS,COL
      EXTERNAL GRCCEWK,GRCALPS
      ZWIDTH = 0.0D0
      IF (IMODE.LT.1.OR.IMODE.GT.2) THEN
         WRITE(*,*) 'GRC::Wrong IMODE in ZWIDTH, ',IMODE,'.'
         STOP
      ENDIF
      DO I = 1,5,2
         E1 = AMZ/2.0D0
         P1 = SQRT(E1**2-AMDQ(I)**2)
         G0 = 1.0D0/3.0D0*(2.0D0*AMZ**2-2.0D0*AMDQ(I)**2
     $        +6.0D0*AMDQ(I)**2)
         IF (IMODE.EQ.1) COL = 3.0D0
         IF (IMODE.EQ.2) COL = 3.0D0*(1.0D0+GRCALPS(AMZ)/PI)
         ZWIDTH = COL*GRCCEWK(2,I)/AMZ**3*P1*3.0D0*G0+ZWIDTH
      ENDDO
      DO I = 2,4,2
         E1 = AMZ/2.0D0
         P1 = SQRT(E1**2-AMUQ(I)**2)
         G0 = 1.0D0/3.0D0*(2.0D0*AMZ**2-2.0D0*AMUQ(I)**2
     $        +6.0D0*AMUQ(I)**2)
         IF (IMODE.EQ.1) COL = 3.0D0
         IF (IMODE.EQ.2) COL = 3.0D0*(1.0D0+GRCALPS(AMZ)/PI)
         ZWIDTH = COL*GRCCEWK(2,I)/AMZ**3*P1*3.0D0*G0+ZWIDTH
      ENDDO
      DO I = 1,3
         E1 = AMZ/2.0D0
         P1 = SQRT(E1**2-AMLP(I)**2)
         G0 = 1.0D0/3.0D0*(2.0D0*AMZ**2-2.0D0*AMLP(I)**2
     $        +6.0D0*AMLP(I)**2)
         ZWIDTH = GRCCEWK(2,2*I+9)/AMZ**3*P1*3.0D0*G0+ZWIDTH
      ENDDO
      DO I = 1,3
         E1 = AMZ/2.0D0
         P1 = SQRT(E1**2-AMNU(I)**2)
         G0 = 1.0D0/3.0D0*(2.0D0*AMZ**2-2.0D0*AMNU(I)**2
     $        +6.0D0*AMNU(I)**2)
         ZWIDTH = GRCCEWK(2,2*I+10)/AMZ**3*P1*3.0D0*G0+ZWIDTH
      ENDDO
      RETURN
      END
C----------------------------------------
C Calculate total width of top.
C IMODE = 1 : LO calculation.
C         2 : with radiative correction.
C----------------------------------------
      DOUBLE PRECISION FUNCTION TWIDTH(IMODE)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER IMODE
      DOUBLE PRECISION XTB,XWT,XWB,WID1,WID2,WID3
      DOUBLE PRECISION EB,PHSE,SINW,COUP,COL,SAV,GRCALPS
      EXTERNAL GRCALPS
      TWIDTH = 0.0D0
      IF (IMODE.LT.1.OR.IMODE.GT.2) THEN
         WRITE(*,*) 'GRC::Wrong IMODE in TWIDTH, ',IMODE,'.'
         STOP
      ENDIF
      EB = (AMUQ(3)**2+AMDQ(3)**2-AMW**2)/2.0D0/AMUQ(3)
      PHSE = 2.0D0*SQRT(EB**2-AMDQ(3)**2)/(16.0D0*PI*AMUQ(3)**2)
      IF (IGAUGE.EQ.1) SINW = 1.0D0-(AMW/AMZ)**2            ! (gf, mw, mz)   
      IF (IGAUGE.EQ.2) SINW = 1.0D0-(AMW/AMZ)**2            ! (alpha, mw, mz)
      IF (IGAUGE.EQ.3) SINW = PI*ALPHA0/SQRT(2D0)/GF/AMW**2 ! (alpha0, gf, mw)
      IF (IGAUGE.EQ.4) SINW = STH2                          ! (sin2W, gf, mw)
      COUP = 4.0D0*PI*ALPHA/SINW/4.0D0
      XTB = 0.5D0*(AMUQ(3)**2+AMDQ(3)**2-AMW**2)
      XWT = 0.5D0*(AMUQ(3)**2-AMDQ(3)**2+AMW**2)
      XWB = 0.5D0*(AMUQ(3)**2-AMDQ(3)**2-AMW**2)
      IF (IMODE.EQ.1) COL = 3.0D0
      IF (IMODE.EQ.2) COL = 3.0D0*(1.0D0+GRCALPS(AMUQ(3))/PI)
      SAV = (1.0D0/3.0D0)*(1.0D0/2.0D0) ! color and spin average.
      WID1 =  4.0D0*(XTB+2.0D0/AMW**2*XWT*XWB)
      WID2 = 24.0D0*AMUQ(3)/AMW*XWB
      WID3 = -4.0D0*(XTB-4.0D0/AMW**2*XWT*XWB)
      TWIDTH = PHSE*SAV*COUP*COL*(F0**2*WID1+F0*F1*WID2+F1**2*WID3)
      RETURN
      END
C---------------------------------------------------------
C Calculate alpha strong for given energy scale (in GeV).
C---------------------------------------------------------
      DOUBLE PRECISION FUNCTION GRCALPS(QSCALE)
      IMPLICIT NONE
      INCLUDE 'inclk.h'
      DOUBLE PRECISION QSCALE
#include "define.h"
#if PDFLIB == 1
      DOUBLE PRECISION PYALEM,PYALPS
      EXTERNAL PYALEM,PYALPS
#elif PDFLIB == 2
      DOUBLE PRECISION ALPHAS2
      EXTERNAL ALPHAS2
#elif PDFLIB == 3
      DOUBLE PRECISION alphasPDF
      EXTERNAL alphasPDF
#elif PDFLIB == 4
      INTEGER NFL,IORDER
      DOUBLE PRECISION ALAMBDA
      COMMON/QCDTABLE/ALAMBDA,NFL,IORDER
      DOUBLE PRECISION XMU,XMD,XMS,XMC,XMB,XMT
      COMMON/MASSTBL/XMU,XMD,XMS,XMC,XMB,XMT
      DOUBLE PRECISION B6,BP6,B5,BP5,B4,BP4,B3,BP3
      DOUBLE PRECISION XLC,XLB,XLT,XLLC,XLLB,XLLT,XLQ,XLLQ
      DOUBLE PRECISION C65,C45,C35,ALF
#endif
C...Set coupling constants
#if PDFLIB == 1
C...PYTHIA Alpha_em, Alpha_s
      GRCALPS = PYALPS(QSCALE**2)
#elif PDFLIB == 2
      GRCALPS = ALPHAS2(QSCALE)
#elif PDFLIB == 3
C...Set Alpha_s
      GRCALPS = alphasPDF(QSCALE)
#elif PDFLIB == 4
C...Set Alpha_s
      B6  = (33.D0-2.D0*6.D0)/PI/12.D0
      BP6 = (153.D0-19.D0*6.D0)/PI/2.0D0/(33.D0-2.D0*6.D0)
      B5  = (33.D0-2.D0*5.D0)/PI/12.D0
      BP5 = (153.D0-19.D0*5.D0)/PI/2.0D0/(33.D0-2.D0*5.D0)
      B4  = (33.D0-2.D0*4.D0)/PI/12.D0
      BP4 = (153.D0-19.D0*4.D0)/PI/2.0D0/(33.D0-2.D0*4.D0)
      B3  = (33.D0-2.D0*3.D0)/PI/12.D0
      BP3 = (153.D0-19.D0*3.D0)/PI/2.0D0/(33.D0-2.D0*3.D0)
      XLC = 2.0D0*LOG(XMC/ALAMBDA)
      XLB = 2.0D0*LOG(XMB/ALAMBDA)
      XLT = 2.0D0*LOG(XMT/ALAMBDA*AMUQ(3)/XMT)
      XLLC = LOG(XLC)
      XLLB = LOG(XLB)
      XLLT = LOG(XLT)
      C65  = 1.0D0/(1.0D0/(B5*XLT)-XLLT*BP5/(B5*XLT)**2)
     $     - 1.0D0/(1.0D0/(B6*XLT)-XLLT*BP6/(B6*XLT)**2)
      C45  = 1.0D0/(1.0D0/(B5*XLB)-XLLB*BP5/(B5*XLB)**2)
     $     - 1.0D0/(1.0D0/(B4*XLB)-XLLB*BP4/(B4*XLB)**2)
      C35  = 1.0D0/(1.0D0/(B4*XLC)-XLLC*BP4/(B4*XLC)**2)
     $     - 1.0D0/(1.0D0/(B3*XLC)-XLLC*BP3/(B3*XLC)**2)+C45
      XLQ = 2.0D0* LOG(QSCALE/ALAMBDA)
      XLLQ = LOG(XLQ)
      IF (NFL.EQ.6) THEN
         ALF = 1.0D0/(1.0D0/(1.0D0/(B6*XLQ)-BP6/(B6*XLQ)**2*XLLQ)+C65)
         IF (IORDER.EQ.1) ALF = 1.0D0/B6/XLQ
      ELSE IF (NFL.EQ.5) THEN
         ALF = 1.0D0/(B5*XLQ)-BP5/(B5*XLQ)**2*XLLQ
         IF (IORDER.EQ.1) ALF = 1.0D0/B5/XLQ
      ELSE IF (NFL.EQ.4) THEN
         ALF = 1.0D0/(1.0D0/(1.0D0/(B4*XLQ)-BP4/(B4*XLQ)**2*XLLQ)+C45)
         IF (IORDER.EQ.1) ALF = 1.0D0/B4/XLQ
      ELSE IF (NFL.EQ.3) THEN
         ALF = 1.0D0/(1.0D0/(1.0D0/(B3*XLQ)-BP3/(B3*XLQ)**2*XLLQ)+C35)
         IF (IORDER.EQ.1) ALF = 1.0D0/B3/XLQ
      ENDIF
      GRCALPS = ALF
#endif
      RETURN
      END
C-----------------------------------------
C Set energy scale and coupling constant.
C-----------------------------------------
      SUBROUTINE GRCSENE
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER I
      DOUBLE PRECISION PT2,PM2,PT2MX
      DOUBLE PRECISION GRCALPS
      EXTERNAL GRCALPS
C...Set renormalization scale
C...Q^2 = s^
      IF (ICOUP.EQ.1) THEN
         GRCQ = GPRI(4) 
C...Q^2 = Average Mt^2
      ELSE IF (ICOUP.EQ.2) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2+PGRC(2,I)**2+PT2
            PM2 = AMASS2(I)+PM2
         ENDDO
         GRCQ = SQRT((PT2+PM2)/(NNF-2))
C...Q^2 = Total Mt^2
      ELSE IF (ICOUP.EQ.3) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2+PGRC(2,I)**2+PT2
            PM2 = AMASS2(I)+PM2
         ENDDO
         GRCQ = SQRT(PT2+PM2)
C...Q^2 = max Mt^2
      ELSE IF (ICOUP.EQ.4) THEN
         PT2MX = 0.0D0
         DO I = 3,NNF
            PT2MX = MAX(PT2MX,
     $           PGRC(1,I)**2+PGRC(2,I)**2+AMASS2(I))
         ENDDO
         GRCQ = SQRT(PT2MX)
C...Q^2 = constant
      ELSE IF (ICOUP.EQ.5) THEN
         IF (GRCQ.EQ.0.0D0) THEN
            WRITE(*,*) 'GRC::Error: Set grcq energy scale'
            STOP
         ENDIF
C...User original scale
      ELSE IF (ICOUP.EQ.6) THEN
         CALL GRCUSRSETQ
      ENDIF
 
C...Set factorization scale
C...as same as the renormalization scale
      IF (IFACT.EQ.0) THEN
         GRCFAQ = GRCQ
C...Q^2 = s^
      ELSE IF (IFACT.EQ.1) THEN
         GRCFAQ = GPRI(4)
C...Q^2 = Average Mt^2
      ELSE IF (IFACT.EQ.2) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2+PGRC(2,I)**2+PT2
            PM2 = AMASS2(I)+PM2
         ENDDO
         GRCFAQ = SQRT((PT2+PM2)/(NNF-2))
C...Q^2 = Total Mt^2
      ELSE IF (IFACT.EQ.3) THEN
         PT2 = 0.0D0
         PM2 = 0.0D0
         DO I = 3,NNF
            PT2 = PGRC(1,I)**2+PGRC(2,I)**2+PT2
            PM2 = AMASS2(I)+PM2
         ENDDO
         GRCFAQ = SQRT(PT2+PM2)
C...Q^2 = max Mt^2
      ELSE IF (IFACT.EQ.4) THEN
         PT2MX = 0.0D0
         DO I = 3,NNF
            PT2MX = MAX(PT2MX,
     $           PGRC(1,I)**2+PGRC(2,I)**2+AMASS2(I))
         ENDDO
         GRCFAQ = SQRT(PT2MX)
C...Q^2 = constant
      ELSE IF (IFACT.EQ.5) THEN
         IF (GRCFAQ.EQ.0.0D0) THEN
            WRITE(*,*) 'GRC::Error: Set grcfaq energy scale'
            STOP
         ENDIF
C...User original scale
      ELSE IF (IFACT.EQ.6) THEN
         CALL GRCUSRSETQ
      ENDIF

C...Set Alpha_em.
C      ALPHA = PYALEM(GRCQ**2)

C...Set Alpha_s
      ALPHAS = GRCALPS(GRCQ)

      RETURN
      END
C-----------------------
C Exchange particle ID.
C-----------------------
      SUBROUTINE GRCEXKN(I1,I2)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)
      SAVE /HEPEUP/
      INTEGER I,I1,I2
      DOUBLE PRECISION PPTMP(4),PUTMP(4),TMPMAS
      DO I=1,4
         PPTMP(I) = PGRC(I,I1)
         PGRC(I,I1) = PGRC(I,I2)
         PGRC(I,I2) = PPTMP(I)
         PUTMP(I) = PUP(I,I1)
         PUP(I,I1) = PUP(I,I2)
         PUP(I,I2) = PUTMP(I)
      ENDDO
      TMPMAS = AMASS1(I1)
      AMASS1(I1) = AMASS1(I2)
      AMASS1(I2) = TMPMAS
      AMASS2(I1) = AMASS1(I1)**2
      AMASS2(I2) = AMASS1(I2)**2
      TMPMAS = PUP(5,I1)
      PUP(5,I1) = PUP(5,I2)
      PUP(5,I2) = TMPMAS
      IF (PGRC(4,I1).LT.AMASS1(I1)) IREJ = 3
      IF (PGRC(4,I2).LT.AMASS1(I2)) IREJ = 3
      RETURN
      END
C----------------------------------
C Make ASCII output file for MCDB.
C----------------------------------
      SUBROUTINE GRCMCDBSRT(MCDBNAME)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
C...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/
      INTEGER IEVT
      COMMON/GRCMCDB/IEVT
      INTEGER I
      CHARACTER MCDBNAME*60
      DOUBLE PRECISION SUMXSEC
      IEVT = 0
      SUMXSEC = 0.0D0
      DO I = 1,NPRUP
         SUMXSEC = XSECUP(I) + SUMXSEC
      ENDDO
C+-------------------------------------------------+
      WRITE(*,*) 'Output file name for MCDB:',MCDBNAME
      OPEN(13,file=MCDBNAME,status='unknown',form='formatted')
      CALL GRCLOGO(13)
      CALL GRCBSRS(13)
      WRITE(13,FMT='(a24,d15.6,a5)') ' Total Cross Section : ',
     $     SUMXSEC,' pb'
      WRITE(13,*)
      WRITE(13,*) '# Event History :'
      WRITE(13,FMT='(2i5,2d15.6)') LPRUP(1),IDWTUP,XSECUP(1),XMAXUP(1)
      WRITE(13,FMT='(4i5,2i6,2d15.6)') PDFGUP(1),PDFSUP(1),PDFGUP(2),
     $     PDFSUP(2),IDBMUP(1),IDBMUP(2),EBMUP(1),EBMUP(2)
      WRITE(13,*)
C+-------------------------------------------------+
      RETURN
      END
C-------------------------------------
C Write unweighted event in the file.
C-------------------------------------
      SUBROUTINE GRCMCDBEVT
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 
      INTEGER JPROC
      COMMON /AMJPRC/JPROC
      INTEGER IEVT
      COMMON/GRCMCDB/IEVT
      INTEGER I,J
      CHARACTER*60 XFMT
C+-------------------------------------------------+
      IEVT = IEVT + 1
      WRITE(13,FMT='(i8,i5,4d15.6)') IEVT,NUP,
     $     GRCSIG(JPROC),SCALUP,AQEDUP,AQCDUP
      DO I = 1,NUP
         WRITE(13,FMT='(6i5,d15.6)') IDUP(I),ISTUP(I),MOTHUP(1,I),
     $        MOTHUP(2,I),ICOLUP(1,I),ICOLUP(2,I),SPINUP(I)
      ENDDO
      DO I = 1,NUP
         WRITE(13,FMT='(5d15.6)') (PUP(J,I),J=1,5)
      ENDDO
C+-------------------------------------------------+
      RETURN
      END
