C===================================================================
C Kinematics type : H2101
C                                1 \  (TAU,YST)
C TAU...GPRI(1) <== XRUN(3)         \
C YST...GPRI(2) <== XRUN(1)          *------ 3
C PDF...GPRI(10)<== XRUN(2)         /
C                                2 /
C===================================================================
      SUBROUTINE KVH2101(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,YST,QM,QMIN,QMAX,RMAS,RWID
      DOUBLE PRECISION THE,THEMIN,THEMAX

C...Initialization.
      AJACOB = 1.0D0

C...Valiable tau,y for PDF
      IF (ISDEP.EQ.1) THEN
         QMIN = 1.0D0
         QMAX = GRCECM**2
C...Set W mass.
         RMAS = AMW             ! given resonance mass.
         RWID = AGW             ! given resonance width.
         RNA = XRUN(3)
         THEMIN = ATAN((QMIN - RMAS**2)/(RMAS*RWID))
         THEMAX = ATAN((QMAX - RMAS**2)/(RMAS*RWID))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         QM = RMAS*RWID*TAN(THE) + RMAS**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((QM-RMAS**2)**2+(RMAS*RWID)**2)/(RMAS*RWID)
C...s-dep. Q^2.
         AJACOB = AJACOB*QM*RWID/PI/RMAS
     $        /((QM-RMAS**2)**2+(RMAS*RWID)**2)

         AMASS2(IPX(3)) = QM
         AMASS1(IPX(3)) = SQRT(QM)
         TAU = QM/GRCECM**2
      ELSE
         TAU = AMASS2(IPX(3))/GRCECM**2
      ENDIF
      GPRI(1) = TAU

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(1)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Weight for decay particles.
      AJACOB = AJACOB/GPRI(3)*GDCYSUM

C...Set Flavor.
      IF (NDIM.GT.1.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(2)
      ENDIF

      RETURN
      END

C------------------------
C Set 1-body kinematics.
C------------------------
      SUBROUTINE KSH2101(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT
      DOUBLE PRECISION E1,E2,Z1,Z2
      DOUBLE PRECISION PF4(4,10),AJACOB

C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.3) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass.
      IF (GPRI(3).LT.AMASS2(IPX(3))) GO TO 999
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.0D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.0D0/GPRI(4)
 
C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Set particle 3.
      PF4(1,IPX(3)) = 0.0D0
      PF4(2,IPX(3)) = 0.0D0
      PF4(3,IPX(3)) = 0.0D0
      PF4(4,IPX(3)) = GPRI(4)

C...Check kinematics. 
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT*2.0D0*PI
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
