C===================================================================
C Kinematics type : H2301
C                                               / 3
C TAU ...GPRI(1) <== XRUN(1)                   /
C YST ...GPRI(2) <== XRUN(2)                  / (COS3,PHI3)
C COS3...GVAL(1) <== XRUN(3)       1 ------->*<------- 2
C PHI3...GVAL(2) <== XRUN(4)       (TAU,YST)/
C Q45 ...GVAL(3) <== XRUN(5)               * Q45 (non-resonance)
C COS4...GVAL(4) <== XRUN(6)              / \
C PHI4...GVAL(5) <== XRUN(7)             /   \ 
C PDF ...GPRI(10)<== XRUN(8)           4/    5\ (COS4,PHI4)
C===================================================================
      SUBROUTINE KVH2301(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,SIN3,COS4,E3,P3
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,P4MIN,P5MIN,R45MN
      DOUBLE PRECISION Q45,Q45MN,Q45MX,COS3,COS3MN,COS3MX

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16

C...Set tau.
      E3MIN = SQRT(GRCPTCUT(IPX(3)-2)**2+AMASS2(IPX(3)))
      E4MIN = SQRT(GRCPTCUT(IPX(4)-2)**2+AMASS2(IPX(4)))
      E5MIN = SQRT(GRCPTCUT(IPX(5)-2)**2+AMASS2(IPX(5)))
      TAUMN = (E3MIN+E4MIN+E5MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Q45.
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      R45MN = MIN(GRCRCONCUT(IPX(4)-2),GRCRCONCUT(IPX(5)-2))
      Q45MN = AMASS2(IPX(4))+AMASS2(IPX(5))+2.0D0*E4MIN*E5MIN
     $     -2.0D0*P4MIN*P5MIN*COS(R45MN)
      Q45MX = (GPRI(4)-E3MIN)**2
      GQ12(2) = MAX(GQ12(2),Q45MN)                           ! Minimum Q45.
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q45MX) ! Maximum Q45.
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(5)
      CALL GRCSQ12(RNA, Q45, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(3) = Q45
      AJACOB = AJACOB*AFACTOR

C...Set Cos3.
      E3 = (GPRI(3)+AMASS2(IPX(3))-Q45)/2.0D0/GPRI(4)
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3+AMASS1(IPX(3)))*(E3-AMASS1(IPX(3))))
      IF (P3.LT.GRCPTCUT(IPX(3)-2)) GO TO 999
      COS3MN = -SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      COS3MX =  SQRT(P3**2 - GRCPTCUT(IPX(3)-2)**2)/P3
      GCOS3(2) = MAX(-1.0D0,COS3MN)              ! Minimum Cos3.
      GCOS3(INT(GCOS3(1))+2) = MIN(1.0D0,COS3MX) ! Maximum Cos3.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS3, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(1) = COS3
      AJACOB = AJACOB*AFACTOR

C...Set Phi3. 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos4, Phi4 in 45 CM frame.
      RNA = XRUN(6)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,COS4,AFACTOR,-1.0D0,1.0D0)
      GVAL(4) = COS4
      AJACOB = AJACOB*AFACTOR

      GVAL(5) = XRUN(7)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI
 
C...Set Pt.
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      GVAL(6) = P3*SIN3

C...Set Flavor.
      IF (NDIM.GT.7.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(8)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------
C Set 3-body kinematics.
C------------------------
      SUBROUTINE KSH2301(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT
      DOUBLE PRECISION E1,E2,E45,Z1,Z2
      DOUBLE PRECISION E3,P3,COS3,SIN3,PHI3
      DOUBLE PRECISION E4,P4,COS4,SIN4,PHI4,E5
      DOUBLE PRECISION PF4(4,10),P45(4),AJACOB

C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.5) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF

C...Check invariant mass.
      IF (GVAL(3).LT.(AMASS1(IPX(4))+AMASS1(IPX(5)))**2) GO TO 999
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2
 
C...Set particle 3.
      E3 = (GPRI(3)+AMASS2(IPX(3))-GVAL(3))/2.0D0/GPRI(4)
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
      COS3 = GVAL(1)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(2)
 
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3
 
C...Set particle 4 in 45 CM frame.
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(5)))/2.D0/SQRT(GVAL(3))
      IF (E4.LT.AMASS1(IPX(4))) GO TO 999
      P4 = SQRT((E4-AMASS1(IPX(4)))*(E4+AMASS1(IPX(4))))
      COS4 = GVAL(4)
      SIN4 = SQRT((1.0D0-COS4)*(1.0D0+COS4))
      PHI4 = GVAL(5)
 
      PF4(1,IPX(4)) = P4*SIN4*COS(PHI4)
      PF4(2,IPX(4)) = P4*SIN4*SIN(PHI4)
      PF4(3,IPX(4)) = P4*COS4
      PF4(4,IPX(4)) = E4
 
C...Set particle 5 in 45 CM frame.
      E5 = (GVAL(3)+AMASS2(IPX(5))-AMASS2(IPX(4)))/2.D0/SQRT(GVAL(3))
      PF4(1,IPX(5)) = -PF4(1,IPX(4))
      PF4(2,IPX(5)) = -PF4(2,IPX(4))
      PF4(3,IPX(5)) = -PF4(3,IPX(4))
      PF4(4,IPX(5)) = E5
 
C...q1
      E45 = (GPRI(3) + GVAL(3) - AMASS2(IPX(3)))/2.0D0/GPRI(4)
      IF (E45.LT.SQRT(GVAL(3))) GO TO 999
      P45(1) = -PF4(1,IPX(3))
      P45(2) = -PF4(2,IPX(3))
      P45(3) = -PF4(3,IPX(3))
      P45(4) = E45

      PQGRC(1,1) = P45(1)
      PQGRC(2,1) = P45(2)
      PQGRC(3,1) = P45(3)
      PQGRC(4,1) = P45(4)

C...Boost from 45 CM frame to Lab. frame.
      CALL GRCW2LB(P45,PF4(1,IPX(4)),PF4(1,IPX(4)))
      CALL GRCW2LB(P45,PF4(1,IPX(5)),PF4(1,IPX(5)))

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/((2*PI)*(32.D0*PI2)*(32.D0*PI2))
     $     *BETA(GVAL(3)/GPRI(3),AMASS2(IPX(3))/GPRI(3))
     $     *BETA(AMASS2(IPX(4))/GVAL(3),AMASS2(IPX(5))/GVAL(3))
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
