C===================================================================
C Kinematics type : H2404
C                                                     3
C TAU   ...GPRI(1) <== XRUN(1)               5       / (COS3,PHI3)
C YST   ...GPRI(2) <== XRUN(2)                \     / 
C COS345...GVAL(1) <== XRUN(3)                 \   *---- 4
C PHI345...GVAL(2) <== XRUN(4)                  \ / (Q34,COS34,PHI34)
C Q34   ...GVAL(3) <== XRUN(5)                   *     (resonance)
C Q345  ...GVAL(4) <== XRUN(6)                  /
C COS3  ...GVAL(5) <== XRUN(7)     (TAU,YST)   /  (Q345,COS345,PHI345)
C PHI3  ...GVAL(6) <== XRUN(8)      1 ------->*<------- 2 
C COS34 ...GVAL(7) <== XRUN(9)               /
C PHI34 ...GVAL(8) <== XRUN(10)             /
C PDF   ...GPRI(10)<== XRUN(11)          6 /
C===================================================================
      SUBROUTINE KVH2404(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,COS3,COS34,E6,P6
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,E6MIN
      DOUBLE PRECISION P3MIN,P4MIN,P5MIN,P6MIN
      DOUBLE PRECISION Q34,Q34MN,Q34MX,R34MN
      DOUBLE PRECISION Q345,Q345MN,Q345MX,E345MIN
      DOUBLE PRECISION COS345,COS345MN,COS345MX

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16
      IF (GQ22(INT(GQ22(1))+2).LT.1.0D-1) GQ22(INT(GQ22(1))+2)=1.0D16

C...Set Q34.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      R34MN = MIN(GRCRCONCUT(IPX(3)-2),GRCRCONCUT(IPX(4)-2))
      Q34MN = AMASS2(IPX(3))+AMASS2(IPX(4))+2.0D0*E3MIN*E4MIN
     $     -2.0D0*P3MIN*P4MIN*COS(R34MN)
      Q34MX = (GRCECM-E5MIN-E6MIN)**2
      GQ12(2) = MAX(GQ12(2),Q34MN)                           ! Minimum Q34
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q34MX) ! Maximum Q34
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(5)
      CALL GRCSQ12(RNA, Q34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(3) = Q34
      AJACOB = AJACOB*AFACTOR

C...Set tau.
      E345MIN = SQRT((SQRT(GVAL(3))+E5MIN)**2-P5MIN**2+P6MIN**2)
      TAUMN = (E6MIN + E345MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set Q345.
      Q345MN = (SQRT(GVAL(3))+E5MIN)**2-P5MIN**2
      Q345MX = (GPRI(4)-E6MIN)**2-P6MIN**2
      GQ22(2) = MAX(GQ22(2),Q345MN)                           ! Minimum Q345.
      GQ22(INT(GQ22(1))+2) = MIN(GQ22(INT(GQ22(1))+2),Q345MX) ! Maximum Q345.
      IF (GQ22(2).GT.GQ22(INT(GQ22(1))+2)) GO TO 999
      RNA = XRUN(6)
      CALL GRCSQ22(RNA, Q345, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(4) = Q345
      AJACOB = AJACOB*AFACTOR

C...Set Cos345.
      E6 = (GPRI(3)+AMASS2(IPX(6))-GVAL(4))/2.0D0/GPRI(4)
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999
      P6 = SQRT(E6**2 - AMASS2(IPX(6)))
      IF (P6.LT.GRCPTCUT(IPX(6)-2)) GO TO 999
      COS345MN = -SQRT(P6**2 - GRCPTCUT(IPX(6)-2)**2)/P6
      COS345MX =  SQRT(P6**2 - GRCPTCUT(IPX(6)-2)**2)/P6
      GCOS3(2) = MAX(-1.0D0,COS345MN)              ! Minimum Cos345.
      GCOS3(INT(GCOS3(1))+2) = MIN(1.0D0,COS345MX) ! Maximum Cos345.
      RNA = XRUN(3)
      CALL GRCSCOS(RNA, COS345, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(1) = COS345
      AJACOB = AJACOB*AFACTOR

C...Set Phi345. 
      GVAL(2) = XRUN(4)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos3, Phi3 in 34 CM frame.
      RNA = XRUN(7)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,COS3,AFACTOR,-1.0D0,1.0D0)
      GVAL(5) = COS3
      AJACOB = AJACOB*AFACTOR

      GVAL(6) = XRUN(8)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos34, Phi34 in 345 CM frame.
      RNA = XRUN(9)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,COS34,AFACTOR,-1.0D0,1.0D0)
      GVAL(7) = COS34
      AJACOB = AJACOB*AFACTOR

      GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(2)
      AJACOB = AJACOB*2.0D0*PI

C...Set Flavor.
      IF (NDIM.GT.10.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(11)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C-------------------------------------
C Set 4-body cascade type kinematics.
C-------------------------------------
      SUBROUTINE KSH2404(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT,Z1,Z2
      DOUBLE PRECISION E1,E2,E3,E4,E5,E6,COS345,SIN345,PHI345
      DOUBLE PRECISION P3,COS3,SIN3,PHI3,COS34,SIN34,PHI34
      DOUBLE PRECISION E34,E345,P34,P345
      DOUBLE PRECISION PF4(4,10),Q1(4),Q2(4),AJACOB
 
C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
 
C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
      PF4(4,IPX(2)) = E2

C...Set Q345.
      COS345 = GVAL(1)
      SIN345 = SQRT((1.0D0-COS345)*(1.0D0+COS345))
      PHI345 = GVAL(2)
      E345 = (GPRI(3)+GVAL(4)-AMASS2(IPX(6)))/2.0D0/GPRI(4)
      IF (E345.LT.SQRT(GVAL(4))) GO TO 999
      P345 = SQRT(E345**2-GVAL(4))

      Q2(1) = P345*SIN345*COS(PHI345)
      Q2(2) = P345*SIN345*SIN(PHI345)
      Q2(3) = P345*COS345
      Q2(4) = E345

      PQGRC(1,2) = Q2(1)
      PQGRC(2,2) = Q2(2)
      PQGRC(3,2) = Q2(3)
      PQGRC(4,2) = Q2(4)
 
C...Set particle 6 in Lab. frame.
      E6 = (GPRI(3)+AMASS2(IPX(6))-GVAL(4))/2.0D0/GPRI(4)
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999
      PF4(1,IPX(6)) = -Q2(1)
      PF4(2,IPX(6)) = -Q2(2)
      PF4(3,IPX(6)) = -Q2(3)
      PF4(4,IPX(6)) = E6

C...Set Q34.
      COS34 = GVAL(7)
      SIN34 = SQRT((1.0D0-COS34)*(1.0D0+COS34))
      PHI34 = GVAL(8)
      E34 = (GVAL(4)+GVAL(3)-AMASS2(IPX(5)))/2.D0/SQRT(GVAL(4))
      IF (E34.LT.SQRT(GVAL(3))) GO TO 999
      P34 = SQRT(E34**2 - GVAL(3))
 
      Q1(1) = P34*SIN34*COS(PHI34)
      Q1(2) = P34*SIN34*SIN(PHI34)
      Q1(3) = P34*COS34
      Q1(4) = E34

      PQGRC(1,1) = Q1(1)
      PQGRC(2,1) = Q1(2)
      PQGRC(3,1) = Q1(3)
      PQGRC(4,1) = Q1(4)

C...Set particle 5 in 345 CM frame.
      E5 = (GVAL(4)+AMASS2(IPX(5))-GVAL(3))/2.D0/SQRT(GVAL(4))
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      PF4(1,IPX(5)) = -Q1(1)
      PF4(2,IPX(5)) = -Q1(2)
      PF4(3,IPX(5)) = -Q1(3)
      PF4(4,IPX(5)) = E5
 
C...Set particle 3 in 34 CM frame.
      COS3 = GVAL(5)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(6)
      E3 = (GVAL(3)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/SQRT(GVAL(3))
      IF (E3.LT.AMASS1(IPX(3))) GO TO 999
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
 
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3
 
C...Set particle 4 in 34 CM frame.
      E4 = (GVAL(3)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.0D0/SQRT(GVAL(3))
      IF (E4.LT.AMASS1(IPX(4))) GO TO 999
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...Boost from 34 CM frame to 345 CM frame.
      CALL GRCW2LB(Q1,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(Q1,PF4(1,IPX(4)),PF4(1,IPX(4)))

C...Boost from 345 CM frame to Lab. frame.
      CALL GRCW2LB(Q2,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(Q2,PF4(1,IPX(4)),PF4(1,IPX(4)))
      CALL GRCW2LB(Q2,PF4(1,IPX(5)),PF4(1,IPX(5)))
      CALL GRCW2LB(Q2,PQGRC(1,1),PQGRC(1,1))

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/(2*PI)**2/(32*PI2)**3
     $     *BETA(GVAL(4)/GPRI(3),AMASS2(IPX(6))/GPRI(3))
     $     *BETA(GVAL(3)/GVAL(4),AMASS2(IPX(5))/GVAL(4))
     $     *BETA(AMASS2(IPX(3))/GVAL(3),AMASS2(IPX(4))/GVAL(3))
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
