C===================================================================
C Kinematics type : H2405
C
C TAU  ...GPRI(1) <== XRUN(1)                    (COS5,PHI5)
C YST  ...GPRI(2) <== XRUN(2)       1 ------>*-------- 5
C E5+E6...GVAL(1) <== XRUN(3)      (TAU,YST)  \
C E5-E6...GVAL(2) <== XRUN(4)                  \       / 3 (COS3,PHI3)
C COS5 ...GVAL(3) <== XRUN(5)                   \     /
C COS6 ...GVAL(4) <== XRUN(6)            E5-E6   *---* (E34,Q34)
C PHI5 ...GVAL(5) <== XRUN(7)                   /     \
C Q34  ...GVAL(6) <== XRUN(8)                  /       \ 4
C COS3 ...GVAL(7) <== XRUN(9)                 /
C PHI3 ...GVAL(8) <== XRUN(10)      2 ------>*-------- 6
C PDF  ...GPRI(10)<== XRUN(11)                        COS6
C===================================================================
      SUBROUTINE KVH2405(AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      DOUBLE PRECISION RNA,AJACOB,AFACTOR
      DOUBLE PRECISION TAU,TAUMN,YST,E34,P34,COS3
      DOUBLE PRECISION E5,P5,COS5,SIN5,E6,P6,COS6
      DOUBLE PRECISION SUMMIN,SUMMAX,SUBMIN,SUBMAX,XCP
      DOUBLE PRECISION COS5MIN1,COS5MIN2,COS5MIN
      DOUBLE PRECISION COS5MAX1,COS5MAX2,COS5MAX
      DOUBLE PRECISION COS6MIN1,COS6MIN2,COS6MIN3,COS6MIN4,COS6MIN
      DOUBLE PRECISION COS6MAX1,COS6MAX2,COS6MAX3,COS6MAX4,COS6MAX
      DOUBLE PRECISION E3MIN,E4MIN,E5MIN,E6MIN
      DOUBLE PRECISION P3MIN,P4MIN,P5MIN,P6MIN
      DOUBLE PRECISION Q34,Q34MN,Q34MX,R34MN

C...Initialization.
      AJACOB = 1.0D0
      IF (GTAU(INT(GTAU(1))+2).EQ.0.0D0) GTAU(INT(GTAU(1))+2) = 1.0D0
      IF (GQ12(INT(GQ12(1))+2).LT.1.0D-1) GQ12(INT(GQ12(1))+2)=1.0D16

C...Set Q34.
      P3MIN = GRCPTCUT(IPX(3)-2)
      P4MIN = GRCPTCUT(IPX(4)-2)
      P5MIN = GRCPTCUT(IPX(5)-2)
      P6MIN = GRCPTCUT(IPX(6)-2)
      E3MIN = SQRT(P3MIN**2+AMASS2(IPX(3)))
      E4MIN = SQRT(P4MIN**2+AMASS2(IPX(4)))
      E5MIN = SQRT(P5MIN**2+AMASS2(IPX(5)))
      E6MIN = SQRT(P6MIN**2+AMASS2(IPX(6)))
      R34MN = MIN(GRCRCONCUT(IPX(3)-2),GRCRCONCUT(IPX(4)-2))
      Q34MN = AMASS2(IPX(3))+AMASS2(IPX(4))+2.0D0*E3MIN*E4MIN
     $     -2.0D0*P3MIN*P4MIN*COS(R34MN)
      Q34MX = (GRCECM-E5MIN-E6MIN)**2
      GQ12(2) = MAX(GQ12(2),Q34MN)                           ! Minimum Q34
      GQ12(INT(GQ12(1))+2) = MIN(GQ12(INT(GQ12(1))+2),Q34MX) ! Maximum Q34
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
      RNA = XRUN(8)
      CALL GRCSQ12(RNA, Q34, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GVAL(6) = Q34
      AJACOB = AJACOB*AFACTOR

C...Set tau.
      TAUMN = (SQRT(Q34)+E5MIN+E6MIN)**2/GRCECM**2
      GTAU(2) = MAX(0.0D0,TAUMN,GTAU(2))                     ! Minimum tau.
      GTAU(INT(GTAU(1))+2) = MIN(1.0D0,GTAU(INT(GTAU(1))+2)) ! Maximum tau.
      RNA = XRUN(1)
      CALL GRCSTAU(RNA, TAU, AFACTOR)
      IF (IREJ.GT.0) GO TO 999
      GPRI(1) = TAU
      AJACOB = AJACOB*AFACTOR

C...Set yst.
      GYST(2) = LOG(SQRT(TAU))               ! Minimum yst.
      GYST(INT(GYST(1))+2) = -LOG(SQRT(TAU)) ! Maximum yst.
      RNA = XRUN(2)
      CALL GRCSYST(RNA, YST, AFACTOR)
      GPRI(2) = YST
      AJACOB = AJACOB*AFACTOR

C...Set x1,x2,s,w
      GPRI(5) = SQRT(TAU)*EXP(YST)
      GPRI(6) = SQRT(TAU)*EXP(-YST)
      GPRI(3) = TAU*GRCECM**2
      GPRI(4) = SQRT(TAU)*GRCECM
C...Set beta,gamma
      GPRI(7) = TANH(YST)
      GPRI(8) = COSH(YST)

C...Set E5+E6
      SUMMIN = (GPRI(3) - Q34)/2.0D0/GPRI(4)
      SUMMAX = GPRI(4) - SQRT(Q34) 
      IF (SUMMIN.GT.SUMMAX) GO TO 999
      GVAL(1) = SUMMIN + (SUMMAX - SUMMIN)*XRUN(3)
      AJACOB = AJACOB*(SUMMAX - SUMMIN)
      E34 = GPRI(4) - GVAL(1)
      IF (E34.LT.SQRT(Q34)) GO TO 999
      P34 = SQRT(E34**2 - Q34)

C...Set E5-E6.
      SUBMIN = -GVAL(1) + 2.0D0*E5MIN
      SUBMAX =  GVAL(1) - 2.0D0*E6MIN
      IF (SUBMIN.GT.SUBMAX) GO TO 999
      GVAL(2) = SUBMIN + (SUBMAX - SUBMIN)*XRUN(4)
      AJACOB = AJACOB*(SUBMAX - SUBMIN)
      E5 = (GVAL(1) - GVAL(2))/2.0D0
      E6 = (GVAL(1) + GVAL(2))/2.0D0
      IF (E5.LT.AMASS1(IPX(5))) GO TO 999
      IF (E6.LT.AMASS1(IPX(6))) GO TO 999
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
      P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
C...Check allowed phase space.
      IF (P5.LT.GRCPTCUT(IPX(5)-2)) GO TO 999
      IF (P6.LT.GRCPTCUT(IPX(6)-2)) GO TO 999
      IF (P5+P6-P34.LT.0) GO TO 999
      IF (P34-P5+P6.LT.0) GO TO 999
      IF (P34+P5-P6.LT.0) GO TO 999

C...Set Cos5.
C...Due to Pt.
      COS5MIN1 = -SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
      COS5MAX1 =  SQRT(P5**2-GRCPTCUT(IPX(5)-2)**2)/P5
C...Due to Eta.
      COS5MIN2 = -E5/P5*TANH(GRCETACUT(IPX(5)-2))
      COS5MAX2 =  E5/P5*TANH(GRCETACUT(IPX(5)-2))
 
      COS5MIN = MAX(-1.0D0,COS5MIN1,COS5MIN2)
      COS5MAX = MIN(1.0D0,COS5MAX1,COS5MAX2)
 
      RNA = XRUN(5)
      CALL GRCSBCS(INT(GSPR(1)),GEPS(1),RNA,
     $     COS5,AFACTOR,COS5MIN,COS5MAX)
      GVAL(3) = COS5
      AJACOB = AJACOB*AFACTOR
      SIN5 = SQRT((1.0D0+COS5)*(1.0D0-COS5))

C...Set Cos6.
C...Due to Pt.
      COS6MIN1 = -SQRT(P6**2-GRCPTCUT(IPX(6)-2)**2)/P6
      COS6MAX1 =  SQRT(P6**2-GRCPTCUT(IPX(6)-2)**2)/P6
C...Due to Eta.
      COS6MIN2 = -E6/P6*TANH(GRCETACUT(IPX(6)-2))
      COS6MAX2 =  E6/P6*TANH(GRCETACUT(IPX(6)-2))
C...Due to Cosine Constraint.
      COS6MIN3 = -(P34 + P5*COS5)/P6
      COS6MAX3 = -(-P34 + P5*COS5)/P6
C...Due to Phi Constraint.
      XCP = (P34**2-P5**2-P6**2)/2.0D0/P5/P6
      IF (XCP.GT.0) THEN
         COS6MIN4 = XCP*(COS5 - SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
         COS6MAX4 = XCP*(COS5 + SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
      ELSE
         COS6MIN4 = XCP*(COS5 + SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
         COS6MAX4 = XCP*(COS5 - SIN5*SQRT(1.0D0/XCP**2 - 1.0D0))
      ENDIF
 
C...Set Cos6.
      COS6MIN = MAX(-1.0D0,COS6MIN1,COS6MIN2,COS6MIN3,COS6MIN4)
      COS6MAX = MIN(1.0D0,COS6MAX1,COS6MAX2,COS6MAX3,COS6MAX4)
  
      RNA = XRUN(6)
      CALL GRCSBCS(INT(GSPR(2)),GEPS(2),RNA,
     $     COS6,AFACTOR,COS6MIN,COS6MAX)
      GVAL(4) = COS6
      AJACOB = AJACOB*AFACTOR

C...Set Phi5.
      GVAL(5) = XRUN(7)*2.0D0*PI
      AJACOB = AJACOB*2.0D0*PI

C...Set Cos34, Phi34 in 34 CM frame.
      RNA = XRUN(9)
      CALL GRCSBCS(INT(GSPR(3)),GEPS(3),RNA,
     $     COS3,AFACTOR,-1.0D0,1.0D0)
      GVAL(7) = COS3
      AJACOB = AJACOB*AFACTOR
 
      GVAL(8) = XRUN(10)*2.0D0*PI - GVAL(5)
      AJACOB = AJACOB*2.0D0*PI

C...Set Flavor.
      IF (NDIM.GT.10.AND.GPRI(10).EQ.0.0D0) THEN
         GPRI(10) = XRUN(11)
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 1
      AJACOB = 0.0D0
      RETURN
      END

C------------------------------------
C Set 4-body fusion type kinematics.
C------------------------------------
      SUBROUTINE KSH2405(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I
      DOUBLE PRECISION FACT,Z1,Z2
      DOUBLE PRECISION E1,E2,E5,E6,P1,P2,P5,P6,PT5,PT6,E34,P34,PT34
      DOUBLE PRECISION E3,E4,P3,COS5,COS6,SIN5,SIN6,PHI5,PHI6
      DOUBLE PRECISION COS34,SIN34,COS3,SIN3,PHI3
      DOUBLE PRECISION PF4(4,10),AJACOB
      DOUBLE PRECISION PQ1(4)
 
C...beta function.
      DOUBLE PRECISION BETA
      BETA(Z1,Z2)=SQRT(1.0D0-2.0D0*(Z1+Z2)+(Z1-Z2)**2)
 
      FACT = 0.38937966E9/(2.0D0*GPRI(3))

      AJACOB = 0.0D0 
      IF (NNF.NE.6) THEN
         WRITE(*,*) 'GRC::Error: wrong Nr. of partcle'
      ENDIF
 
C...Set Energy and Momentum.
      E1 = (GPRI(3)+AMASS2(IPX(1))-AMASS2(IPX(2)))/2.D0/GPRI(4)
      P1 = SQRT((E1-AMASS1(IPX(1)))*(E1+AMASS1(IPX(1))))
      E2 = (GPRI(3)+AMASS2(IPX(2))-AMASS2(IPX(1)))/2.D0/GPRI(4)
      P2 = SQRT((E2-AMASS1(IPX(2)))*(E2+AMASS1(IPX(2))))
 
C...Set particle 1.
      PF4(1,IPX(1)) = 0.0D0
      PF4(2,IPX(1)) = 0.0D0
      PF4(3,IPX(1)) = P1
      PF4(4,IPX(1)) = E1
 
C...Set particle 2.
      PF4(1,IPX(2)) = 0.0D0
      PF4(2,IPX(2)) = 0.0D0
      PF4(3,IPX(2)) = -P2
      PF4(4,IPX(2)) = E2

C...Set particle 5.
      E5 = (GVAL(1) - GVAL(2))/2.0D0
      P5 = SQRT(E5**2 - AMASS2(IPX(5)))
      COS5 = GVAL(3)
      SIN5 = SQRT((1.0D0-COS5)*(1.0D0+COS5))
      PT5 = P5*SIN5
      PHI5 = GVAL(5)
 
      PF4(1,IPX(5)) = P5*SIN5*COS(PHI5)
      PF4(2,IPX(5)) = P5*SIN5*SIN(PHI5)
      PF4(3,IPX(5)) = P5*COS5
      PF4(4,IPX(5)) = E5
 
C...Set particle 6.
      E6 = (GVAL(1) + GVAL(2))/2.0D0
      P6 = SQRT(E6**2 - AMASS2(IPX(6)))
      COS6 = GVAL(4)
      SIN6 = SQRT((1.0D0-COS6)*(1.0D0+COS6))
      PT6 = P6*SIN6
 
C...Set particle 34.
      E34 = GPRI(4) - GVAL(1)
      P34 = SQRT(E34**2 - GVAL(6))
      COS34 = (P6*COS6 + P5*COS5)/P34
      SIN34 = SQRT((1.0D0-COS34)*(1.0D0+COS34))
      PT34 = P34*SIN34
      PHI6 = PHI5 + ACOS((PT34**2-PT5**2-PT6**2)/2.0D0/PT5/PT6)
 
      PF4(1,IPX(6)) = P6*SIN6*COS(PHI6)
      PF4(2,IPX(6)) = P6*SIN6*SIN(PHI6)
      PF4(3,IPX(6)) = P6*COS6
      PF4(4,IPX(6)) = E6
 
      PQ1(1) = -PF4(1,IPX(5)) - PF4(1,IPX(6))
      PQ1(2) = -PF4(2,IPX(5)) - PF4(2,IPX(6))
      PQ1(3) = -PF4(3,IPX(5)) - PF4(3,IPX(6))
      PQ1(4) = E34

      PQGRC(1,1) = PQ1(1)
      PQGRC(2,1) = PQ1(2)
      PQGRC(3,1) = PQ1(3)
      PQGRC(4,1) = PQ1(4)
 
C...Set particle 3,4 in 34 CM frame.
      E3 = (GVAL(6)+AMASS2(IPX(3))-AMASS2(IPX(4)))/2.0D0/SQRT(GVAL(6))
      P3 = SQRT((E3-AMASS1(IPX(3)))*(E3+AMASS1(IPX(3))))
      COS3 = GVAL(7)
      SIN3 = SQRT((1.0D0-COS3)*(1.0D0+COS3))
      PHI3 = GVAL(8)
      PF4(1,IPX(3)) = P3*SIN3*COS(PHI3)
      PF4(2,IPX(3)) = P3*SIN3*SIN(PHI3)
      PF4(3,IPX(3)) = P3*COS3
      PF4(4,IPX(3)) = E3
 
      E4 = (GVAL(6)+AMASS2(IPX(4))-AMASS2(IPX(3)))/2.0D0/SQRT(GVAL(6))
      PF4(1,IPX(4)) = -PF4(1,IPX(3))
      PF4(2,IPX(4)) = -PF4(2,IPX(3))
      PF4(3,IPX(4)) = -PF4(3,IPX(3))
      PF4(4,IPX(4)) = E4

C...Boost from 34 CM frame to Lab. frame.
      CALL GRCW2LB(PQ1,PF4(1,IPX(3)),PF4(1,IPX(3)))
      CALL GRCW2LB(PQ1,PF4(1,IPX(4)),PF4(1,IPX(4)))

C...Check kinematics.
      DO I=1,NNF
         IF (PF4(4,IPX(I)).LT.AMASS1(IPX(I))) GO TO 999
      ENDDO

C...Set Jacobian.
      AJACOB = FACT/8.0D0/(2.0D0*PI)**8/SIN5/SIN6/8.0D0
     $     /ABS(SIN(ACOS((PT34**2-PT5**2-PT6**2)/2.0D0/PT5/PT6)))
     $     *BETA(AMASS2(IPX(3))/GVAL(6),AMASS2(IPX(4))/GVAL(6))
 
      RETURN
 999  CONTINUE
      IREJ = 3
      AJACOB = 0.0D0
      RETURN
      END
