C  Subroutine ps2lha
C  Fill the PS radiation information in the LHA common.
C  The original hard interaction information is moved to the end 
C  of the records.
C-----------------------------------------------------------------------
C  The final structure of the event record in HEPEUP.
C-----------------------------------------------------------------------
C       i                                       MOTHUP  ISTUP
C-----------------------------------------------------------------------
C       1  initial parton (+z) of PS             0   0     -1
C       2  initial parton (-z) of PS             0   0     -1
C........................................................................
C       3  1st PS rad from 1                     1   0      1
C     ...
C     m+2  last (m'th) PS rad from 1             1   0      1
C........................................................................
C     m+3  1st PS rad from 2                     2   0      1
C     ...
C     N+2  last (n'th) PS rad from 2 (N=m+n)     2   0      1
C........................................................................
C     N+3  W for W+jets events                   1   2      2
C     N+4  1st jet of hard int. for W+jets       1   2      1
C     ...
C N+NUP-2  last jet of hard int. for W+jets      1   2      1
C N+NUP-1  1st decay product of W for W+jets   N+3   0      1
C   N+NUP  2nd decay product of W for W+jets   N+3   0      1
C-----------------------------------------------------------------------
C  Note: N = m + n (number of PS radiations; "nrad" in the program)
C  The current version (6.4) of PYTHIA does not support virtual spacelike 
C  particles (ISTUP = -2).
C  We can leave the particle info linking the PS and the hard interaction 
C  (final virtual partons from PS and initial-state partons of the hard 
C  interaction) if they are allowed.
C-----------------------------------------------------------------------
      subroutine ps2lha
      IMPLICIT REAL*8 (A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

C  For Parton Shower
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)
C  ipps(j,i): j'th pointer for the i'th beam (i = 1,2)
C       j = 1 starting point (beam particle)
C           2 final colliding parton
C           3 end point (usually a radiation)

      real*8 bdir(3),ptemp(4)
      integer icsave(2,2)

      SCALUP= Qps

C  Count the generated PS radiation (timelike particles).
C  ipps(1,ibeam) points to a beam particle (also timelike).
      nrad= 0
      do ibeam=1,2
         do i=ipps(1,ibeam)+1,ipps(3,ibeam)
            if( NLPTN(6,i).eq.1 ) then
               nrad= nrad +1
            endif
         enddo
      enddo

      if( nrad.le.0 ) return

C  For safety, ...
      if( ( NUP +nrad ).gt.MAXNUP ) then
         print *,'!!! ps2lha: HEPEUP over flow !!!'
         print '(5X,''NUP, nrad '',2I6)', NUP,nrad
         stop
      endif

      if( ISTUP(1).ne.-1 .or. ISTUP(2).ne.-1 ) then
         print *,'!!! At least, one of the 1st two records is not ',
     >        'an initial-state particl in HEPEUP. !!!'
         stop
      endif

C  The largest color index already used.
      icmax= 0
      do i=1,NUP
         do j=1,2
            if( ICOLUP(j,i).gt.icmax ) icmax= ICOLUP(j,i)
         enddo
      enddo

C  Save the color info of the initial-state particles
      do i=1,2
         do j=1,2
            icsave(j,i)= ICOLUP(j,i)
         enddo
      enddo

C  Move the event records to the part after the PS radiation region.
C  The momenta are tentatively boosted to the cm frame.
      call bstcm( PUP(1,1), PUP(1,2),
     >     pcm, ecm1, ecm2, bdir, gamma, phi, theta )

      do i=NUP,3,-1
         call bstfw( PUP(1,i), bdir, gamma, phi, theta, ptemp )
         idest= i +nrad
         call lhaPartCopy( i, idest )
         do k=1,4
            PUP(k,idest)= ptemp(k)
         enddo
      enddo

C  Boost the momenta to the frame defined by the colliding partons 
C  after the PS.
C  The pointers to parent particles are also fixed here.
      call bstcm( PLPTN(4,ipps(2,1)), PLPTN(4,ipps(2,2)),
     >     pcm, ecm1, ecm2, bdir, gamma, phi, theta )
      do i=nrad+3,nrad+NUP
         if( MOTHUP(1,i).gt.2 ) MOTHUP(1,i)= MOTHUP(1,i) +nrad
         if( MOTHUP(2,i).gt.2 ) MOTHUP(2,i)= MOTHUP(2,i) +nrad
         do k=1,4
            ptemp(k)= PUP(k,i)
         enddo
         call bstbk( ptemp, bdir, gamma, phi, theta, PUP(1,i) )
      enddo

C  Fill the radiation information.
      ip= 2
      do ibeam=1,2
         do i=ipps(1,ibeam)+1,ipps(3,ibeam)
            if( NLPTN(6,i).eq.1 ) then
               ip= ip +1
               IDUP(ip)= NLPTN(1,i)
               ISTUP(ip)= 1
               MOTHUP(1,ip)= ibeam
               MOTHUP(2,ip)= 0
               ICOLUP(1,ip)= 0
               ICOLUP(2,ip)= 0
               do k=1,4
                  PUP(k,ip)= PLPTN(k+3,i)
               enddo
               PUP(5,ip)= ptnmas( iabs( IDUP(ip) ) )
               VTIMUP(ip)= 0.d0
               SPINUP(ip)= 0.d0
            endif
         enddo
      enddo

C  Fill the particle info of the initial state.
C  They are forced to be on-shell.
      ip1= ipps(1,1) +1
      ip2= ipps(1,2) +1
      call bstcm( PLPTN(4,ip1), PLPTN(4,ip2),
     >     pcm, ecm1, ecm2, bdir, gamma, phi, theta )
      amsq1= ptnmas( iabs( NLPTN(1,ip1) ) )**2
      amsq2= ptnmas( iabs( NLPTN(1,ip2) ) )**2
      s0= ( ecm1 +ecm2 )**2
      pz0= sqrt( ( ( s0 -amsq1 -amsq2 )**2/4.d0 -amsq1*amsq2 )/s0 )

      ptemp(1)= 0.d0
      ptemp(2)= 0.d0
      do i=1,2
         ip= ipps(1,i) +1
         IDUP(i)= NLPTN(1,ip)
         ISTUP(i)= -1
         MOTHUP(1,i)= 0
         MOTHUP(2,i)= 0
         ICOLUP(1,i)= 0
         ICOLUP(2,i)= 0
         am= ptnmas( iabs( IDUP(i) ) )
         if( i.eq.1 ) then
            ptemp(3)= pz0
         else
            ptemp(3)= -pz0
         endif
         ptemp(4)= sqrt( pz0**2 +am**2 )
         call bstbk( ptemp, bdir, gamma, phi, theta, PUP(1,i) )
         PUP(5,i)= am
         VTIMUP(i)= 0.d0
         SPINUP(i)= 0.d0
      enddo

C  Set the color indeces, going backwards from the hard interaction.
      do i=nrad+2,3,-1
         ibeam= MOTHUP(1,i)
         id= IDUP(i)
         if( iabs( id ).le.6 ) then
            if( id.gt.0 ) then
               if( icsave(1,ibeam).eq.0 ) then
C    g* -> qbar* q
                  icmax= icmax +1
                  ICOLUP(1,i)= icmax
                  icsave(1,ibeam)= icmax
               else
C    q* -> g* q
                  ICOLUP(1,i)= icsave(2,ibeam)
                  icsave(2,ibeam)= 0
               endif
            else
               if( icsave(2,ibeam).eq.0 ) then
C    g* -> q* qbar
                  icmax= icmax +1
                  ICOLUP(2,i)= icmax
                  icsave(2,ibeam)= icmax
               else
C    qbar* -> g* qbar
                  ICOLUP(2,i)= icsave(1,ibeam)
                  icsave(1,ibeam)= 0
               endif
            endif
         elseif( id.eq.21 ) then
            if( icsave(1,ibeam)*icsave(2,ibeam).ne.0 ) then
C    g* -> g* g
               if( QPRAND(idum).lt.0.5d0 ) then
                  ICOLUP(1,i)= icsave(2,ibeam)
                  icmax= icmax +1
                  ICOLUP(2,i)= icmax
                  icsave(2,ibeam)= icmax
               else
                  ICOLUP(2,i)= icsave(1,ibeam)
                  icmax= icmax +1
                  ICOLUP(1,i)= icmax
                  icsave(1,ibeam)= icmax
               endif
            elseif( icsave(1,ibeam).eq.0 ) then
C    qbar* -> qbar* g
               ICOLUP(1,i)= icsave(2,ibeam)
               icmax= icmax +1
               ICOLUP(2,i)= icmax
               icsave(2,ibeam)= icmax
            elseif( icsave(2,ibeam).eq.0 ) then
C    q* -> q* g
               ICOLUP(2,i)= icsave(1,ibeam)
               icmax= icmax +1
               ICOLUP(1,i)= icmax
               icsave(1,ibeam)= icmax
            endif
         else
            print '('' !!! Unsupported PS radiation !!! '',
     >           ''id = '',I6)',IDUP(i)
            stop
         endif
         if( icsave(1,ibeam).eq.0 .and. icsave(2,ibeam).eq.0 ) then
            print '(''!!! No color flow remaining for beam '',
     >           I2,'' after setting'',I3,''th record !!!'')',
     >           ibeam,i
            stop
         endif
      enddo
C  Initial state.
      do i=1,2
         do j=1,2
            ICOLUP(j,i)= icsave(j,i)
         enddo
      enddo

      NUP= NUP +nrad
      return
      end
