      subroutine lll501( result )

C  Calculate the matrix element in the LLL (Limited Leading-Log) 
C  approximation.
C
C  Process 501 is "q g -> fbar f q"; f = a lepton or a quark.
C  The flavor combination can be distinguished by looking at the initial 
C  state "kfcode".
C  The branch mode in the collinear approx. is unique: g -> qbar q 
C  in the initial state.
C  The "kfcode" may not strictly correspond to this event since it 
C  corresponds to the particle configuration of the event to generate.

      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'

      parameter ( nextrn= 5, nextrn0= nextrn -1 )
      integer kfcode1(nextrn),kcharg1(nextrn),kf0(nextrn0)
      real*8 bdir(3),ptemp(4),p0(4,nextrn0)
      parameter ( Tr= 0.5d0 )

      result= 0.d0

C  Save the particle information of the hard interaction, because 
C  it is replaced in the calculation of non-radiative process.
C  It would be enough to save/restore "kfcode" since "amass1/2" are 
C  recalculated when the LHA common is filled.
      do i=1,nextrn
         kfcode1(i)= kfcode(i)
         kcharg1(i)= kcharg(i)
      enddo

C  Squared cm energy of this event.
C  The input event must be in the cm frame with initial-state particles 
C  aligned along the z axis.
      stot= ( pgrc(4,1) +pgrc(4,2) )**2

C  === Initial-state radiation ===

C  The branch is unique.
      irad= 5
      imoth= 2

C  Momentum transfer.
      do i=1,4
         ptemp(i)= pgrc(i,imoth) -pgrc(i,irad)
      enddo
      qsq= ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 -ptemp(4)**2

C  No approximation if the radiation is harder than the factorization scale.
      if( qsq.ge.grcfaq**2 ) return

C  Boost to the cm frame of the non-radiation subsystem.
      call bstcm( pgrc(1,1), ptemp,
     >     pcm, ecm1, ecm2, bdir, gamma, phi, theta )

C  Squared cm energy of the non-radiative subsystem.
C  The branch parameter is defined by its fraction.
      s0= ( ecm1 +ecm2 )**2
      zbr= s0/stot

C  The initial-state particles must be massless in this approximation.
      pini= sqrt( s0 )/2.d0
C  Set the four-momenta in the subsystem's cm frame.
      p0(1,1)= 0.d0
      p0(2,1)= 0.d0
      p0(3,1)= pini
      p0(4,1)= pini
      p0(1,2)= 0.d0
      p0(2,2)= 0.d0
      p0(3,2)= -pini
      p0(4,2)= pini

C  Boost the final-state particles to the CM frame.
      j= 2
      do i=3,nextrn
         if( i.ne.irad ) then
            j= j +1
            call bstfw( pgrc(1,i), bdir, gamma, phi, theta, p0(1,j) )
         endif
      enddo

C  Matrix element of the non-radiative process.
      kf0(1)= min( kfcode1(1), kfcode1(2) )
      kf0(2)= -kf0(1)
      kf0(3)= kfcode1(3)
      kf0(4)= kfcode1(4)
      call me500( kf0, p0, result0 )

C  Matrix element in the factorized collinear approximation 
      factor= 8*pi*alphas*Tr*( zbr**2 +( 1 -zbr )**2 )/zbr/qsq
      result= result +result0*factor

C  Restore the saved variables.
      do i=1,nextrn
         kfcode(i)= kfcode1(i)
         kcharg(i)= kcharg1(i)
      enddo

C  === Final-state radiation ===
C  No candidate combination.

      return
      end

      subroutine mzero501
      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'
      common/msave/psave(4,10),am1save(10),am2save(10)
C  Save the original information.
      do i=1,5
         do j=1,4
            psave(j,i)= pgrc(j,i)
         enddo
         am1save(i)= amass1(i)
         am2save(i)= amass2(i)
      enddo
C  For the process 501, the radiation (quark) may be massive.
      pgrc(4,5)= sqrt( pgrc(1,5)**2 +pgrc(2,5)**2 +pgrc(3,5)**2 )
C  Modification is only for the initial-state partons.
      ecm= pgrc(4,3) +pgrc(4,4) +pgrc(4,5)
      pini= ecm/2.d0
      pgrc(1,1)= 0.d0
      pgrc(2,1)= 0.d0
      pgrc(3,1)= pini
      pgrc(4,1)= pini
      pgrc(1,2)= 0.d0
      pgrc(2,2)= 0.d0
      pgrc(3,2)= -pini
      pgrc(4,2)= pini
      amass1(1)= 0.d0
      amass2(1)= 0.d0
      amass1(5)= 0.d0
      amass2(5)= 0.d0
      return
      end

      subroutine mrest501
      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'
      common/msave/psave(4,10),am1save(10),am2save(10)
      do i=1,5
         do j=1,4
            pgrc(j,i)= psave(j,i)
         enddo
         amass1(i)= am1save(i)
         amass2(i)= am2save(i)
      enddo
      return
      end
