C        _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C       _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C      _/_/ GR@PPA (GRace system @ PP/Anti-p)                _/_/
C     _/_/  File Name  : grcpygen.F(decides all kinematics) _/_/
C    _/_/   Date       : July/27/2005                      _/_/
C   _/_/    Author     : Soushi Tsuno (tsuno@fnal.gov)    _/_/
C  _/_/     Institute  : Tsukuba Univ. Japan             _/_/
C _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C (S) GRCINIT     GR@PPA initialization.
C (S) GRCBSIT     BASES initialization.
C (S) GRCIVAL     Initialization of integration variables.
C (S) GRCIKIN     Set kinematical parameters.
C (S) GRCSPID     Set particle ID.
C (S) GRCSTAU     Set tau.
C (S) GRCSYST     Set yst.
C (S) GRCSCOS     Set cosine(theta).
C (S) GRCSQ12     Set Q^2 of first cluster.
C (S) GRCSQ22     Set Q^2 of second cluster.
C (S) GRCSBCS     Set sub system cosine(theta).
C (S) GRCSVAL     Set kinematical variables.
C (S) GRCSKIN     Set kinematics.
C (S) GRCPYPRO    Set input parameters and process IDs.
C (S) GRCPYGEN    Perform Integration/event generation.
C (S) GRCSDCY     Set decay products.
C (S) GRCKDCY     Make kinematics for decay particles.
C (S) GRCRDCY     Output character for decay mode.
C (F) GRCCDCY     Return decay flag: none:0,W-decay:1,Z-decay:2,ZW-decay:3
C (S) GRCLOGO     Print the title.
C (S) GRCBSRS     Write Summary.
C (S) GRCSSMR     Smear to z-boosted system.
C (S) GRCSBWM     Set Breit Wigner type mass of bosons (optional).
C (S) GRCSWAP     Swap particle ID, if necessary.
C (S) GRCXSET     Set (phase space) region of the cross section.
C (S) GRCFFIL     Fill flavor of each particle.
C (S) GRCPRTNGG   Treat gluon gluon collision from PDF.
C (S) GRCPRTNQQ   Treat quark anti-quark collision from PDF.
C (S) GRCPRTNQQB  Treat quark anti-quark' collision from PDF.
C (S) GRCPRTNQG   Treat quark gluon collision from PDF.
C (S) GRCPRTNQQA  Treat up(down)-quark up(down)-quark collision from PDF.
C (S) GRCPRTNQQS  Treat up(down)-quark down(up)-quark collision from PDF.
C (S) GRCPRTNBG   Special case, treat bottom-gluon collision from PDF.
C (S) GRCQJET     Set flavors in final state partons of QCD jets proc..
C (S) GRCWJET     Set flavors in final state partons of W + jets proc..
C (S) GRCWWJT     Set flavors in final state partons of WW + jets proc..
C (S) GRCSCUT     Apply kinematical cuts.
C----------------------------------------
C Initialization of BASES.
C Default iteration = 5 (itmx1, itmx2)
C----------------------------------------
      SUBROUTINE GRCBSIT
      IMPLICIT NONE
      INCLUDE 'grchad.inc'

      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL,ITMX1,ITMX2
      DOUBLE PRECISION XL,XU,ACC1,ACC2
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      COMMON /BPARM2/ ACC1,ACC2,ITMX1,ITMX2

      INTEGER I

C...Set variables for BASES
      NWILD = NDIM
      IF (NWILD.gt.14) NWILD = 14
      DO I =  1,NDIM
         XL(I)  =   0.D0
         XU(I)  =   1.D0
         IG(I)  =   1
      ENDDO
      ITMX1 = 5
      ITMX2 = 5
 
      INBS = 0
      INKV = 0
      INKC = 0
      IN4M = 0
      INAC = 0
 
      NFPRT = 1000000

      RETURN
      END
 
C----------------------------------------
C Initialization of kinematics variable.
C Set default parameter.
C----------------------------------------
      SUBROUTINE GRCIVAL
      IMPLICIT NONE
      INCLUDE 'grchad.inc'

      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL
      DOUBLE PRECISION XL,XU 
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL

      INTEGER I,J
 
      DO I = 1,32
         XRUN(I) = 0.0D0
         GVAL(I) = 0.0D0
      ENDDO
      DO I = 1,10
         GPRI(I) = 0.0D0
      ENDDO
      DO I = 1,18
         GTAU(I) = 0.0D0
         GYST(I) = 0.0D0
         GCOS3(I) = 0.0D0
         GQ12(I) = 0.0D0
         GQ22(I) = 0.0D0
      ENDDO
      DO I = 1,5
         GSPR(I) = 0.0D0
         GEPS(I) = 0.0D0
      ENDDO
      DO I = 1,8
         GRCDSIG(I) = 0.0D0
      ENDDO
 
      IREJ = 0
      YACOB = 1.0D0

      IF (INPFL.LT.1) INPFL = 5
      IF (IFACT.LT.1) IFACT = 0
      IF (ISDEP.LT.1) ISDEP = 0
      IF (IGRCGEF.LT.1) IGRCGEF = 0
      IF (NDIM.LT.7) THEN
         KTYPE = 1
         CXTYPE = 1
      ELSE IF (NDIM.LT.10) THEN
         KTYPE = 2
         CXTYPE = 2
      ELSE IF (NDIM.LT.13) THEN
         KTYPE = 6
         CXTYPE = 5
      ENDIF
      GTAU(1)   = 1
      GTAU(14)  = 1
      GYST(1)   = 1
      GYST(14)  = 1
      GCOS3(1)  = 1
      GCOS3(14) = 1
      GQ12(1)   = 1
      GQ12(14)  = 1
      GQ22(1)   = 1
      GQ22(14)  = 1
      DO I = 1,5
         GCON(I) = 1.0D0
         GSPR(I) = 1
         GEPS(I) = 1.0D0
      ENDDO

      IF (GETACUT.LT.0.0001) GETACUT = 10.0D0

C...N jets process.
      IF (IGSUB.GT.179.AND.IGSUB.LT.190) THEN
         DO I = 1,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      IF (IGSUB.GT.300.AND.IGSUB.LT.332) THEN
         DO I = 1,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

C...W + jets process.
      IF (IGSUB.GT.100.AND.IGSUB.LT.110) THEN
         DO I = 3,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      IF (IGSUB.GT.420.AND.IGSUB.LT.452) THEN
         DO I = 3,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

C...Z + jets process.
      IF (IGSUB.GT.110.AND.IGSUB.LT.120) THEN
         DO I = 3,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      IF (IGSUB.GT.500.AND.IGSUB.LT.526) THEN
         DO I = 3,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

C...VV + jets process.
      IF (IGSUB.GT.552.AND.IGSUB.LT.580) THEN
         DO I = 5,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      IF (IGSUB.GT.119.AND.IGSUB.LT.150) THEN
         DO I = 5,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

C...bbbb process.
      IF (IGSUB.GT.159.AND.IGSUB.LT.170) THEN
         DO I = 1,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      IF (IGSUB.GT.399.AND.IGSUB.LT.408) THEN
         DO I = 1,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

C...tt process.
      IF (IGSUB.EQ.170.OR.IGSUB.EQ.171.OR.IGSUB.EQ.408.OR.IGSUB.EQ.409
     $     .OR.IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.412) THEN
         IF (GRCPTCUT(1).LT.GPTCUT) GRCPTCUT(1) = GPTCUT
         IF (GRCPTCUT(4).LT.GPTCUT) GRCPTCUT(4) = GPTCUT
         IF (GRCPTCUT(7).LT.GPTCUT) GRCPTCUT(7) = GPTCUT
         IF (GRCETACUT(1).GT.GETACUT) GRCETACUT(1) = GETACUT
         IF (GRCETACUT(4).GT.GETACUT) GRCETACUT(4) = GETACUT
         IF (GRCETACUT(7).GT.GETACUT) GRCETACUT(7) = GETACUT
         IF (GRCRCONCUT(1).LT.GRCONCUT) GRCRCONCUT(1) = GRCONCUT
         IF (GRCRCONCUT(4).LT.GRCONCUT) GRCRCONCUT(4) = GRCONCUT
         IF (GRCRCONCUT(7).LT.GRCONCUT) GRCRCONCUT(7) = GRCONCUT
      ENDIF

C...aa +jets  process.
      IF ( (IGSUB.GE.150.AND.IGSUB.LE.151)
     >     .OR.(IGSUB.GE.600.AND.IGSUB.LE.602) ) THEN
c        write(6,*)' GRCIVAL: ',IGSUB,GPTCUT,GETACUT,GRCONCUT
         DO I = 1,8
            IF (GRCPTCUT(I).LT.GPTCUT) GRCPTCUT(I) = GPTCUT
            IF (GRCETACUT(I).GT.GETACUT) GRCETACUT(I) = GETACUT
            IF (GRCRCONCUT(I).LT.GRCONCUT) GRCRCONCUT(I) = GRCONCUT
         ENDDO
      ENDIF

      RETURN
      END

C----------
C Set tau.
C----------
      SUBROUTINE GRCSTAU(RN,TAU,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR,II
      DOUBLE PRECISION TAUMIN,TAUMAX,RN,TAU,AJACOB,TAURE,GAMRE,RNA
      DOUBLE PRECISION THEMIN,THEMAX,THE
      DOUBLE PRECISION XIMIN,XIMAX,XI
 
      IF (IREJ.GT.0) RETURN
 
      KPAR = 0
      AJACOB = 1.0D0
      GTAU(8) = 0.0D0
      GTAU(INT(GTAU(1))+8) = 1.0D0
      DO II = 1,INT(GTAU(1))
         IF ((RN.GT.GTAU(II+7)).AND.(RN.LE.GTAU(II+8))) THEN
            RNA = (RN - GTAU(II+7))/(GTAU(II+8) - GTAU(II+7))
            KPAR = INT(GTAU(II+13))
            TAUMIN = GTAU(II+1)
            TAUMAX = GTAU(II+2)
            AJACOB = AJACOB/(GTAU(II+8) - GTAU(II+7))
         ENDIF
      ENDDO

      IF (KPAR.EQ.1) THEN
         TAU = TAUMIN*(TAUMAX/TAUMIN)**RNA
         AJACOB = AJACOB*LOG(TAUMAX/TAUMIN)
      ELSE IF (KPAR.EQ.2) THEN
         XIMIN = 1.0D0/TAUMAX
         XIMAX = 1.0D0/TAUMIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/TAUMAX**2
         XIMAX = 1.0D0/TAUMIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU**2*0.5D0
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/TAUMAX**3
         XIMAX = 1.0D0/TAUMIN**3
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = XI**(-1.0D0/3.0D0)
         AJACOB = AJACOB*(XIMAX - XIMIN)*TAU**3/3.0D0
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/SQRT(TAUMAX)
         XIMAX = 1.0D0/SQRT(TAUMIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         TAU = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*SQRT(TAU)*2.0D0
      ELSE IF (KPAR.EQ.6) THEN
         TAU = TAUMIN + (TAUMAX - TAUMIN)*RNA
         AJACOB = AJACOB*(TAUMAX - TAUMIN)/TAU
      ELSE IF (KPAR.EQ.7) THEN
         TAU = TAUMIN + (TAUMAX - TAUMIN)*SQRT(RNA)
         AJACOB = AJACOB*(TAUMAX - TAUMIN)**2
     $        /(2.0D0*(TAU - TAUMIN))
      ELSE IF (KPAR.EQ.8) THEN
         TAURE = AMW**2/GRCECM**2
         GAMRE = AMW*AGW/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ELSE IF (KPAR.EQ.9) THEN
         TAURE = AMZ**2/GRCECM**2
         GAMRE = AMZ*AGZ/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ELSE IF (KPAR.EQ.10) THEN
         TAURE = AMUQ(3)**2/GRCECM**2
         GAMRE = AMUQ(3)*AGUQ(3)/GRCECM**2
         THEMIN = ATAN((TAUMIN - TAURE)/GAMRE)
         THEMAX = ATAN((TAUMAX - TAURE)/GAMRE)
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         TAU = GAMRE*TAN(THE) + TAURE
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((TAU - TAURE)**2 + GAMRE**2)/(GAMRE*TAU)
      ENDIF
 
      RETURN
      END

C----------
C Set yst.
C----------
      SUBROUTINE GRCSYST(RN, YST, AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR,II
      DOUBLE PRECISION YST,AJACOB,RN,RNA
      DOUBLE PRECISION YSTMIN,YSTMAX,AUPP,ALOW,YST0
 
      IF (IREJ.GT.0) RETURN
 
      KPAR = 0
      AJACOB = 1.0D0
      GYST(8) = 0.0D0
      GYST(INT(GYST(1))+8) = 1.0D0
      DO II = 1,INT(GYST(1))
         IF (RN.GT.GYST(II+7).AND.RN.LE.GYST(II+8)) THEN
            RNA = (RN - GYST(II+7))/(GYST(II+8) - GYST(II+7))
            KPAR = INT(GYST(II+13))
            YSTMIN = GYST(II+1)
            YSTMAX = GYST(II+2)
            AJACOB = AJACOB/(GYST(II+8) - GYST(II+7))
         ENDIF
      ENDDO
 
      IF (KPAR.EQ.1) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA
         AJACOB = AJACOB*(YSTMAX - YSTMIN)
      ELSE IF (KPAR.EQ.2) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*SQRT(RNA)
         AJACOB = AJACOB*0.5*(YSTMAX - YSTMIN)**2/(YST - YSTMIN)
      ELSE IF (KPAR.EQ.3) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(YSTMAX - YSTMIN)**3/(YST - YSTMIN)**2/3.0D0
      ELSE IF (KPAR.EQ.4) THEN
         YST = YSTMIN + (YSTMAX - YSTMIN)*RNA**(3.0/2.0)
         AJACOB = AJACOB*1.5
     $        *((YSTMAX - YSTMIN)**2*(YST - YSTMIN))*(1.0/3.0)
      ELSE IF (KPAR.EQ.5) THEN
         AUPP = ATAN(EXP(YSTMAX))
         ALOW = ATAN(EXP(YSTMIN))
         YST = LOG(TAN(ALOW + (AUPP - ALOW)*RNA))
         AJACOB = AJACOB*(AUPP - ALOW)*2*COSH(YST)
      ELSE IF (KPAR.EQ.6) THEN
         RNA = ACOS(-1.0D0)/3.0D0*(2.0D0*RNA - 1.0D0)
         YST = YSTMAX*TAN(RNA)/TAN(ACOS(-1.0D0)/3.0D0)
         AJACOB = AJACOB*ACOS(-1.0D0)/3.0D0*2.0D0*YSTMAX/COS(RNA)**2
     $        /TAN(ACOS(-1.0D0)/3.0D0)
      ELSE IF (KPAR.EQ.7) THEN
         YST0 = -0.5D0*LOG(GPRI(1))
         AUPP = LOG(MAX(1D-10,EXP(YST0 - YSTMIN) - 1D0))
         ALOW = LOG(MAX(1D-10,EXP(YST0 - YSTMAX) - 1D0))
         YST = YST0 - LOG(1D0 + EXP(ALOW + RNA*(AUPP - ALOW)))
         AJACOB = AJACOB*(AUPP - ALOW)*(1 - EXP(YST - YST0))
      ELSE
         YST0 = -0.5D0*LOG(GPRI(1))
         AUPP = LOG(MAX(1D-10,EXP(YST0 + YSTMIN) - 1D0))
         ALOW = LOG(MAX(1D-10,EXP(YST0 + YSTMAX) - 1D0))
         YST = LOG(1D0 + EXP(AUPP + RNA*(ALOW - AUPP))) -YST0
         AJACOB = AJACOB*(ALOW - AUPP)*(1 - EXP(-1.0*(YST + YST0)))
      ENDIF
 
      RETURN
      END

C-----------
C Set cos3.
C-----------
      SUBROUTINE GRCSCOS(RN, COS3, AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR,II
      DOUBLE PRECISION COS3, AJACOB,RN,RNA
      DOUBLE PRECISION COS3MIN,COS3MAX,DMIN,DMAX,D,Y,TETA,D13,D23
      DOUBLE PRECISION EQ1,PQ1,T1MAX,T1MIN,T1,EPS
      DOUBLE PRECISION MAS1,ABDB,XI,TEMAX,TEMIN,YMAX,YMIN
      DOUBLE PRECISION E1,E2,E3,E4,P1,P2,P3,P4
      DOUBLE PRECISION SQM3,SQM4
 
      IF (IREJ.GT.0) RETURN
 
C...Set squared mass on cos(theta-hat)
C...2-2 process
      IF (KTYPE.EQ.2201) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = AMASS2(IPX(4))
C...3-body final state
      ELSE IF (KTYPE.EQ.2301) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = GVAL(3)
      ELSE IF (KTYPE.EQ.2302) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = GVAL(3)
      ELSE IF (KTYPE.EQ.2303) THEN
         SQM3 = AMASS2(IPX(3))
         SQM4 = GVAL(3)
C...4-body final state
      ELSE IF (KTYPE.EQ.2401) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(4)
      ELSE IF (KTYPE.EQ.2402) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(4)
      ELSE IF (KTYPE.EQ.2403) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(4)
      ELSE IF (KTYPE.EQ.2404) THEN
         SQM3 = (SQRT(GVAL(3)) + AMASS1(IPX(5)))**2
         SQM4 = AMASS2(IPX(6))
      ELSE IF (KTYPE.EQ.2504) THEN
         SQM3 = AMASS2(IPX(7))
         SQM4 = AMASS2(IPX(7))
      ELSE IF (KTYPE.EQ.2501) THEN
         SQM3 = GVAL(3)
         SQM4 = GVAL(5)
      ELSE IF (KTYPE.EQ.2502) THEN
         SQM3 = GVAL(4)
         SQM4 = GVAL(5)
      ELSE IF (KTYPE.EQ.2503) THEN
         SQM3 = GVAL(4)
         SQM4 = GVAL(5)
      ELSE IF (KTYPE.EQ.2601) THEN
         SQM3 = GVAL(5)
         SQM4 = GVAL(6)
      ELSE IF (KTYPE.EQ.2602) THEN
         SQM3 = GVAL(5)
         SQM4 = GVAL(6)
      ELSE IF (KTYPE.EQ.2701) THEN
         SQM3 = GVAL(7)
         SQM4 = AMASS2(IPX(9))
      ELSE IF (KTYPE.EQ.2702) THEN
         SQM3 = GVAL(7)
         SQM4 = AMASS2(IPX(9))
      ENDIF
 
      AJACOB = 1.0D0
      GCOS3(8) = 0.0D0
      GCOS3(INT(GCOS3(1))+8) = 1.0D0
      DO II = 1,INT(GCOS3(1))
         IF (RN.GT.GCOS3(II+7).AND.RN.LE.GCOS3(II+8)) THEN
            RNA = (RN - GCOS3(II+7))/(GCOS3(II+8) - GCOS3(II+7))
            KPAR = INT(GCOS3(II+13))
            COS3MIN = GCOS3(II+1)
            COS3MAX = GCOS3(II+2)
            AJACOB = AJACOB/(GCOS3(II+8) - GCOS3(II+7))
         ENDIF
      ENDDO
 
      E1 = GPRI(3)/2.D0/GPRI(4)
      P1 = E1
      E2 = GPRI(3)/2.D0/GPRI(4)
      P2 = E2
 
      IF (KPAR.EQ.1) THEN
         COS3 = COS3MIN + (COS3MAX - COS3MIN)*RNA
         AJACOB = AJACOB*(COS3MAX - COS3MIN)
C...1/u
      ELSE IF (KPAR.EQ.2) THEN
         E3 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         P3 = SQRT(E3**2 - SQM3)
         DMIN = 2*(E1*E3 + P1*P3*COS3MIN) - SQM3
         DMAX = 2*(E1*E3 + P1*P3*COS3MAX) - SQM3
         D = DMIN*(DMAX/DMIN)**RNA
         COS3 = E1*E3/P1/P3 - (D + SQM3)/2.D0/P1/P3
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P3
C...1/t
      ELSE IF (KPAR.EQ.3) THEN
         E4 = (GPRI(3) + SQM4 - SQM3)/2.D0/GPRI(4)
         P4 = SQRT(E4**2 - SQM4)
         DMIN = 2*(E1*E4 + P1*P4*COS3MIN) - SQM4
         DMAX = 2*(E1*E4 + P1*P4*COS3MAX) - SQM4
         D = DMIN*(DMAX/DMIN)**RNA
         COS3 = -E1*E4/P1/P4 + (D + SQM4)/2.D0/P1/P4
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P4
C...1/u + 1/t
      ELSE IF (KPAR.EQ.4) THEN
         E3 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         P3 = SQRT(E3**2 - SQM3)
         E4 = (GPRI(3) + SQM4 - SQM3)/2.D0/GPRI(4)
         P4 = SQRT(E4**2 - SQM4)
         IF(RNA.LE.0.5D0) THEN
            RNA = RNA*2.0D0
            DMIN = 2*(E1*E3 + P1*P3*COS3MIN) - SQM3
            DMAX = 2*E1*E3 - SQM3
            D = DMIN*(DMAX/DMIN)**RNA
            COS3 = E1*E3/P1/P3 - (D + SQM3)/2.D0/P1/P3
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/P1/P3
         ELSE
            RNA = 2.0D0 - RNA*2.0D0
            DMIN = 2*(E1*E4 + P1*P4*COS3MIN) - SQM4
            DMAX = 2*E1*E4 - SQM4
            D = DMIN*(DMAX/DMIN)**RNA
            COS3 = -E1*E4/P1/P4 + (D + SQM4)/2.D0/P1/P4
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/P1/P4
         ENDIF
      ELSE IF (KPAR.EQ.5) THEN
         MAS1 = GCON(1)
         XI = (E1 + P1)/MAS1
         ABDB = 1.0D0 + MAS1**2/P1/(E1 + P1)
         TEMAX = (ABDB + COS3MAX)/(ABDB - COS3MAX)
         TEMIN = (ABDB + COS3MIN)/(ABDB - COS3MIN)
         YMAX = (LOG(TEMAX)/LOG(XI) + 2)/4.D0
         YMIN = (LOG(TEMIN)/LOG(XI) + 2)/4.D0
         IF (COS3MIN.LT.-1.D0+1.D-10) YMIN = 0
         IF (COS3MAX.GT. 1.D0-1.D-10) YMAX = 1
         Y = YMIN + (YMAX - YMIN)*RNA
         TETA = XI**(4*Y - 2)
         COS3 = ABDB*(TETA - 1)/(TETA + 1)
         IF (ABS(COS3).GT.1) COS3 = SIGN(1.D0,COS3)
         D13 = 4*E1/(TETA + 1)
         D23 = TETA*D13
         AJACOB = AJACOB*LOG(XI)/2.D0/E1/P1*D13*D23*(YMAX-YMIN)
      ELSE IF (KPAR.EQ.6) THEN
         EQ1 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1**2 - SQM3)
         T1MAX = -SQM3 + 2*(E1*EQ1 + P1*PQ1)
         T1MIN = -SQM3 + 2*(E1*EQ1 - P1*PQ1)
         T1 = T1MIN + (T1MAX - T1MIN)*RNA
         COS3 = (-T1 - SQM3 + E1*EQ1*2)/2.D0/P1/PQ1
         AJACOB = AJACOB*(T1MAX-T1MIN)/(2.0D0*P1*PQ1)
      ELSE IF (KPAR.EQ.7) THEN
         EQ1 = (GPRI(3) + SQM3 - SQM4)/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1**2 - SQM3)
         T1MAX = -SQM3 + 2*(E1*EQ1 + P1*PQ1)
         T1MIN = -SQM3 + 2*(E1*EQ1 - P1*PQ1)
         T1 = T1MIN*(T1MAX/T1MIN)**RNA
         COS3 = (-T1 - SQM3 + E1*EQ1*2)/2.D0/P1/PQ1
         AJACOB = AJACOB*T1*LOG(T1MAX/T1MIN)/(2*P1*PQ1)
      ELSE IF (KPAR.EQ.8) THEN
         EPS = GCON(1)
         DMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         DMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         D = DMIN + (DMAX - DMIN)*RNA
         COS3 = COS3MAX*(1.0D0 + EPS)*TANH(D)
         AJACOB = AJACOB*(DMAX - DMIN)/COSH(D)**2*(1.0D0 + EPS)
     $        *COS3MAX
 
         COS3 = (COS3MAX-COS3MIN)/2.0D0*COS3+(COS3MAX+COS3MIN)/2.0D0
         AJACOB = AJACOB*(COS3MAX-COS3MIN)/2.0D0
      ENDIF
 
      RETURN
      END
 
C----------
C Set Q12.
C----------
      SUBROUTINE GRCSQ12(RN, Q12, AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR,II
      DOUBLE PRECISION RN,RNA,Q12,AJACOB,Q12MIN,Q12MAX
      DOUBLE PRECISION THE,THEMAX,THEMIN
      DOUBLE PRECISION XIMIN,XIMAX,XI,E3MAX,E3,E3MIN
 
      IF (IREJ.GT.0) GO TO 999 
      IF (GQ12(2).GT.GQ12(INT(GQ12(1))+2)) GO TO 999
 
      AJACOB = 1.0D0
      GQ12(8) = 0.0D0
      GQ12(INT(GQ12(1))+8) = 1.0D0
      DO II = 1,INT(GQ12(1))
         IF (RN.GT.GQ12(II+7).AND.RN.LE.GQ12(II+8)) THEN
            RNA = (RN - GQ12(II+7))/(GQ12(II+8) - GQ12(II+7))
            KPAR = INT(GQ12(II+13))
            Q12MIN = GQ12(II+1)
            Q12MAX = GQ12(II+2)
            AJACOB = AJACOB/(GQ12(II+8) - GQ12(II+7))
         ENDIF
      ENDDO

      IF (KPAR.EQ.1) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA
         AJACOB = AJACOB*(Q12MAX - Q12MIN)
      ELSE IF (KPAR.EQ.2) THEN
         Q12 = Q12MIN*(Q12MAX/Q12MIN)**RNA
         AJACOB = AJACOB*Q12*LOG(Q12MAX/Q12MIN)
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/SQRT(Q12MAX)
         XIMAX = 1.0D0/SQRT(Q12MIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*2.0D0/XI**3
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/Q12MAX
         XIMAX = 1.0D0/Q12MIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q12**2
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/Q12MAX**2
         XIMAX = 1.0D0/Q12MIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q12 = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q12**3*0.5D0
      ELSE IF (KPAR.EQ.6) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*SQRT(RNA)
         AJACOB = AJACOB*(Q12MAX - Q12MIN)**2
     $        /(2.0D0*(Q12 - Q12MIN))
      ELSE IF (KPAR.EQ.7) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(Q12MAX - Q12MIN)**3/(Q12 - Q12MIN)**2/3.0D0
      ELSE IF (KPAR.EQ.8) THEN
         Q12 = Q12MIN + (Q12MAX - Q12MIN)*RNA**(2.0D0/3.0D0)
         AJACOB = AJACOB*2.0D0/3.0D0*(Q12MAX - Q12MIN)**(3.0D0/2.0D0)
     $        /SQRT(Q12 - Q12MIN)
      ELSE IF (KPAR.EQ.9) THEN
         E3MIN = GRCPTCUT(IPX(3)-2)
         E3MAX = (GPRI(3) - Q12MIN)/2.0D0/GPRI(4)
         E3 = E3MIN*(E3MAX/E3MIN)**RNA
         AJACOB = AJACOB*E3*LOG(E3MAX/E3MIN)
         Q12 = GPRI(3) - 2.0D0*GPRI(4)*E3
         AJACOB = AJACOB*2.0D0*GPRI(4)
      ELSE IF (KPAR.EQ.10) THEN
         THEMIN = ATAN((Q12MIN - AMW**2)/(AMW*AGW))
         THEMAX = ATAN((Q12MAX - AMW**2)/(AMW*AGW))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMW*AGW*TAN(THE) + AMW**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMW**2)**2 + (AMW*AGW)**2)/(AMW*AGW)
      ELSE IF (KPAR.EQ.11) THEN
         THEMIN = ATAN((Q12MIN - AMZ**2)/(AMZ*AGZ))
         THEMAX = ATAN((Q12MAX - AMZ**2)/(AMZ*AGZ))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMZ*AGZ*TAN(THE) + AMZ**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMZ**2)**2 + (AMZ*AGZ)**2)/(AMZ*AGZ)
      ELSE IF (KPAR.EQ.12) THEN
         THEMIN = ATAN((Q12MIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THEMAX = ATAN((Q12MAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMUQ(3)**2)**2
     $        + (AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
      ELSE IF (KPAR.EQ.13) THEN
         THEMIN = ATAN((Q12MIN - AMH**2)/(AMH*AGH))
         THEMAX = ATAN((Q12MAX - AMH**2)/(AMH*AGH))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q12 = AMH*AGH*TAN(THE) + AMH**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q12 - AMH**2)**2 + (AMH*AGH)**2)/(AMH*AGH)
 
         IF (Q12.LT.1.0D0) THEN
            WRITE(*,*) Q12,THE,RNA,TAN(THE)
         ENDIF
 
      ENDIF
 
      RETURN
 999  CONTINUE
      IREJ = 1
      RETURN
      END
 
C----------
C Set Q22.
C----------
      SUBROUTINE GRCSQ22(RN, Q22, AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR,II
      DOUBLE PRECISION RN,RNA,Q22,AJACOB,Q22MIN,Q22MAX
      DOUBLE PRECISION THE,THEMAX,THEMIN
      DOUBLE PRECISION XIMIN,XIMAX,XI

      IF (IREJ.GT.0) GO TO 999
      IF (GQ22(2).GT.GQ22(INT(GQ22(1))+2)) GO TO 999
 
      AJACOB = 1.0D0
      GQ22(8) = 0.0D0
      GQ22(INT(GQ22(1))+8) = 1.0D0
      DO II = 1,INT(GQ22(1))
         IF (RN.GT.GQ22(II+7).AND.RN.LE.GQ22(II+8)) THEN
            RNA = (RN - GQ22(II+7))/(GQ22(II+8) - GQ22(II+7))
            KPAR = INT(GQ22(II+13))
            Q22MIN = GQ22(II+1)
            Q22MAX = GQ22(II+2)
            AJACOB = AJACOB/(GQ22(II+8) - GQ22(II+7))
         ENDIF
      ENDDO
 
      IF (Q22MIN.GT.Q22MAX) THEN
         IREJ = 1
         WRITE(*,*) 'Warning : Q22min > Q22max'
         RETURN
      ENDIF

      IF (KPAR.EQ.1) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA
         AJACOB = AJACOB*(Q22MAX - Q22MIN)
      ELSE IF (KPAR.EQ.2) THEN
         Q22 = Q22MIN*(Q22MAX/Q22MIN)**RNA
         AJACOB = AJACOB*Q22*LOG(Q22MAX/Q22MIN)
      ELSE IF (KPAR.EQ.3) THEN
         XIMIN = 1.0D0/SQRT(Q22MAX)
         XIMAX = 1.0D0/SQRT(Q22MIN)
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/XI**2
         AJACOB = AJACOB*(XIMAX - XIMIN)*2.0D0/XI**3
      ELSE IF (KPAR.EQ.4) THEN
         XIMIN = 1.0D0/Q22MAX
         XIMAX = 1.0D0/Q22MIN
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/XI
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q22**2
      ELSE IF (KPAR.EQ.5) THEN
         XIMIN = 1.0D0/Q22MAX**2
         XIMAX = 1.0D0/Q22MIN**2
         XI = XIMAX + (XIMIN - XIMAX)*RNA
         Q22 = 1.0D0/SQRT(XI)
         AJACOB = AJACOB*(XIMAX - XIMIN)*Q22**3*0.5D0
      ELSE IF (KPAR.EQ.6) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*SQRT(RNA)
         AJACOB = AJACOB*(Q22MAX - Q22MIN)**2
     $        /(2.0D0*(Q22 - Q22MIN))
      ELSE IF (KPAR.EQ.7) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA**(1.0D0/3.0D0)
         AJACOB = AJACOB*(Q22MAX - Q22MIN)**3/(Q22 - Q22MIN)**2/3.0D0
      ELSE IF (KPAR.EQ.8) THEN
         Q22 = Q22MIN + (Q22MAX - Q22MIN)*RNA**(2.0D0/3.0D0)
         AJACOB = AJACOB*2.0D0/3.0D0*(Q22MAX - Q22MIN)**(3.0D0/2.0D0)
     $        /SQRT(Q22 - Q22MIN)
      ELSE IF (KPAR.EQ.9) THEN
C        Empty
      ELSE IF (KPAR.EQ.10) THEN
         THEMIN = ATAN((Q22MIN - AMW**2)/(AMW*AGW))
         THEMAX = ATAN((Q22MAX - AMW**2)/(AMW*AGW))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMW*AGW*TAN(THE) + AMW**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMW**2)**2 + (AMW*AGW)**2)/(AMW*AGW)
      ELSE IF (KPAR.EQ.11) THEN
         THEMIN = ATAN((Q22MIN - AMZ**2)/(AMZ*AGZ))
         THEMAX = ATAN((Q22MAX - AMZ**2)/(AMZ*AGZ))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMZ*AGZ*TAN(THE) + AMZ**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMZ**2)**2 + (AMZ*AGZ)**2)/(AMZ*AGZ)
      ELSE IF (KPAR.EQ.12) THEN
         THEMIN = ATAN((Q22MIN - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THEMAX = ATAN((Q22MAX - AMUQ(3)**2)/(AMUQ(3)*AGUQ(3)))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMUQ(3)*AGUQ(3)*TAN(THE) + AMUQ(3)**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMUQ(3)**2)**2
     $        + (AMUQ(3)*AGUQ(3))**2)/(AMUQ(3)*AGUQ(3))
      ELSE IF (KPAR.EQ.13) THEN
         THEMIN = ATAN((Q22MIN - AMH**2)/(AMH*AGH))
         THEMAX = ATAN((Q22MAX - AMH**2)/(AMH*AGH))
         THE = THEMIN + (THEMAX - THEMIN)*RNA
         Q22 = AMH*AGH*TAN(THE) + AMH**2
         AJACOB = AJACOB*(THEMAX - THEMIN)
     $        *((Q22 - AMH**2)**2 + (AMH*AGH)**2)/(AMH*AGH)
      ENDIF
 
      RETURN
 999  CONTINUE
      IREJ = 1
      RETURN
      END

C----------------------------
C Set subsequent cos(theta).
C----------------------------
      SUBROUTINE GRCSBCS(KPAR,EPS,RN,SUBCOS,AJACOB,SCSMIN,SCSMAX)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER KPAR
      DOUBLE PRECISION SUBCOS,RN,RNA,AJACOB,EPS,SCSMIN,SCSMAX
      DOUBLE PRECISION PF4(4,10),Q1(4),Q2(4),PK1(4),PK2(4)
      DOUBLE PRECISION E1,E2,E5,E6,EE1,EE2,EQ1,EQ2
      DOUBLE PRECISION P1,P5,P2,P6,PP1,PP2,PQ1
      DOUBLE PRECISION COS1,SIN1,SIN3,SIN4,PHI1,ETA
      DOUBLE PRECISION XI,XIMIN,XIMAX
      DOUBLE PRECISION D,DMIN,DMAX
      DOUBLE PRECISION V,VMIN,VMAX
      DOUBLE PRECISION G,GMIN,GMAX
      DOUBLE PRECISION D13,D24,D14,D23
 
      IF (KPAR.EQ.2.AND.EPS.LT.1.0D-5) THEN
         IREJ = 3
         WRITE(*,*) "ERROR: WRONG SUBSYSTEM PARAMETER"
      ENDIF
 
      IF (KPAR.EQ.1) THEN
         SUBCOS = SCSMIN + (SCSMAX - SCSMIN)*RN
         AJACOB = SCSMAX - SCSMIN
      ELSE IF (KPAR.EQ.2) THEN
         DMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         DMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         D = DMIN + (DMAX - DMIN)*RN
         SUBCOS = (1.0D0 + EPS)*TANH(D)
         AJACOB = (DMAX - DMIN)/COSH(D)**2*(1.0D0 + EPS)
 
         SUBCOS = (SCSMAX-SCSMIN)/2.0D0*SUBCOS+(SCSMAX+SCSMIN)/2.0D0
         AJACOB = AJACOB*(SCSMAX-SCSMIN)/2.0D0
C...1/u
      ELSE IF (KPAR.EQ.3) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E5 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
         P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
         DMIN = 2*(E1*E5 + P1*P5*SCSMIN) - AMASS2(IPX(5))
         DMAX = 2*(E1*E5 + P1*P5*SCSMAX) - AMASS2(IPX(5))
 
         D = DMIN*(DMAX/DMIN)**RN
         SUBCOS = E1*E5/P1/P5 - (D + AMASS2(IPX(5)))/2.D0/P1/P5
         AJACOB = D*LOG(DMAX/DMIN)/2.0D0/P1/P5
C...1/t
      ELSE IF (KPAR.EQ.4) THEN
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
         E6 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
         DMIN = 2*(E2*E6 + P2*P6*SCSMIN) - AMASS2(IPX(6))
         DMAX = 2*(E2*E6 + P2*P6*SCSMAX) - AMASS2(IPX(6))
 
         XIMIN = 1.0D0/DMAX
         XIMAX = 1.0D0/DMIN
         XI = XIMAX + (XIMIN - XIMAX)*RN
         SUBCOS = -E2*E6/P2/P6 + (1.0D0/XI + AMASS2(IPX(6)))/2.D0/P2/P6
         AJACOB = (XIMAX - XIMIN)/XI**2/2.0D0/P2/P6
 
C...1/u
      ELSE IF (KPAR.EQ.5) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E5 = (GPRI(4) - GVAL(1) + GVAL(2))/2.0D0
         P5 = SQRT(E5**2 - AMASS2(IPX(5)))
 
         VMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         VMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         V = VMIN + (VMAX - VMIN)*RN
 
         RNA = (1.0D0 + EPS)*TANH(V)
         AJACOB = (VMAX - VMIN)/COSH(V)**2*(1.0D0 + EPS)
 
         RNA = (RNA + 1.0D0)/2.0D0
         AJACOB = AJACOB/2.0D0
 
 
         DMAX = 2.0D0*E1*E5 - 2.0D0*P1*P5*SCSMIN - AMASS2(IPX(5))
         DMIN = 2.0D0*E1*E5 - 2.0D0*P1*P5*SCSMAX - AMASS2(IPX(5))
 
         D = DMIN*(DMAX/DMIN)**RNA
         SUBCOS = (2.0D0*E1*E5 - D - AMASS2(IPX(5)))/2.0D0/P1/P5
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P1/P5
 
C...1/t
      ELSE IF (KPAR.EQ.6) THEN
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
         E6 = (GPRI(4) - GVAL(1) - GVAL(2))/2.0D0
         P6 = SQRT(E6**2 - AMASS2(IPX(6)))
 
         VMIN = -LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         VMAX = LOG(1.0D0 + 2.0D0/EPS)/2.0D0
         V = VMIN + (VMAX - VMIN)*RN
 
         RNA = (1.0D0 + EPS)*TANH(V)
         AJACOB = (VMAX - VMIN)/COSH(V)**2*(1.0D0 + EPS)
 
         RNA = (RNA + 1.0D0)/2.0D0
         AJACOB = AJACOB/2.0D0
 
 
         DMIN = 2.0D0*E2*E6 + 2.0D0*P2*P6*SCSMIN - AMASS2(IPX(6))
         DMAX = 2.0D0*E2*E6 + 2.0D0*P2*P6*SCSMAX - AMASS2(IPX(6))
 
         IF (DMAX.LT.DMIN) THEN
            WRITE(*,*) SCSMIN,SCSMAX,DMAX,DMIN
         ENDIF
 
         D = DMIN*(DMAX/DMIN)**RNA
         SUBCOS = -(2.0D0*E2*E6 - D - AMASS2(IPX(6)))/2.0D0/P2/P6
         AJACOB = AJACOB*D*LOG(DMAX/DMIN)/2.0D0/P2/P6
 
      ELSE IF (KPAR.EQ.7) THEN
         E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.D0/GPRI(4)
         P1 = SQRT((E1+AMASS1(1))*(E1-AMASS1(1)))
         E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.D0/GPRI(4)
         P2 = SQRT((E2+AMASS1(2))*(E2-AMASS1(2)))
 
C...Particle 1
         PF4(1,1) = 0
         PF4(2,1) = 0
         PF4(3,1) = P1
         PF4(4,1) = E1
 
C...Particle 2
         PF4(1,2) = 0
         PF4(2,2) = 0
         PF4(3,2) = -P2
         PF4(4,2) = E2
 
C...q1 q2
         COS1 = GVAL(1)
         SIN1 = SQRT((1.0D0 - COS1)*(1.0D0 + COS1))
         PHI1 = GVAL(2)
 
         EQ1 = (GPRI(3) + GVAL(3) - GVAL(4))/2.D0/GPRI(4)
         PQ1 = SQRT(EQ1*EQ1 - GVAL(3))
         Q1(1) = PQ1*SIN1*COS(PHI1)
         Q1(2) = PQ1*SIN1*SIN(PHI1)
         Q1(3) = PQ1*COS1
         Q1(4) = EQ1
 
         EQ2 = (GPRI(3) + GVAL(4) - GVAL(3))/2.D0/GPRI(4)
         Q2(1) = -Q1(1)
         Q2(2) = -Q1(2)
         Q2(3) = -Q1(3)
         Q2(4) = EQ2
 
         E5 = (GVAL(4)+AMASS2(IPX(5))-AMASS2(IPX(6)))
     $        /2.0D0/SQRT(GVAL(4))
         P5 = SQRT((E5-AMASS1(IPX(5)))*(E5+AMASS1(IPX(5))))
         E6 = (GVAL(4)+AMASS2(IPX(6))-AMASS2(IPX(5)))
     $        /2.0D0/SQRT(GVAL(4))
         P6 = P5
 
C...p1-p5
         IF (RN.LT.0.25D0) THEN
C            idg=1
            CALL LABTOK(PF4(1,1),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = RN*4.0D0
            AJACOB = 4.0D0
            EE1 = PK1(4)
            PP1 = SQRT((EE1-AMASS1(1))*(EE1+AMASS1(1)))
            ETA = 0.0D0
 
            DMIN = 2.0D0*(EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P5)-AMASS2(1)-AMASS2(IPX(5))
 
            EPS = 0
            IF (DMIN.LE.0) THEN
               EPS = AMASS2(1) - DMIN
               DMIN = DMIN + EPS
            ENDIF
 
C            eta = 2.0d0
            ETA = 1.0D0
            DMAX = 2.0D0*(EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P5)-AMASS2(1)-AMASS2(IPX(5))+EPS
 
            IF (DMIN.GT.DMAX) GOTO 999
 
            D = DMIN*(DMAX/DMIN)**RN
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/PP1/P5/2.D0
            D = D - EPS
C            d1 = d
            D = (D+AMASS2(1)+AMASS2(IPX(5)))/2.0D0
            SUBCOS = EE1*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(1)/(EE1+PP1)-D
            SUBCOS = 1.D0 + SUBCOS/PP1/P5
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN3 = SQRT((1.0D0-SUBCOS)*(1.0D0+SUBCOS))
 
            D13 = D
            D24 = E6*PK2(4)+P6*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D14 = E6*PK1(4)+P6*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = E5*PK2(4)-P5*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d13.gt.min(d24,d14,d23)) goto 999
            IF (D13.GT.D24) GOTO 999
 
         ELSE IF (RN.LT.0.5D0) THEN
C...p2-p6
C            idg=2
            CALL LABTOK(PF4(1,2),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.25D0)*4.0D0
            AJACOB = 4.0D0
            EE2 = PK2(4)
            PP2 = SQRT((EE2-AMASS1(2))*(EE2+AMASS1(2)))
            ETA=0.D0
 
            GMIN = 2.0D0*(EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P6)-AMASS2(2)-AMASS2(IPX(6))
            EPS=0
            IF (GMIN.LE.0) THEN
               EPS = AMASS2(2) - GMIN
               GMIN = GMIN+EPS
            ENDIF
 
C            eta = 2.0d0
            ETA = 1.0D0
            GMAX=2.0D0*(EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P6)-AMASS2(2)-AMASS2(IPX(6))+EPS
 
            IF (GMIN.GT.GMAX) GOTO 999
 
            G = GMIN*(GMAX/GMIN)**RN
            AJACOB = AJACOB*G*LOG(GMAX/GMIN)/PP2/P6/2.0D0
            G = G - EPS
C            g1 = g
            G = (G+AMASS2(2)+AMASS2(IPX(6)))/2.0D0
            SUBCOS = EE2*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(2)/(EE2+PP2)-G
            SUBCOS = 1.0D0 + SUBCOS/PP2/P6
 
C            subcos = -subcos
 
            IF (ABS(SUBCOS).GT.1) GOTO 999
 
            SIN4 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
 
            D13 = E5*PK1(4)+P5*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = G
            D14 = E6*PK1(4)-P6*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = E5*PK2(4)+P5*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d24.gt.min(d13,d14,d23)) goto 999
            IF (D24.GT.D13) GOTO 999
 
         ELSE IF (RN.LT.0.75D0) THEN
C...p1-p6
C         idg=3
            CALL LABTOK(PF4(1,1),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.5D0)*4.0D0
            AJACOB = 4.0D0
            EE1 = PK1(4)
            PP1 = SQRT((EE1-AMASS1(1))*(EE1+AMASS1(1)))
            ETA = 0.0D0
 
            DMIN = 2.0D0*(EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P6)-AMASS2(1)-AMASS2(IPX(6))
            EPS = 0
            IF (DMIN.LE.0) THEN
               EPS = AMASS2(1) - DMIN
               DMIN = DMIN + EPS
            END IF
 
C            eta = 2.0d0
            ETA = 1.0D0
            DMAX = 2.0D0*(EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)
     $           /(EE1+PP1)+ETA*PP1*P6)-AMASS2(1)-AMASS2(IPX(6))+EPS
 
            IF (DMIN.GT.DMAX) GOTO 999
 
            D = DMIN*(DMAX/DMIN)**RN
            AJACOB = AJACOB*D*LOG(DMAX/DMIN)/PP1/P6/2.0D0
            D = D - EPS
            D = (D + AMASS2(1) + AMASS2(IPX(6)))/2.0D0
            SUBCOS = EE1*AMASS2(IPX(6))/(E6+P6)+P6*AMASS2(1)/(EE1+PP1)-D
            SUBCOS = 1.0D0 + SUBCOS/PP1/P6
 
C            subcos = -subcos
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN4 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
            D14 = D
            D13 = E5*PK1(4)+P5*(PK1(1)*SIN4*COS(GVAL(8))
     $           +PK1(2)*SIN4*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = E6*PK2(4)-P6*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D23 = E5*PK2(4)+P5*(PK2(1)*SIN4*COS(GVAL(8))
     $           +PK2(2)*SIN4*SIN(GVAL(8))+PK2(3)*SUBCOS)
 
C            if (d14.gt.min(d24,d13,d23)) goto 999
            IF (D14.GT.D23) GOTO 999
 
         ELSE
C...p2-p5
C         idg=4
            CALL LABTOK(PF4(1,2),Q2,PF4(1,1),PF4(1,2), PK1,PK2)
            RN = (RN - 0.75D0)*4.0D0
            AJACOB = 4.0D0
            EE2 = PK2(4)
            PP2 = SQRT((EE2-AMASS1(2))*(EE2+AMASS1(2)))
            ETA = 0.D0
            GMIN = 2.0D0*(EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P5)-AMASS2(2)-AMASS2(IPX(5))
            EPS = 0.0D0
            IF (GMIN.LE.0) THEN
               EPS = AMASS2(2) - GMIN
               GMIN = GMIN + EPS
            END IF
 
C            eta = 2.0d0
            ETA = 1.0D0
            GMAX = 2.0D0*(EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)
     $           /(EE2+PP2)+ETA*PP2*P5)-AMASS2(2)-AMASS2(IPX(5))+EPS
 
            IF (GMIN.GT.GMAX) GOTO 999
 
            G = GMIN*(GMAX/GMIN)**RN
            AJACOB = AJACOB*G*LOG(GMAX/GMIN)/PP2/P5/2.0D0
            G = G - EPS
            SUBCOS = EE2*AMASS2(IPX(5))/(E5+P5)+P5*AMASS2(2)/(EE2+PP2)-G
            SUBCOS = 1.0D0 + SUBCOS/PP2/P5
 
            IF(ABS(SUBCOS).GT.1) GOTO 999
 
            SIN3 = SQRT((1.0D0 - SUBCOS)*(1.0D0 + SUBCOS))
 
            D13 = E5*PK1(4)-P5*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D24 = E6*PK2(4)+P6*(PK2(1)*SIN3*COS(GVAL(8))
     $           +PK2(2)*SIN3*SIN(GVAL(8))+PK2(3)*SUBCOS)
            D14 = E6*PK1(4)+P6*(PK1(1)*SIN3*COS(GVAL(8))
     $           +PK1(2)*SIN3*SIN(GVAL(8))+PK1(3)*SUBCOS)
            D23 = G
 
C            if (d23.gt.min(d13,d14,d24)) goto 999
            IF (D23.GT.D14) GOTO 999
 
         ENDIF
      ENDIF
 
      RETURN
 999  CONTINUE
      IREJ = 1
      RETURN
      END
 
C**********************************************************************
C*======================                                              *
C* FUNCTION DRN( ISEED)                                               *
C*======================                                              *
C*  Machine-independent Random number generator                       *
C*     General purpose Version,  OK as long as >= 32 bits             *
C*((Arguement))                                                       *
C*  ISEED: Seed                                                       *
C*                                                                    *
C**********************************************************************
 
*     REAL FUNCTION DRN*8(ISEED)
      DOUBLE PRECISION FUNCTION DRN(ISEED)
 
      COMMON/RANDM/RDM(31),RM1,RM2,IA1,IC1,M1,IX1,
     .                             IA2,IC2,M2,IX2,
     .                             IA3,IC3,M3,IX3
 
      REAL*8 PYR
 
C Generate Next number in sequence
 
      IX1    = MOD( IA1*IX1+IC1, M1 )
      IX2    = MOD( IA2*IX2+IC2, M2 )
      IX3    = MOD( IA3*IX3+IC3, M3 )
      J      = 1 + (31*IX3)/M3
 
CCC      DRN    = PYR(0)
      DRN    = RDM(J)
 
      RDM(J) = ( FLOAT(IX1)+FLOAT(IX2)*RM2 )*RM1
 
C Omit following statement if function arguement passed by value:
 
      ISEED = IX1
      RETURN
      END
 
C----------------------------
C Set kinematical variables.
C----------------------------
      SUBROUTINE GRCSVAL
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      DOUBLE PRECISION AJACOB

      IF (KTYPE.EQ.2101) CALL KVH2101(AJACOB)
      IF (KTYPE.EQ.2201) CALL KVH2201(AJACOB)
      IF (KTYPE.EQ.2301) CALL KVH2301(AJACOB)
      IF (KTYPE.EQ.2302) CALL KVH2302(AJACOB)
      IF (KTYPE.EQ.2303) CALL KVH2303(AJACOB)
      IF (KTYPE.EQ.2401) CALL KVH2401(AJACOB)
      IF (KTYPE.EQ.2402) CALL KVH2402(AJACOB)
      IF (KTYPE.EQ.2403) CALL KVH2403(AJACOB)
      IF (KTYPE.EQ.2404) CALL KVH2404(AJACOB)
      IF (KTYPE.EQ.2405) CALL KVH2405(AJACOB)
      IF (KTYPE.EQ.2501) CALL KVH2501(AJACOB)
      IF (KTYPE.EQ.2502) CALL KVH2502(AJACOB)
      IF (KTYPE.EQ.2503) CALL KVH2503(AJACOB)
      IF (KTYPE.EQ.2504) CALL KVH2504(AJACOB)
      IF (KTYPE.EQ.2505) CALL KVH2505(AJACOB)
      IF (KTYPE.EQ.2601) CALL KVH2601(AJACOB)
      IF (KTYPE.EQ.2602) CALL KVH2602(AJACOB)
      IF (KTYPE.EQ.2701) CALL KVH2701(AJACOB)
      IF (KTYPE.EQ.2702) CALL KVH2702(AJACOB)
      YACOB = YACOB*AJACOB

C...Check Flag.
      IF (IREJ.GT.0) GO TO 999
C...Fill temp.Kinematics.
      CALL GRCSKIN(PGRC,AJACOB)
C...Set Coupling constant.
      CALL GRCSENE
      IF (YACOB.LT.0) IREJ = 1

      RETURN
 999  CONTINUE
      IREJ = 1
      YACOB = 0.0D0
      RETURN
      END

C------------------------
C Set n-body kinematics.
C------------------------
      SUBROUTINE GRCSKIN(PF4,AJACOB)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      DOUBLE PRECISION AJACOB
      DOUBLE PRECISION PF4(4,10)

      IF (KTYPE.EQ.2101) CALL KSH2101(PF4,AJACOB)
      IF (KTYPE.EQ.2201) CALL KSH2201(PF4,AJACOB)
      IF (KTYPE.EQ.2301) CALL KSH2301(PF4,AJACOB)
      IF (KTYPE.EQ.2302) CALL KSH2302(PF4,AJACOB)
      IF (KTYPE.EQ.2303) CALL KSH2303(PF4,AJACOB)
      IF (KTYPE.EQ.2401) CALL KSH2401(PF4,AJACOB)
      IF (KTYPE.EQ.2402) CALL KSH2402(PF4,AJACOB)
      IF (KTYPE.EQ.2403) CALL KSH2403(PF4,AJACOB)
      IF (KTYPE.EQ.2404) CALL KSH2404(PF4,AJACOB)
      IF (KTYPE.EQ.2405) CALL KSH2405(PF4,AJACOB)
      IF (KTYPE.EQ.2501) CALL KSH2501(PF4,AJACOB)
      IF (KTYPE.EQ.2502) CALL KSH2502(PF4,AJACOB)
      IF (KTYPE.EQ.2503) CALL KSH2503(PF4,AJACOB)
      IF (KTYPE.EQ.2504) CALL KSH2504(PF4,AJACOB)
      IF (KTYPE.EQ.2505) CALL KSH2505(PF4,AJACOB)
      IF (KTYPE.EQ.2601) CALL KSH2601(PF4,AJACOB)
      IF (KTYPE.EQ.2602) CALL KSH2602(PF4,AJACOB)
      IF (KTYPE.EQ.2701) CALL KSH2701(PF4,AJACOB)
      IF (KTYPE.EQ.2702) CALL KSH2702(PF4,AJACOB)

      RETURN
      END
 
C-----------------------
C Set kinematical cuts.
C-----------------------
      SUBROUTINE GRCSCUT
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
      INTEGER I,J,IUSRCUT,IZSYSM
      DOUBLE PRECISION PPZ,PPP
      DOUBLE PRECISION SUMX,SUMY,SUMPT

      DOUBLE PRECISION GRCDRCN
      EXTERNAL GRCDRCN

C...Set 4-Momenmum for Initial State Particles
      DO I=1,4
         PLGRC(I,1) = PGRC(I,1)
         PLGRC(I,2) = PGRC(I,2)
      ENDDO

      SUMX = 0.0D0
      SUMY = 0.0D0

C...Special... must be optimized...
      IZSYSM = 0
      IF (IGPCOMB(GRCFLAV+1).EQ.1) IZSYSM = 1

      IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.444.OR.IGSUB.EQ.448
     $     .OR.IGSUB.EQ.551.OR.IGSUB.EQ.556.OR.IGSUB.EQ.567
     $     .OR.IGSUB.EQ.577) THEN
         IF (GRCFLAV.GT.132) THEN
            IZSYSM = 0
            IF (KFCODE(1).LT.0) IZSYSM = 1
         ENDIF
         IF (KFCODE(3).GT.0) THEN
            IZSYSM = 1-IZSYSM
         ENDIF
      ENDIF

      IF (IGSUB.EQ.302.OR.IGSUB.EQ.310.OR.IGSUB.EQ.319
     $     .OR.IGSUB.EQ.327) THEN
         IZSYSM = 0
      ENDIF

      IF (IGSUB.EQ.504.OR.IGSUB.EQ.512.OR.IGSUB.EQ.520
     $     .OR.IGSUB.EQ.560) THEN
         IZSYSM = 0
      ENDIF

      IF (NNF.GT.3) THEN
C...Set Px,Py,Pz to Pt,Phi,Eta for Final State Particles
         DO I=3,NNF

C...Set pt,phi,eta,E
            PLGRC(1,I) = SQRT(PGRC(1,I)**2+PGRC(2,I)**2)
            IF (PGRC(2,I).GT.0) THEN
               PLGRC(2,I) = ACOS(PGRC(1,I)/PLGRC(1,I))
            ELSE
               PLGRC(2,I) = -ACOS(PGRC(1,I)/PLGRC(1,I))
            ENDIF
C...Set pz
            IF (IZSYSM.EQ.0) THEN
               PPZ = GPRI(8)*(PGRC(3,I) + GPRI(7)*PGRC(4,I))
            ELSE
               PPZ = GPRI(8)*(-PGRC(3,I) + GPRI(7)*PGRC(4,I))
            ENDIF
            PPP = SQRT(PGRC(1,I)**2+PGRC(2,I)**2+PPZ**2)
C...Set energy
            PLGRC(4,I) = GPRI(8)*(PGRC(4,I) + GPRI(7)*PGRC(3,I))
C...Set pseudo-rapidity
            PLGRC(3,I) = 0.5D0*LOG((PPP+PPZ)/(PPP-PPZ))
C...Set "true" rapidity
C         plgrc(3,i) = 0.5d0*log((plgrc(4,i)+ppz)/(plgrc(4,i)-ppz))

C...Pt and Eta Cut
            IF (PLGRC(1,I).LT.GRCPTCUT(I-2)) GO TO 999
            IF (ABS(PLGRC(3,I)).GT.GRCETACUT(I-2)) GO TO 999
 
            SUMX = PGRC(1,I) + SUMX
            SUMY = PGRC(2,I) + SUMY
         ENDDO
C...Set SumPt
         SUMPT = SQRT(SUMX**2 + SUMY**2)
C...MET Cut
         IF (SUMPT.LT.GRCMETCUT) GO TO 999
C...Set Rcone
         DO I = 3,NNF
            DO J = 3,NNF
               IF (I.NE.J) THEN
                  IF (GRCDRCN(I,J).LT.GRCRCONCUT(I-2)
     $                 .AND.GRCRCONCUT(J-2).GT.0.0D0) THEN
                     GO TO 999
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDIF

C...Call user defined cut
      IUSRCUT = 0
      CALL GRCUSRCUT(IUSRCUT)
      IF (IUSRCUT.EQ.1) GO TO 999
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END


C-------------------------
C Smear to PYTHIA/HERWIG.
C-------------------------
      SUBROUTINE GRCSSMR
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      INTEGER I,J,IMR,ISEL,IZSYSM
      DOUBLE PRECISION XMRESO(4),AJACOB,X1MASS
      CHARACTER*60 XFMT


      IF (IBSWRT.LT.2) THEN

C...Set Phase Region
         CALL GRCXSET(ISEL)
         GRCIREG = ISEL

C...Set Kinematics.
         CALL GRCSKIN(PGRC,AJACOB)

C...Apply Kinematical Cuts
         CALL GRCSCUT

C...Set Kinematics to PYTHIA
         IF (KFCODE(1).LT.7) THEN
            AMASS1(1) = GAMS(IABS(KFCODE(1)))
         ELSE
            AMASS1(1) = AMG
         ENDIF
         IF (KFCODE(2).LT.7) THEN
            AMASS1(2) = GAMS(IABS(KFCODE(2)))
         ELSE
            AMASS1(2) = AMG
         ENDIF

C...Special... must be optimized...
         IZSYSM = 0
         IF (IGPCOMB(GRCFLAV+1).EQ.1) THEN
            IZSYSM = 1
         ENDIF

         IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.444
     $        .OR.IGSUB.EQ.448.OR.IGSUB.EQ.551.OR.IGSUB.EQ.556
     $        .OR.IGSUB.EQ.567.OR.IGSUB.EQ.577) THEN
            IF (GRCFLAV.GT.132) THEN
               IZSYSM = 0
               IF (KFCODE(1).LT.0) IZSYSM = 1
            ENDIF
            IF (KFCODE(3).GT.0) THEN
               IZSYSM = 1-IZSYSM
            ENDIF
         ENDIF

         IF (IGSUB.EQ.302.OR.IGSUB.EQ.310.OR.IGSUB.EQ.319
     $        .OR.IGSUB.EQ.327) THEN
            IZSYSM = 0
         ENDIF

         IF (IGSUB.EQ.504.OR.IGSUB.EQ.512.OR.IGSUB.EQ.520
     $        .OR.IGSUB.EQ.560) THEN
            IZSYSM = 0
         ENDIF

         IF (IZSYSM.EQ.1) THEN
            DO I = 3,NNF
               PGRC(3,I) = -PGRC(3,I)
            ENDDO
            DO I = 1,4
               PQGRC(3,I) = -PQGRC(3,I)
            ENDDO
         ENDIF

C...Set Particle Combination
         CALL GRCSWAP(ISEL)
C...Set Helicity State.
         CALL SPDETH(ISEL)
C...Set Color String
         CALL SPDETC(ISEL)

C...For special usage. 
      ELSE IF (IBSWRT.EQ.2) THEN

C...For special usage.
C...int(1*nup)+int(3*(nup-2))+int(2*nup)+int(nup)+real(5*nup)+3
         XFMT = '(##i4,##i3,##i4,##f4.0,##f11.4)'
         WRITE(XFMT(2:3),FMT='(i2)') NUP
         WRITE(XFMT(7:8),FMT='(i2)') 3*(NUP-2)
         WRITE(XFMT(12:13),FMT='(i2)') 2*NUP
         WRITE(XFMT(17:18),FMT='(i2)') NUP
         WRITE(XFMT(24:25),FMT='(i2)') 5*NUP+3
         READ(14,FMT=XFMT) 
     $        (IDUP(J),J=1,NUP),(ISTUP(J),J=3,NUP),
     $        ((MOTHUP(I,J),I=1,2),J=3,NUP),
     $        ((ICOLUP(I,J),I=1,2),J=1,NUP),
     $        (SPINUP(J),J=1,NUP),
     $        ((PUP(I,J),I=1,5),J=1,NUP),SCALUP,AQEDUP,AQCDUP

C...Status codes for entries.
         ISTUP(1) = -1
         ISTUP(2) = -1
      ENDIF

      RETURN
      END
 
C-----------------
C Set B.W masses.
C-----------------
      SUBROUTINE GRCSBWM(PEBW)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'
 
#include "define.h"
 
#if PSMODEL > 0
      INTEGER MSTP,MSTI
      DOUBLE PRECISION PARP,PARI
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
#else
      DOUBLE PRECISION PARP(50)
#endif
 
      INTEGER I,IDUMMY,IMR,IMRX(4)
      DOUBLE PRECISION XMR,XMRG,XMRMIN,XMRMAX
      DOUBLE PRECISION AMRDN,AMRUP,AMRTO,XMRESO(4)
      DOUBLE PRECISION PQ1,PQ2,COSQ1,COSQ2,SINQ1,SINQ2,AAA,BBB,CCC
      DOUBLE PRECISION PEBW(4,8),AJACOB

      DOUBLE PRECISION DRN
      EXTERNAL DRN

C...Decide W, Z, H
      IMR = 0
      DO I=3,NNF
         IF (ABS(KFCODE(I)).EQ.24) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMW
            XMRG = AGW
         ELSE IF (ABS(KFCODE(I)).EQ.23) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMZ
            XMRG = AGZ
         ELSE IF (ABS(KFCODE(I)).EQ.25) THEN
            IMR = IMR + 1
            IMRX(IMR) = I
            XMR = AMH
            XMRG = AGH
         ENDIF
      ENDDO
 
      IF (IMR.EQ.0) THEN
CCC         write(*,*) 'Error: No B.W. particle'
      ENDIF
 
C...Breit Wigner Mass
      DO I=1,IMR
         IF (KTYPE.LT.10) THEN
            XMRMIN = PARP(42)
            XMRMAX = PGRC(4,IMRX(IMR))
         ELSE
            PQ1 = GVAL(12)
            PQ2 = GVAL(13)
            COSQ1 = GVAL(5)
            COSQ2 = GVAL(6)
            SINQ1 = SQRT((1.0D0 - COSQ1)*(1.0D0 + COSQ1))
            SINQ2 = SQRT((1.0D0 - COSQ2)*(1.0D0 + COSQ2))
 
            AAA = (PQ1*COSQ1 + PQ2*COSQ2)**2
            BBB = (PQ1*SINQ1 - PQ2*SINQ2)**2
            CCC = (PQ1*SINQ1 + PQ2*SINQ2)**2
 
            XMRMIN = SQRT(MAX(PARP(42)**2,
     $           PGRC(4,IMRX(IMR))**2-AAA-CCC))
            XMRMAX = SQRT(PGRC(4,IMRX(IMR))**2-AAA-BBB)
         ENDIF
 
         AMRDN = ATAN((XMRMIN**2 - XMR**2)/(XMR*XMRG))
         AMRUP = ATAN((XMRMAX**2 - XMR**2)/(XMR*XMRG))
 
         AMRTO = AMRDN + (AMRUP - AMRDN)*DRN(IDUMMY)
         XMRESO(I) = SQRT(XMR**2 + XMR*XMRG*TAN(AMRTO))
         AMASS1(IMRX(IMR)) = XMRESO(I)
         AMASS2(IMRX(IMR)) = XMRESO(I)**2
 
      ENDDO
 
C...Set new kinematics
      CALL GRCSKIN(PEBW,AJACOB)
 
      RETURN
      END

C---------------------------
C Set particle combination.
C---------------------------
      SUBROUTINE GRCSPID
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INTEGER I
      NNF = (KTYPE-2000)/100+2
      DO I = 1,NNF
         IPX(I) = MOD(CXTYPE/(10**(NNF-I)),10)
      ENDDO
      RETURN
      END
 
C    _/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C   _/_/  Set momentum fraction from PDF  _/_/
C  _/_/  Date   10/11/2000               _/_/
C _/_/  Author Soushi Tsuno             _/_/
C_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
C
C subroutine grcprtngg
C  _______________________
C | Initial |             |
C |  State  | gluon,gluon |
C +=========+=============+
C | grcflav |      0      |
C | symmetry|      0      |
C +---------+-------------+
C
C subroutine grcprtnqq
C  _____________________________________________________________________
C | Initial |  _ |  _ |  _ |  _ | _  | _  | _  | _  |  _ | _  |  _ | _  |
C |  State  | uu | dd | cc | ss | uu | dd | cc | ss | bb | bb | tt | tt |
C +=========+====+====+====+====+====+====+====+====+====+====+====+====+
C | grcflav |  1 |  2 |  3 |  4 |  5 |  6 |  7 |  8 |  9 | 10 | 11 | 12 |
C | symmetry|  0 |  0 |  0 |  0 |  1 |  1 |  1 |  1 |  0 |  1 |  0 |  1 |
C +---------+----+----+----+----+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | uc | ds | cu | sd | uc | ds | cu | sd |
C           +====+====+====+====+====+====+====+====+
C           | 133| 134| 135| 136| 137| 138| 139| 140|
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C b-content  _______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | db | sb | bd | bs | bd | bs | db | sb |
C           +====+====+====+====+====+====+====+====+
C           | 141| 142| 143| 144| 145| 146| 147| 148|
C           |  0 |  0 |  1 |  1 |  1 |  1 |  0 |  0 |
C           +----+----+----+----+----+----+----+----+
C top-content_______________________________________
C           |  _ |  _ |  _ |  _ | _  | _  | _  | _  |
C           | ut | ct | tu | tc | ut | ct | tu | tc |
C           +====+====+====+====+====+====+====+====+
C           | 149| 150| 151| 152| 153| 154| 155| 156|
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqb
C Without quark mixing
C  _________________________________________________
C | Initial |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C |  State  | ud | cs | du | sc | ud | cs | du | sc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 13 | 14 | 15 | 16 | 17 | 18 | 19 | 20 |
C | symmetry|  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C +---------+----+----+----+----+----+----+----+----+
C With quark mixing
C  _________________________________________________
C | Initial |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C |  State  | us | cd | su | dc | us | cd | su | dc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 21 | 22 | 23 | 24 | 25 | 26 | 27 | 28 |
C | symmetry|  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |  _ |  _ | _  | _  | _  | _  |  _ |  _ |
C           | ub | cb | bu | bc | ub | cb | bu | bc |
C           +====+====+====+====+====+====+====+====+
C           | 29 | 30 | 31 | 32 | 33 | 34 | 35 | 36 |
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqg
C  _________________________________________________
C | Initial |    |    |    |    | _  | _  | _  | _  |
C |  State  | ug | dg | cg | sg | ug | dg | cg | sg |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 37 | 38 | 39 | 40 | 41 | 42 | 43 | 44 |
C | symmetry|  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    |  _ |  _ |  _ |  _ |
C           | gu | gd | gc | gs | gu | gd | gc | gs |
C           +====+====+====+====+====+====+====+====+
C           | 45 | 46 | 47 | 48 | 49 | 50 | 51 | 52 |
C           |  1 |  1 |  1 |  1 |  1 |  1 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    | _  |    |  _ |    | _  |    |  _ |
C           | bg | bg | gb | gb | tg | tg | gt | gt |
C           +====+====+====+====+====+====+====+====+
C           | 53 | 54 | 55 | 56 | 57 | 58 | 59 | 60 |
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqa
C  _____________________________________________________________________
C | Initial |    |    |    |    | __ | __ | __ | __ |    | __ |    | __ |
C |  State  | uu | dd | cc | ss | uu | dd | cc | ss | bb | bb | tt | tt |
C +=========+====+====+====+====+====+====+====+====+====+====+====+====+
C | grcflav | 61 | 62 | 63 | 64 | 65 | 66 | 67 | 68 | 69 | 70 | 71 | 72 |
C | symmetry|  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |
C +---------+----+----+----+----+----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | uc | ds | cu | sd | uc | ds | cu | sd |
C           +====+====+====+====+====+====+====+====+
C           | 73 | 74 | 75 | 76 | 77 | 78 | 79 | 80 |
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | bd | bs | db | sb | bd | bs | db | sb |
C           +====+====+====+====+====+====+====+====+
C           | 81 | 82 | 83 | 84 | 85 | 86 | 87 | 88 |
C           |  1 |  1 |  0 |  0 |  1 |  1 |  0 |  0 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | tu | tc | ut | ct | tu | tc | ut | ct |
C           +====+====+====+====+====+====+====+====+
C           | 89 | 90 | 91 | 92 | 93 | 94 | 95 | 96 |
C           |  1 |  1 |  0 |  0 |  1 |  1 |  0 |  0 |
C           +----+----+----+----+----+----+----+----+
C
C subroutine grcprtnqqs
C  _________________________________________________
C | Initial |    |    |    |    |    |    |    |    |
C |  State  | ud | us | cd | cs | du | dc | su | sc |
C +=========+====+====+====+====+====+====+====+====+
C | grcflav | 97 | 98 | 99 | 100| 101| 102| 103| 104|
C | symmetry|  0 |  0 |  0 |  0 |  1 |  1 |  1 |  1 |
C +---------+----+----+----+----+----+----+----+----+
C            _______________________________________
C           | __ | __ | __ | __ | __ | __ | __ | __ |
C           | ud | us | cd | cs | du | dc | su | sc |
C           +====+====+====+====+====+====+====+====+
C           | 105| 106| 107| 108| 109| 110| 111| 112|
C           |  0 |  0 |  0 |  0 |  1 |  1 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C            _______________________________________
C           |    |    |    |    | __ | __ | __ | __ |
C           | ub | cb | bu | bc | ub | cb | bu | bc |
C           +====+====+====+====+====+====+====+====+
C           | 113| 114| 115| 116| 117| 118| 119| 120|
C           |  0 |  0 |  1 |  1 |  0 |  0 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+
C            ___________________________________________________________
C           |    |    |    |    |    |    | __ | __ | __ | __ | __ | __ |
C           | td | ts | tb | dt | st | bt | td | ts | tb | dt | st | bt |
C           +====+====+====+====+====+====+====+====+====+====+====+====+
C           | 121| 122| 123| 124| 125| 126| 127| 128| 129| 130| 131| 132|
C           |  0 |  0 |  0 |  1 |  1 |  1 |  0 |  0 |  0 |  1 |  1 |  1 |
C           +----+----+----+----+----+----+----+----+----+----+----+----+
C
C----------------------
C Initial gluon gluon.
C----------------------
      SUBROUTINE GRCPRTNGG
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES
 
C...LHA PDF
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL
      DOUBLE PRECISION SUM
 
      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
C  QCDPS instead of a PDF function.
         call QPEVINI
         call runQCDPS( 1, 21, 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, 21, -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, 21, -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         SUM= fPDF1*fPDF2
      else
C...gluon from p, gluon from pbar
         call grcpdf1( 0, GPRI(5), GRCFAQ, GL1 )
         call grcpdf1( 0, GPRI(6), GRCFAQ, GL2 )
         SUM = GL1*GL2
C...Reject very small/large x
         IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
            GO TO 999
         ENDIF
      endif

      GRCFLAV = 0
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

      RETURN
 999  CONTINUE
      YACOB = 0.0D0
      RETURN
      END
 
C---------------------------
C Initial quark anti-quark.
C---------------------------
      SUBROUTINE GRCPRTNQQ
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES
 
C...Proton Variable
      INTEGER I
      DOUBLE PRECISION TERM(36)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)

C...Grace Variable
      INTEGER ISEL,INPA
      DOUBLE PRECISION SUM,RANSUM
      INTEGER FMAX,FMIN
      DOUBLE PRECISION RNA,GFLA(18)

      INTEGER IGFLVTMP(36)
      DATA IGFLVTMP/  1,  2,  3,  4,  5,  6,  7,  8,133,134,135,136,
     $              137,138,139,140,  9, 10,141,142,143,144,145,146,
     $              147,148, 11, 12,149,150,151,152,153,154,155,156/

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

C...Loop over quark flavours (up to b).
      INPA = 8
      TERM(1) = FLHA1(2)*FLHA2(-2*(1-2*IGPP)) ! grcflav:  1, u     u-bar
      TERM(2) = FLHA1(1)*FLHA2(-1*(1-2*IGPP)) ! grcflav:  2, d     d-bar
      TERM(3) = FLHA1(4)*FLHA2(-4*(1-2*IGPP)) ! grcflav:  3, c     c-bar
      TERM(4) = FLHA1(3)*FLHA2(-3*(1-2*IGPP)) ! grcflav:  4, s     s-bar
      TERM(5) = FLHA1(-2)*FLHA2(2*(1-2*IGPP)) ! grcflav:  5, u-bar u
      TERM(6) = FLHA1(-1)*FLHA2(1*(1-2*IGPP)) ! grcflav:  6, d-bar d
      TERM(7) = FLHA1(-4)*FLHA2(4*(1-2*IGPP)) ! grcflav:  7, c-bar c
      TERM(8) = FLHA1(-3)*FLHA2(3*(1-2*IGPP)) ! grcflav:  8, s-bar s

      IF (INPFL.GT.4) THEN
         INPA = 10
         TERM(9)  = FLHA1(5)*FLHA2(-5*(1-2*IGPP)) ! grcflav:  9, b     b-bar
         TERM(10) = FLHA1(-5)*FLHA2(5*(1-2*IGPP)) ! grcflav: 10, b-bar b
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 12
         TERM(11) = FLHA1(6)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 11, t     t-bar
         TERM(12) = FLHA1(-6)*FLHA2(6*(1-2*IGPP)) ! grcflav: 12, t-bar t
      ENDIF

      IF (IGSUB.EQ.302.OR.IGSUB.EQ.310.OR.IGSUB.EQ.319
     $     .OR.IGSUB.EQ.327.OR.IGSUB.EQ.428.OR.IGSUB.EQ.436
     $     .OR.IGSUB.EQ.444.OR.IGSUB.EQ.448.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.512.OR.IGSUB.EQ.520.OR.IGSUB.EQ.560
     $     .OR.IGSUB.EQ.567.OR.IGSUB.EQ.577) THEN
         INPA = 16
         TERM(9)  = FLHA1(2)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 133, u     c-bar
         TERM(10) = FLHA1(1)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 134, d     s-bar
         TERM(11) = FLHA1(4)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 135, c     u-bar
         TERM(12) = FLHA1(3)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 136, s     d-bar
         TERM(13) = FLHA1(-2)*FLHA2(4*(1-2*IGPP)) ! grcflav: 137, u-bar c
         TERM(14) = FLHA1(-1)*FLHA2(3*(1-2*IGPP)) ! grcflav: 138, d-bar s
         TERM(15) = FLHA1(-4)*FLHA2(2*(1-2*IGPP)) ! grcflav: 139, c-bar u
         TERM(16) = FLHA1(-3)*FLHA2(1*(1-2*IGPP)) ! grcflav: 140, s-bar d

         IF (INPFL.GT.4) THEN
            INPA = 26
            TERM(17) = FLHA1(5)*FLHA2(-5*(1-2*IGPP)) ! grcflav:   9, b    b-bar
            TERM(18) = FLHA1(-5)*FLHA2(5*(1-2*IGPP)) ! grcflav:  10, b-bar b
            TERM(19) = FLHA1(1)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 141, d    b-bar
            TERM(20) = FLHA1(3)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 142, s    b-bar
            TERM(21) = FLHA1(5)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 143, b    d-bar
            TERM(22) = FLHA1(5)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 144, b    s-bar
            TERM(23) = FLHA1(-5)*FLHA2(1*(1-2*IGPP)) ! grcflav: 145, b-bar d
            TERM(24) = FLHA1(-5)*FLHA2(3*(1-2*IGPP)) ! grcflav: 146, b-bar s
            TERM(25) = FLHA1(-1)*FLHA2(5*(1-2*IGPP)) ! grcflav: 147, d-bar b
            TERM(26) = FLHA1(-3)*FLHA2(5*(1-2*IGPP)) ! grcflav: 148, s-bar b
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 36
            TERM(27) = FLHA1(6)*FLHA2(-6*(1-2*IGPP)) ! grcflav:  11, t    t-bar
            TERM(28) = FLHA1(-6)*FLHA2(6*(1-2*IGPP)) ! grcflav:  12, t-bar t
            TERM(29) = FLHA1(2)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 149, u    t-bar
            TERM(30) = FLHA1(4)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 150, c    t-bar
            TERM(31) = FLHA1(6)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 151, t    u-bar
            TERM(32) = FLHA1(6)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 152, t    c-bar
            TERM(33) = FLHA1(-2)*FLHA2(6*(1-2*IGPP)) ! grcflav: 153, u-bar t
            TERM(34) = FLHA1(-4)*FLHA2(6*(1-2*IGPP)) ! grcflav: 154, c-bar t
            TERM(35) = FLHA1(-6)*FLHA2(2*(1-2*IGPP)) ! grcflav: 155, t-bar u
            TERM(36) = FLHA1(-6)*FLHA2(4*(1-2*IGPP)) ! grcflav: 156, t-bar c
         ENDIF
      ENDIF

      GFLA(1) = 1
      GFLA(2) = 0.0D0
      GFLA(INT(GFLA(1))+2) = 1.0D0
      GFLA(10) = 1.0D0
      GFLA(INT(GFLA(1))+10) = INPA
      DO I=1,INT(GFLA(1))
         IF (GPRI(10).GT.GFLA(I+1).AND.GPRI(10).LE.GFLA(I+2)) THEN
            RNA = (GPRI(10) - GFLA(I+1))/(GFLA(I+2) - GFLA(I+1))
            FMIN = INT(GFLA(I+9))
            FMAX = INT(GFLA(I+10))
            YACOB = YACOB/(GFLA(I+2) - GFLA(I+1))
         ENDIF
      ENDDO
 
      IF (FMIN.EQ.1) THEN
         FMIN = 0
      ENDIF
 
      SUM = 0.0D0
      DO I = FMIN+1,FMAX
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = FMIN
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL
      YACOB = SUM*YACOB

      IF (IGSUB.EQ.302.OR.IGSUB.EQ.310.OR.IGSUB.EQ.319
     $     .OR.IGSUB.EQ.327.OR.IGSUB.EQ.428.OR.IGSUB.EQ.436
     $     .OR.IGSUB.EQ.444.OR.IGSUB.EQ.448.OR.IGSUB.EQ.504
     $     .OR.IGSUB.EQ.512.OR.IGSUB.EQ.520.OR.IGSUB.EQ.560
     $     .OR.IGSUB.EQ.567.OR.IGSUB.EQ.577) THEN
         GRCFLAV = IGFLVTMP(ISEL)
      ENDIF

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif

      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END
 
C----------------------------
C Initial quark anti-quark'.
C----------------------------
      SUBROUTINE GRCPRTNQQB
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES
 
C...Proton Variable
      DOUBLE PRECISION TERM(16)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER I,ISEL,INPA,FLAGPA
      DOUBLE PRECISION SUM,XS1,XS2,XSS,RNA,RANSUM

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...Without CKM
      IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
         XS1 = 1.0D0
         XS2 = 0.0D0
C...With CKM
      ELSE
         XS1 = 0.8D0
         XS2 = 0.2D0
      ENDIF

C...For Z/gamma processes.
      IF (IGSUB.EQ.507.OR.IGSUB.EQ.515.OR.IGSUB.EQ.523
     $     .OR.IGSUB.EQ.563) THEN
         XS1 = 5.0D0
         XS2 = 2.0D0
      ENDIF

      XSS = XS1/(XS1 + XS2)

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

      IF (GPRI(10).LT.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
C...Diagonal component of CKM
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 13, u     d-bar
         TERM(2) = FLHA1(4)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 14, c     s-bar
         TERM(3) = FLHA1(-1)*FLHA2(2*(1-2*IGPP)) ! grcflav: 15, d-bar u
         TERM(4) = FLHA1(-3)*FLHA2(4*(1-2*IGPP)) ! grcflav: 16, s-bar c
         TERM(5) = FLHA1(-2)*FLHA2(1*(1-2*IGPP)) ! grcflav: 17, u-bar d
         TERM(6) = FLHA1(-4)*FLHA2(3*(1-2*IGPP)) ! grcflav: 18, c-bar s
         TERM(7) = FLHA1(1)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 19, d     u-bar
         TERM(8) = FLHA1(3)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 20, s     c-bar
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Off-Diagonal component of CKM
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 21, u     s-bar
         TERM(2) = FLHA1(4)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 22, c     d-bar
         TERM(3) = FLHA1(-3)*FLHA2(2*(1-2*IGPP)) ! grcflav: 23, s-bar u
         TERM(4) = FLHA1(-1)*FLHA2(4*(1-2*IGPP)) ! grcflav: 24, d-bar c
         TERM(5) = FLHA1(-2)*FLHA2(3*(1-2*IGPP)) ! grcflav: 25, u-bar s
         TERM(6) = FLHA1(-4)*FLHA2(1*(1-2*IGPP)) ! grcflav: 26, c-bar d
         TERM(7) = FLHA1(3)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 27, s     u-bar
         TERM(8) = FLHA1(1)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 28, d     c-bar

         IF (INPFL.EQ.5) THEN
            INPA = 16
            TERM(9)  = FLHA1(2)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 29, u     b-bar
            TERM(10) = FLHA1(4)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 30, c     b-bar
            TERM(11) = FLHA1(-5)*FLHA2(2*(1-2*IGPP)) ! grcflav: 31, b-bar u
            TERM(12) = FLHA1(-5)*FLHA2(4*(1-2*IGPP)) ! grcflav: 32, b-bar c
            TERM(13) = FLHA1(-2)*FLHA2(5*(1-2*IGPP)) ! grcflav: 33, u-bar b
            TERM(14) = FLHA1(-4)*FLHA2(5*(1-2*IGPP)) ! grcflav: 34, c-bar b
            TERM(15) = FLHA1(5)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 35, b     u-bar
            TERM(16) = FLHA1(5)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 36, b     c-bar
         ENDIF
 
         IF (INPFL.EQ.6) THEN
            WRITE(*,*) 'Error: No mixing with top'
            STOP
         ENDIF
      ENDIF

      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF

C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF
 
      IF (FLAGPA.EQ.1) THEN
         GRCFLAV = ISEL + 12
      ELSE IF (FLAGPA.EQ.2) THEN
         GRCFLAV = ISEL + 20
      ELSE
         WRITE(*,*) 'Error: strange random num. for PDF.',RNA
         GO TO 999
      ENDIF
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif

      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C----------------------
C Initial quark gluon.
C----------------------
      SUBROUTINE GRCPRTNQG
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES
 
C...Proton Variable
      INTEGER I
      DOUBLE PRECISION TERM(24)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL,INPA
      DOUBLE PRECISION SUM,RANSUM
      INTEGER FMAX,FMIN
      DOUBLE PRECISION RNA,GFLA(18)

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = FLHA1(2)*FLHA2(0)             ! grcflav: 37, u     gluon
      TERM(2)  = FLHA1(1)*FLHA2(0)             ! grcflav: 38, d     gluon
      TERM(3)  = FLHA1(4)*FLHA2(0)             ! grcflav: 39, c     gluon
      TERM(4)  = FLHA1(3)*FLHA2(0)             ! grcflav: 40, s     gluon
      TERM(5)  = FLHA1(-2)*FLHA2(0)            ! grcflav: 41, u-bar gluon
      TERM(6)  = FLHA1(-1)*FLHA2(0)            ! grcflav: 42, d-bar gluon
      TERM(7)  = FLHA1(-4)*FLHA2(0)            ! grcflav: 43, c-bar gluon
      TERM(8)  = FLHA1(-3)*FLHA2(0)            ! grcflav: 44, s-bar gluon
      TERM(9)  = FLHA1(0)*FLHA2(2*(1-2*IGPP))  ! grcflav: 45, gluon u    
      TERM(10) = FLHA1(0)*FLHA2(1*(1-2*IGPP))  ! grcflav: 46, gluon d    
      TERM(11) = FLHA1(0)*FLHA2(4*(1-2*IGPP))  ! grcflav: 47, gluon c    
      TERM(12) = FLHA1(0)*FLHA2(3*(1-2*IGPP))  ! grcflav: 48, gluon s    
      TERM(13) = FLHA1(0)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 49, gluon u-bar
      TERM(14) = FLHA1(0)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 50, gluon d-bar
      TERM(15) = FLHA1(0)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 51, gluon c-bar
      TERM(16) = FLHA1(0)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 52, gluon s-bar

      IF (INPFL.GT.4) THEN
         INPA = 20
         TERM(17) = FLHA1(5)*FLHA2(0)             ! grcflav: 53, b     gluon
         TERM(18) = FLHA1(-5)*FLHA2(0)            ! grcflav: 54, b-bar gluon
         TERM(19) = FLHA1(0)*FLHA2(5*(1-2*IGPP))  ! grcflav: 55, gluon b
         TERM(20) = FLHA1(0)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 56, gluon b-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 24
         TERM(21) = FLHA1(6)*FLHA2(0)             ! grcflav: 57, t     gluon
         TERM(22) = FLHA1(-6)*FLHA2(0)            ! grcflav: 58, t-bar gluon
         TERM(23) = FLHA1(0)*FLHA2(6*(1-2*IGPP))  ! grcflav: 59, gluon t
         TERM(24) = FLHA1(0)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 60, gluon t-bar
      ENDIF
 
      GFLA(1) = 1
      GFLA(2) = 0.0D0
      GFLA(INT(GFLA(1))+2) = 1.0D0
      GFLA(10) = 1.0D0
      GFLA(INT(GFLA(1))+10) = INPA
      DO I=1,INT(GFLA(1))
         IF (GPRI(10).GT.GFLA(I+1).AND.GPRI(10).LE.GFLA(I+2)) THEN
            RNA = (GPRI(10) - GFLA(I+1))/(GFLA(I+2) - GFLA(I+1))
            FMIN = INT(GFLA(I+9))
            FMAX = INT(GFLA(I+10))
            YACOB = YACOB/(GFLA(I+2) - GFLA(I+1))
         ENDIF
      ENDDO
 
      IF (FMIN.EQ.1) THEN
         FMIN = 0
      ENDIF
 
      SUM = 0.0D0
      DO I = FMIN+1,FMAX
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = FMIN
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL + 36
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------------
C Initial up(down)-quark up(down)-quark'.
C-----------------------------------------
      SUBROUTINE GRCPRTNQQA
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES

C...Proton Variable
      INTEGER I
      DOUBLE PRECISION TERM(24)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)

C...Grace Variable
      INTEGER ISEL,INPA
      DOUBLE PRECISION SUM,RANSUM,RNA,XS1,XS2,XSS
      INTEGER FLAGPA

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...Weight PDF
      XS1 = 0.5D0
      XS2 = 0.5D0
      XSS = XS1/(XS1 + XS2)

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

      IF (GPRI(10).LE.XSS) THEN
         FLAGPA = 1
         RNA = GPRI(10)/XSS
         YACOB = YACOB/XSS

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(2*(1-2*IGPP))   ! grcflav: 61, u    u
         TERM(2) = FLHA1(1)*FLHA2(1*(1-2*IGPP))   ! grcflav: 62, d    d
         TERM(3) = FLHA1(4)*FLHA2(4*(1-2*IGPP))   ! grcflav: 63, c    c
         TERM(4) = FLHA1(3)*FLHA2(3*(1-2*IGPP))   ! grcflav: 64, s    s
         TERM(5) = FLHA1(-2)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 65, ubar ubar
         TERM(6) = FLHA1(-1)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 66, dbar dbar
         TERM(7) = FLHA1(-4)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 67, cbar cbar
         TERM(8) = FLHA1(-3)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 68, sbar sbar

         IF (INPFL.GT.4) THEN
            INPA = 10
            TERM(9)  = FLHA1(5)*FLHA2(5*(1-2*IGPP))   ! grcflav: 69, b    b
            TERM(10) = FLHA1(-5)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 70, bbar bbar
         ENDIF
      
         IF (INPFL.EQ.6) THEN
            INPA = 12
            TERM(11) = FLHA1(6)*FLHA2(6*(1-2*IGPP))   ! grcflav: 71, t    t
            TERM(12) = FLHA1(-6)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 72, tbar tbar
         ENDIF
      ELSE
         FLAGPA = 2
         RNA = (GPRI(10) - XSS)/(1.0D0 - XSS)
         YACOB = YACOB/(1.0D0 - XSS)

C...Loop over quark flavours (up to b).
         INPA = 8
         TERM(1) = FLHA1(2)*FLHA2(4*(1-2*IGPP))    ! grcflav: 73, u    c
         TERM(2) = FLHA1(1)*FLHA2(3*(1-2*IGPP))    ! grcflav: 74, d    s
         TERM(3) = FLHA1(4)*FLHA2(2*(1-2*IGPP))    ! grcflav: 75, c    u
         TERM(4) = FLHA1(3)*FLHA2(1*(1-2*IGPP))    ! grcflav: 76, s    d
         TERM(5) = FLHA1(-2)*FLHA2(-4*(1-2*IGPP))  ! grcflav: 77, ubar cbar
         TERM(6) = FLHA1(-1)*FLHA2(-3*(1-2*IGPP))  ! grcflav: 78, dbar sbar
         TERM(7) = FLHA1(-4)*FLHA2(-2*(1-2*IGPP))  ! grcflav: 79, cbar ubar
         TERM(8) = FLHA1(-3)*FLHA2(-1*(1-2*IGPP))  ! grcflav: 80, sbar dbar

         IF (INPFL.GT.4) THEN
            INPA = 16
            TERM(9)  = FLHA1(5)*FLHA2(1*(1-2*IGPP))   ! grcflav: 81, b    d
            TERM(10) = FLHA1(5)*FLHA2(3*(1-2*IGPP))   ! grcflav: 82, b    s
            TERM(11) = FLHA1(1)*FLHA2(5*(1-2*IGPP))   ! grcflav: 83, d    b
            TERM(12) = FLHA1(3)*FLHA2(5*(1-2*IGPP))   ! grcflav: 84, s    b
            TERM(13) = FLHA1(-5)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 85, bbar dbar
            TERM(14) = FLHA1(-5)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 86, bbar sbar
            TERM(15) = FLHA1(-1)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 87, dbar bbar
            TERM(16) = FLHA1(-3)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 88, sbar bbar
         ENDIF

         IF (INPFL.EQ.6) THEN
            INPA = 24
            TERM(17) = FLHA1(6)*FLHA2(2*(1-2*IGPP))   ! grcflav: 89, t    u
            TERM(18) = FLHA1(6)*FLHA2(4*(1-2*IGPP))   ! grcflav: 90, t    c
            TERM(19) = FLHA1(2)*FLHA2(6*(1-2*IGPP))   ! grcflav: 91, u    t
            TERM(20) = FLHA1(4)*FLHA2(6*(1-2*IGPP))   ! grcflav: 92, c    t
            TERM(21) = FLHA1(-6)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 93, tbar ubar
            TERM(22) = FLHA1(-6)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 94, tbar cbar
            TERM(23) = FLHA1(-2)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 95, ubar tbar
            TERM(24) = FLHA1(-4)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 96, cbar tbar
         ENDIF
      ENDIF
 
      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      IF (FLAGPA.EQ.1) THEN
         GRCFLAV = ISEL + 60
      ELSE IF (FLAGPA.EQ.2) THEN
         GRCFLAV = ISEL + 72
      ELSE
         WRITE(*,*) 'GRC::Error: Strange random num. for PDF.',RNA
         GO TO 999
      ENDIF
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

C-----------------------------------------
C Initial up(down)-quark down(up)-quark'.
C-----------------------------------------
      SUBROUTINE GRCPRTNQQS
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES

C...Proton Variable
      INTEGER I
      DOUBLE PRECISION TERM(36)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL,INPA
      DOUBLE PRECISION SUM,RANSUM,RNA

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

C...Loop over quark flavours (up to b).
      INPA = 16
      TERM(1)  = FLHA1(2)*FLHA2(1*(1-2*IGPP))   ! grcflav:  97, u     d
      TERM(2)  = FLHA1(2)*FLHA2(3*(1-2*IGPP))   ! grcflav:  98, u     s
      TERM(3)  = FLHA1(4)*FLHA2(1*(1-2*IGPP))   ! grcflav:  99, c     d
      TERM(4)  = FLHA1(4)*FLHA2(3*(1-2*IGPP))   ! grcflav: 100, c     s
      TERM(5)  = FLHA1(1)*FLHA2(2*(1-2*IGPP))   ! grcflav: 101, d     u
      TERM(6)  = FLHA1(1)*FLHA2(4*(1-2*IGPP))   ! grcflav: 102, d     c
      TERM(7)  = FLHA1(3)*FLHA2(2*(1-2*IGPP))   ! grcflav: 103, s     u
      TERM(8)  = FLHA1(3)*FLHA2(4*(1-2*IGPP))   ! grcflav: 104, s     c
      TERM(9)  = FLHA1(-2)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 105, u-bar d-bar
      TERM(10) = FLHA1(-2)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 106, u-bar s-bar
      TERM(11) = FLHA1(-4)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 107, c-bar d-bar
      TERM(12) = FLHA1(-4)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 108, c-bar s-bar
      TERM(13) = FLHA1(-1)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 109, d-bar u-bar
      TERM(14) = FLHA1(-1)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 110, d-bar c-bar
      TERM(15) = FLHA1(-3)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 111, s-bar u-bar
      TERM(16) = FLHA1(-3)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 112, s-bar c-bar

      IF (INPFL.GT.4) THEN
         INPA = 24
         TERM(17) = FLHA1(2)*FLHA2(5*(1-2*IGPP))   ! grcflav: 113, u     b
         TERM(18) = FLHA1(4)*FLHA2(5*(1-2*IGPP))   ! grcflav: 114, c     b
         TERM(19) = FLHA1(5)*FLHA2(2*(1-2*IGPP))   ! grcflav: 115, b     u
         TERM(20) = FLHA1(5)*FLHA2(4*(1-2*IGPP))   ! grcflav: 116, b     c
         TERM(21) = FLHA1(-2)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 117, u-bar b-bar
         TERM(22) = FLHA1(-4)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 118, c-bar b-bar
         TERM(23) = FLHA1(-5)*FLHA2(-2*(1-2*IGPP)) ! grcflav: 119, b-bar u-bar
         TERM(24) = FLHA1(-5)*FLHA2(-4*(1-2*IGPP)) ! grcflav: 120, b-bar c-bar
      ENDIF

      IF (INPFL.EQ.6) THEN
         INPA = 36
         TERM(25) = FLHA1(6)*FLHA2(1*(1-2*IGPP))   ! grcflav: 121, t     d
         TERM(26) = FLHA1(6)*FLHA2(3*(1-2*IGPP))   ! grcflav: 122, t     s
         TERM(27) = FLHA1(6)*FLHA2(5*(1-2*IGPP))   ! grcflav: 123, t     b
         TERM(28) = FLHA1(1)*FLHA2(6*(1-2*IGPP))   ! grcflav: 124, d     t
         TERM(29) = FLHA1(3)*FLHA2(6*(1-2*IGPP))   ! grcflav: 125, s     t
         TERM(30) = FLHA1(5)*FLHA2(6*(1-2*IGPP))   ! grcflav: 126, b     t
         TERM(31) = FLHA1(-6)*FLHA2(-1*(1-2*IGPP)) ! grcflav: 127, t-bar d-bar
         TERM(32) = FLHA1(-6)*FLHA2(-3*(1-2*IGPP)) ! grcflav: 128, t-bar s-bar
         TERM(33) = FLHA1(-6)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 129, t-bar b-bar
         TERM(34) = FLHA1(-1)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 130, d-bar t-bar
         TERM(35) = FLHA1(-3)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 131, s-bar t-bar
         TERM(36) = FLHA1(-5)*FLHA2(-6*(1-2*IGPP)) ! grcflav: 132, b-bar t-bar
      ENDIF
 
      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.EQ.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RNA = GPRI(10)
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL + 96
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END


C----------------------------
C Initial bottom-quark gluon
C----------------------------
      SUBROUTINE GRCPRTNBG
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      include 'inclk.h'

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES

C...Proton Variable
      INTEGER I
      DOUBLE PRECISION TERM(4)

C...Use the code originally written for LHAPDF.
      DOUBLE PRECISION FLHA1(-6:6),FLHA2(-6:6)
 
C...Grace Variable
      INTEGER ISEL,INPA
      DOUBLE PRECISION SUM,RANSUM,RNA

      IF (GPRI(5).GT.1.OR.GPRI(6).GT.1) THEN
         IREJ = 1
         GO TO 999
      ENDIF

      IF (INPFL.NE.5) THEN
         WRITE(*,*) 'GRC::Error Wrong Nr. of flavor for PDF.'
         WRITE(*,*) '           Set INPFL=5'
         STOP
      ENDIF

C...PDF
      call grcpdf( GPRI(5), GRCFAQ, FLHA1 )
      call grcpdf( GPRI(6), GRCFAQ, FLHA2 )

C...Loop over quark flavours (up to b).
      INPA = 4
      TERM(1) = FLHA1(5)*FLHA2(0)             ! grcflav: 53, b     gluon
      TERM(2) = FLHA1(-5)*FLHA2(0)            ! grcflav: 54, b-bar gluon
      TERM(3) = FLHA1(0)*FLHA2(5*(1-2*IGPP))  ! grcflav: 55, gluon b
      TERM(4) = FLHA1(0)*FLHA2(-5*(1-2*IGPP)) ! grcflav: 56, gluon b-bar

      SUM = 0.0D0
      DO I = 1,INPA
         SUM = SUM + TERM(I)
      ENDDO

C...Reject very small/large x
      IF (SUM.LE.0.0D0) THEN
C         write(*,*) 'reject',sum,GPRI(5),GPRI(6),GRCFAQ
         GO TO 999
      ENDIF
 
C...Select one of the possible terms at random.
      RNA = GPRI(10)
      RANSUM = RNA*SUM
      ISEL = 0
      DO WHILE (RANSUM.GT.0.0D0)
         ISEL = ISEL + 1
         RANSUM = RANSUM - TERM(ISEL)
      ENDDO

      IF (ISEL.EQ.0) THEN
c         WRITE(*,*) 'GRC::Error Mismatch intial flavor, ISEL=0 .'
         GO TO 999
      ENDIF

      GRCFLAV = ISEL + 52
      YACOB = SUM*YACOB

C...Set flavor
      CALL GRCFFIL

C  QCDPS
      if( ishower.eq.1 .and. 
C     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 ) ) then
     >     ( mod(JFLAG,2).eq.1 .or. IBASES.eq.0 
     >     .or. labcut.eq.1 ) ) then
         YACOB= YACOB/TERM(ISEL)
         call QPEVINI
         call runQCDPS( 1, kfcode(1), 1, GRCFAQ, GPRI(5), fPDF1 )
         if( fPDF1.le.0.d0 ) goto 999
         if( IGPP.eq.0 ) then
            call runQCDPS( 1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         else
            call runQCDPS( -1, kfcode(2), -1, GRCFAQ, GPRI(6), fPDF2 )
         endif
         if( fPDF2.le.0.d0 ) goto 999
         YACOB= YACOB * fPDF1*fPDF2
      endif
 
      RETURN
 999  CONTINUE
      IREJ = 2
      YACOB = 0.0D0
      RETURN
      END

      subroutine grcpdf( xxx, scale, xpdf )
C  Return the PDF value of the "proton" in an array.
C  (input) xxx: x value (from 0 up to 1)
C          scale: energy scale in GeV
C  (output) xpdf(i) = x * pdf for the parton "i"
C               "i" is the PDG code except for the gluon.
C               i = 0 for gluon.
C  Feb. 22, 2008, S. Odaka
      implicit real*8(a-h,o-z)
      real*8 xpdf(-6:6)

#include "define.h"
#if PDFLIB == 1
      real*8 xpp(-25:25)
#endif

      do i=-6,6
         xpdf(i)= 0.d0
      enddo

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      call PYPDFU( 2212, xxx, scale**2, xpp )
      do i=-6,6
         xpdf(i)= xpp(i)
      enddo

#elif PDFLIB == 2
C...PDFLIB in CERNLIB
      CALL STRUCTM( xxx, scale,
     >     upv, dnv, ups, dns, str, chm, btm, top, glu )
      xpdf(-6)= top
      xpdf(-5)= btm
      xpdf(-4)= chm
      xpdf(-3)= str
      xpdf(-2)= ups
      xpdf(-1)= dns
      xpdf(0)= glu
      xpdf(1)= dnv +dns
      xpdf(2)= upv +ups
      xpdf(3)= str
      xpdf(4)= chm
      xpdf(5)= btm
      xpdf(6)= top

#elif PDFLIB == 3
C...LHAPDF
      CALL evolvePDF( xxx, scale, xpdf )

#elif PDFLIB == 4
C...CTEQ6
      xpdf(-5)= xxx*Ctq6Pdf( -5, xxx, scale )
      xpdf(-4)= xxx*Ctq6Pdf( -4, xxx, scale )
      xpdf(-3)= xxx*Ctq6Pdf( -3, xxx, scale )
      xpdf(-2)= xxx*Ctq6Pdf( -1, xxx, scale )
      xpdf(-1)= xxx*Ctq6Pdf( -2, xxx, scale )
      xpdf(0)= xxx*Ctq6Pdf( 0, xxx, scale )
      xpdf(1)= xxx*Ctq6Pdf( 2, xxx, scale )
      xpdf(2)= xxx*Ctq6Pdf( 1, xxx, scale )
      xpdf(3)= xxx*Ctq6Pdf( 3, xxx, scale )
      xpdf(4)= xxx*Ctq6Pdf( 4, xxx, scale )
      xpdf(5)= xxx*Ctq6Pdf( 5, xxx, scale )
#endif

      return
      end

      subroutine grcpdf1( ipart, xxx, scale, xpdf1 )
C  Return the PDF value of the "proton" for a given parton.
C  (input) ipart: parton ID in the PDG code
C                 ipart = 0 can be used for gluon.
C                 Of course, ipart = 21 is also valid.
C          xxx: x value (from 0 up to 1)
C          scale: energy scale in GeV
C  (output) xpdf1 = x * pdf for the parton "ipart"
C  Feb. 22, 2008, S. Odaka
      implicit real*8(a-h,o-z)

#include "define.h"
#if PDFLIB == 1
      real*8 xpp(-25:25)
#elif PDFLIB == 3
      real*8 xpdf(-6:6)
#endif

      xpdf1= 0.d0
      if( iabs( ipart ).gt.6 .and. ipart.ne.21 ) return

#if PDFLIB == 1
C...Internal PDF from PYTHIA
      call PYPDFU( 2212, xxx, scale**2, xpp )
      xpdf1= xpp(ipart)

#elif PDFLIB == 2
C...PDFLIB in CERNLIB
      CALL STRUCTM( xxx, scale,
     >     upv, dnv, ups, dns, str, chm, btm, top, glu )
      ipabs= iabs( ipart )
      if( ipart.eq.0 .or. ipart.eq.21 ) then
         xpdf1= glu
      elseif( ipart.eq.2 ) then
         xpdf1= upv +ups
      elseif( ipart.eq.1 ) then
         xpdf1= dnv +dns
      elseif( ipart.eq.-2 ) then
         xpdf1= ups
      elseif( ipart.eq.-1 ) then
         xpdf1= dns
      elseif( ipabs.eq.3 ) then
         xpdf1= str
      elseif( ipabs.eq.4 ) then
         xpdf1= chm
      elseif( ipabs.eq.5 ) then
         xpdf1= btm
      elseif( ipabs.eq.6 ) then
         xpdf1= top
      endif

#elif PDFLIB == 3
C...LHAPDF
      CALL evolvePDF( xxx, scale, xpdf )
      if( iabs( ipart ).le.6 ) then
         xpdf1= xpdf(ipart)
      elseif( ipart.eq.21 ) then
         xpdf1= xpdf(0)
      endif

#elif PDFLIB == 4
C...CTEQ6
      if( ipart.eq.0 .or. ipart.eq.21 ) then
         xpdf1= xxx*Ctq6Pdf( 0, xxx, scale )
      elseif( ipart.eq.2 ) then
         xpdf1= xxx*Ctq6Pdf( 1, xxx, scale )
      elseif( ipart.eq.1 ) then
         xpdf1= xxx*Ctq6Pdf( 2, xxx, scale )
      elseif( ipart.eq.-2 ) then
         xpdf1= xxx*Ctq6Pdf( -1, xxx, scale )
      elseif( ipart.eq.-1 ) then
         xpdf1= xxx*Ctq6Pdf( -2, xxx, scale )
      elseif( iabs( ipart ).ge.3 .and. iabs( ipart ).le.5 ) then
         xpdf1= xxx*Ctq6Pdf( ipart, xxx, scale )
      endif
#endif

      return
      end

C--------------------------------------
C Set flavors.
C Set kinematics of initial particles.
C--------------------------------------
      SUBROUTINE GRCFFIL
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,J,ID1,ID2,K3,K4,K5,K6
      INTEGER IDCYST,IJBR,IDJBR1(30),IDJBR2(30),IDJBR3(30)
      INTEGER JOUT(4),KOUT(4)
      DOUBLE PRECISION E1,E2,TMPCKMR(30)
      DOUBLE PRECISION AJACOB,RNA,TMPMAS,RANSUM
      DOUBLE PRECISION Q2

      DOUBLE PRECISION GWIDTMP

      INTEGER IKFGP(157),IKFGA(157)
      INTEGER IGENE(6)

C...large matrix for flavor combination.
      DATA IKFGP/21, 2, 1, 4, 3,-2,-1,-4,-3, 5,-5, 6,-6, 2, 4,-1,
     $           -3,-2,-4, 1, 3, 2, 4,-3,-1,-2,-4, 3, 1, 2, 4,-5,
     $           -5,-2,-4, 5, 5, 2, 1, 4, 3,-2,-1,-4,-3,21,21,21,
     $           21,21,21,21,21, 5,-5,21,21, 6,-6,21,21, 2, 1, 4,
     $            3,-2,-1,-4,-3, 5,-5, 6,-6, 2, 1, 4, 3,-2,-1,-4,
     $           -3, 5, 5, 1, 3,-5,-5,-1,-3, 6, 6, 2, 4,-6,-6,-2,
     $           -4, 2, 2, 4, 4, 1, 1, 3, 3,-2,-2,-4,-4,-1,-1,-3,
     $           -3, 2, 4, 5, 5,-2,-4,-5,-5, 6, 6, 6, 1, 3, 5,-6,
     $           -6,-6,-1,-3,-5, 2, 1, 4, 3,-2,-1,-4,-3, 1, 3, 5,
     $            5,-5,-5,-1,-3, 2, 4, 6, 6,-2,-4,-6,-6/
      DATA IKFGA/21,-2,-1,-4,-3, 2, 1, 4, 3,-5, 5,-6, 6,-1,-3, 2, 
     $            4, 1, 3,-2,-4,-3,-1, 2, 4, 3, 1,-2,-4,-5,-5, 2, 
     $            4, 5, 5,-2,-4,21,21,21,21,21,21,21,21, 2, 1, 4, 
     $            3,-2,-1,-4,-3,21,21, 5,-5,21,21, 6,-6, 2, 1, 4,
     $            3,-2,-1,-4,-3, 5,-5, 6,-6, 4, 3, 2, 1,-4,-3,-2,
     $           -1, 1, 3, 5, 5,-1,-3,-5,-5, 2, 4, 6, 6,-2,-4,-6,
     $           -6, 1, 3, 1, 3, 2, 4, 2, 4,-1,-3,-1,-3,-2,-4,-2,
     $           -4, 5, 5, 2, 4,-5,-5,-2,-4, 1, 3, 5, 6, 6, 6,-1,
     $           -3,-5,-6,-6,-6,-4,-3,-2,-1, 4, 3, 2, 1,-5,-5,-1,
     $           -3, 1, 3, 5, 5,-6,-6,-2,-4, 6, 6, 2, 4/

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Masses, Charge*3 and KFcode of initial state particles
      KFCODE(1) = IKFGP(GRCFLAV+1)
      KFCODE(2) = IKFGA(GRCFLAV+1)
      DO I = 1,2
         IF (KFCODE(I).EQ.21) THEN
            KCHARG(I) = 0
         ELSE IF (KFCODE(I).GT.0
     $           .AND.MOD(IABS(KFCODE(I)),2).EQ.0) THEN
            KCHARG(I) = 2
         ELSE IF (KFCODE(I).GT.0
     $           .AND.MOD(IABS(KFCODE(I)),2).EQ.1) THEN
            KCHARG(I) = -1
         ELSE IF (KFCODE(I).LT.0
     $           .AND.MOD(IABS(KFCODE(I)),2).EQ.0) THEN
            KCHARG(I) = -2
         ELSE IF (KFCODE(I).LT.0
     $           .AND.MOD(IABS(KFCODE(I)),2).EQ.1) THEN
            KCHARG(I) = 1
         ELSE
            WRITE(*,*) 'GRC::Error: Strange flavor in KFCODE(',I,').'
         ENDIF
      ENDDO
 
      IF (GRCFLAV.EQ.0) THEN
         AMASS1(1) = AMG
         AMASS1(2) = AMG
      ELSE IF (GRCFLAV.LT.13) THEN
         AMASS1(1) = GAMS(IABS(KFCODE(1)))
         AMASS1(2) = GAMS(IABS(KFCODE(2)))
         AMUQ(1) = GAMS(IABS(KFCODE(1)))
      ELSE IF (GRCFLAV.LT.37) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            AMUQ(1) = AMUQ(IGENE(IABS(KFCODE(1))))
            AMDQ(1) = AMDQ(IGENE(IABS(KFCODE(2))))
         ELSE
            AMUQ(1) = AMUQ(IGENE(IABS(KFCODE(2))))
            AMDQ(1) = AMDQ(IGENE(IABS(KFCODE(1))))
         ENDIF
         AMASS1(1) = GAMS(IABS(KFCODE(1)))
         AMASS1(2) = GAMS(IABS(KFCODE(2)))
      ELSE IF (GRCFLAV.LT.61) THEN
         IF (KFCODE(1).EQ.21) THEN
            AMASS1(1) = AMG
            AMASS1(2) = GAMS(IABS(KFCODE(2)))
            AMUQ(1) = AMASS1(2)
         ELSE
            AMASS1(1) = GAMS(IABS(KFCODE(1)))
            AMASS1(2) = AMG
            AMUQ(1) = AMASS1(1)
         ENDIF
         AMDQ(1) = GAMS(IABS(KFCODE(5)))
      ELSE IF (GRCFLAV.LT.157) THEN
         AMASS1(1) = GAMS(IABS(KFCODE(1)))
         AMASS1(2) = GAMS(IABS(KFCODE(2)))
      ENDIF
      AMASS2(1) = AMASS1(1)**2
      AMASS2(2) = AMASS1(2)**2
 
C...Set Initial Energy and Momentum
      E1 = (GPRI(3)+AMASS2(1)-AMASS2(2))/2.0D0/GPRI(4)
      E2 = (GPRI(3)+AMASS2(2)-AMASS2(1))/2.0D0/GPRI(4)

      IF (E1.LT.AMASS1(1).OR.E2.LT.AMASS1(2)) GO TO 999
      IF (0.5D0*GPRI(5)*GRCECM.LT.AMASS1(1))  GO TO 999
      IF (0.5D0*GPRI(6)*GRCECM.LT.AMASS1(2))  GO TO 999

      IF (IGPCOMB(GRCFLAV+1).EQ.1) THEN
         TMPMAS = AMASS1(1)
         AMASS1(1) = AMASS1(2)
         AMASS1(2) = TMPMAS
         AMASS2(1) = AMASS1(1)**2
         AMASS2(2) = AMASS1(2)**2
      ENDIF

C######################
C top pair production.
C######################
      IF (IGSUB.EQ.408.OR.IGSUB.EQ.409.OR.IGSUB.EQ.410
     $     .OR.IGSUB.EQ.411.OR.IGSUB.EQ.412) THEN
C#############
C tt + 0 jet.
C#############
         IF (IGSUB.EQ.408.OR.IGSUB.EQ.409) THEN
C...Set flavor for WW decay.
            RNA = XRUN(17)
            CALL GRCWWDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(IPX(3)) = KOUT(1)
            KFCODE(IPX(4)) = KOUT(2)
            KFCODE(IPX(5)) = KOUT(3)
            KFCODE(IPX(6)) = KOUT(4)
            AMASS1(IPX(3)) = GAMS(IABS(KFCODE(IPX(3))))
            AMASS1(IPX(4)) = GAMS(IABS(KFCODE(IPX(4))))
            AMASS1(IPX(5)) = GAMS(IABS(KFCODE(IPX(5))))
            AMASS1(IPX(6)) = GAMS(IABS(KFCODE(IPX(6))))
            AMASS2(IPX(3)) = AMASS1(IPX(3))**2
            AMASS2(IPX(4)) = AMASS1(IPX(4))**2
            AMASS2(IPX(5)) = AMASS1(IPX(5))**2
            AMASS2(IPX(6)) = AMASS1(IPX(6))**2
         ENDIF

C#############
C tt + 0 jet.
C#############
         IF (IGSUB.EQ.410.OR.IGSUB.EQ.411.OR.IGSUB.EQ.412) THEN
C...Set flavor for WW decay.
            RNA = XRUN(21)
            CALL GRCWWDCY(3,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(IPX(3)) = KOUT(1)
            KFCODE(IPX(4)) = KOUT(2)
            KFCODE(IPX(5)) = KOUT(3)
            KFCODE(IPX(6)) = KOUT(4)
            AMASS1(IPX(3)) = GAMS(IABS(KFCODE(IPX(3))))
            AMASS1(IPX(4)) = GAMS(IABS(KFCODE(IPX(4))))
            AMASS1(IPX(5)) = GAMS(IABS(KFCODE(IPX(5))))
            AMASS1(IPX(6)) = GAMS(IABS(KFCODE(IPX(6))))
            AMASS2(IPX(3)) = AMASS1(IPX(3))**2
            AMASS2(IPX(4)) = AMASS1(IPX(4))**2
            AMASS2(IPX(5)) = AMASS1(IPX(5))**2
            AMASS2(IPX(6)) = AMASS1(IPX(6))**2

            IF (IGSUB.EQ.412) THEN
               KFCODE(9) = MIN(KFCODE(1),KFCODE(2))
               AMASS1(9) = GAMS(IABS(KFCODE(9)))
               AMASS2(9) = AMASS1(9)**2
            ENDIF
         ENDIF
      ENDIF

C...QCD jets.
      IF (IGSUB.GT.300.AND.IGSUB.LT.332) THEN
C#########
C 2 jets.
C#########
         IF (IGSUB.GT.300.AND.IGSUB.LT.309) THEN
C...Set jet flavor.
            RNA = XRUN(6)
            Q2 = GPRI(3)
            CALL GRCQJET(0,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = JOUT(1)
            KFCODE(4) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(3) = 21
            IF (JOUT(2).EQ.9) KFCODE(4) = 21
            AMASS1(3) = GAMS(IABS(JOUT(1)))
            AMASS1(4) = GAMS(IABS(JOUT(2)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#########
C 3 jets.
C#########
         ELSE IF (IGSUB.GT.308.AND.IGSUB.LT.318) THEN
C...Set jet flavor.
            RNA = XRUN(9)
            Q2 = GPRI(3)
            CALL GRCQJET(0,3,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = JOUT(1)
            KFCODE(4) = JOUT(2)
            KFCODE(5) = JOUT(3)
            IF (JOUT(1).EQ.9) KFCODE(3) = 21
            IF (JOUT(2).EQ.9) KFCODE(4) = 21
            IF (JOUT(3).EQ.9) KFCODE(5) = 21
            AMASS1(3) = GAMS(IABS(JOUT(1)))
            AMASS1(4) = GAMS(IABS(JOUT(2)))
            AMASS1(5) = GAMS(IABS(JOUT(3)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
C#########
C 4 jets.
C#########
         ELSE IF (IGSUB.GT.317.AND.IGSUB.LT.332) THEN
C...Set jet flavor.
            RNA = XRUN(12)
            Q2 =  GPRI(3)
            CALL GRCQJET(0,4,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = JOUT(1)
            KFCODE(4) = JOUT(2)
            KFCODE(5) = JOUT(3)
            KFCODE(6) = JOUT(4)
            IF (JOUT(1).EQ.9) KFCODE(3) = 21
            IF (JOUT(2).EQ.9) KFCODE(4) = 21
            IF (JOUT(3).EQ.9) KFCODE(5) = 21
            IF (JOUT(4).EQ.9) KFCODE(6) = 21
            AMASS1(3) = GAMS(IABS(JOUT(1)))
            AMASS1(4) = GAMS(IABS(JOUT(2)))
            AMASS1(5) = GAMS(IABS(JOUT(3)))
            AMASS1(6) = GAMS(IABS(JOUT(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
         ENDIF

         IF (IGSUB.EQ.302.OR.IGSUB.EQ.310
     $        .OR.IGSUB.EQ.319.OR.IGSUB.EQ.327) THEN
            AMASS1(1) = GAMS(IABS(KFCODE(1)))
            AMASS1(2) = GAMS(IABS(KFCODE(2)))
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C###########
C W + jets.
C###########
      IF (IGSUB.GT.420.AND.IGSUB.LT.452) THEN
C############
C W + 0 jet.
C############
         IF (IGSUB.EQ.421) THEN
            IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
               ID1 = IABS(KFCODE(1))
               ID2 = IABS(KFCODE(2))
            ELSE
               ID1 = IABS(KFCODE(2))
               ID2 = IABS(KFCODE(1))
            ENDIF
            YACOB = YACOB*GRCCKM(IGENE(ID1),IGENE(ID2))

C...Set flavor for W decay.
            RNA = XRUN(6)
            CALL GRCWDCY(0,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C############
C W + 1 jet.
C############
         ELSE IF (IGSUB.GT.421.AND.IGSUB.LT.424) THEN
C...Set jet flavor.
            RNA = XRUN(10)
            Q2 = ((GPRI(3)-GVAL(3))/2.0D0/GPRI(4))**2
            CALL GRCWJET(1,1,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS2(5) = AMASS1(5)**2

C...Set flavor for W decay.
            RNA = XRUN(9)
            CALL GRCWDCY(1,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C W + 2 jets.
C#############
         ELSE IF (IGSUB.GT.423.AND.IGSUB.LT.431) THEN
C...Set jet flavor.
            RNA = XRUN(13)
            Q2 = GVAL(4)
            CALL GRCWJET(1,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2

C...Set flavor for W decay.
            RNA = XRUN(12)
            CALL GRCWDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C W + 3 jets.
C#############
         ELSE IF (IGSUB.GT.430.AND.IGSUB.LT.439) THEN
C...Set jet flavor.
            RNA = XRUN(16)
            Q2 = GVAL(5)
            CALL GRCWJET(1,3,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            KFCODE(7) = JOUT(3)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            IF (JOUT(3).EQ.9) KFCODE(7) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS1(7) = GAMS(IABS(JOUT(3)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
            AMASS2(7) = AMASS1(7)**2

C...Set flavor for W decay.
            RNA = XRUN(15)
            CALL GRCWDCY(3,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C W + 4 jets.
C#############
         ELSE IF (IGSUB.GT.438.AND.IGSUB.LT.452) THEN
C...Set jet flavor.
            RNA = XRUN(19)
            Q2 =  GPRI(3)-GVAL(9)
            CALL GRCWJET(1,4,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            KFCODE(7) = JOUT(3)
            KFCODE(8) = JOUT(4)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            IF (JOUT(3).EQ.9) KFCODE(7) = 21
            IF (JOUT(4).EQ.9) KFCODE(8) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS1(7) = GAMS(IABS(JOUT(3)))
            AMASS1(8) = GAMS(IABS(JOUT(4)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Set flavor for W decay.
            RNA = XRUN(18)
            CALL GRCWDCY(4,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
         ENDIF

         IF (IGSUB.EQ.428.OR.IGSUB.EQ.436.OR.IGSUB.EQ.444
     $        .OR.IGSUB.EQ.448) THEN
            IF (GRCFLAV.GT.132) THEN
               AMASS1(1) = GAMS(IABS(KFCODE(1)))
               AMASS1(2) = GAMS(IABS(KFCODE(2)))
               IF (KFCODE(1).LT.0) THEN
                  TMPMAS = AMASS1(1)
                  AMASS1(1) = AMASS1(2)
                  AMASS1(2) = TMPMAS
               ENDIF
            ENDIF
            IF (KFCODE(3).GT.0) THEN
               TMPMAS = AMASS1(1)
               AMASS1(1) = AMASS1(2)
               AMASS1(2) = TMPMAS
            ENDIF
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...Z + jets.
      IF (IGSUB.GT.499.AND.IGSUB.LT.526) THEN
C############
C Z + 0 jet.
C############
         IF (IGSUB.EQ.500) THEN
C...Set flavor for Z decay.
            RNA = XRUN(6)
            CALL GRCZDCY(0,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C############
C Z + 1 jet.
C############
         ELSE IF (IGSUB.GT.500.AND.IGSUB.LT.503) THEN
C...Set jet flavor.
            IF (MAX(KFCODE(1),KFCODE(2)).EQ.21) THEN
               KFCODE(5) = MIN(KFCODE(1),KFCODE(2))
               AMASS1(5) = GAMS(IABS(KFCODE(5)))
            ELSE
               KFCODE(5) = 21
               AMASS1(5) = 0.0D0
            ENDIF
            AMASS2(5) = AMASS1(5)**2

C...Set flavor for Z decay.
            RNA = XRUN(9)
            CALL GRCZDCY(1,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C Z + 2 jets.
C#############
         ELSE IF (IGSUB.GT.502.AND.IGSUB.LT.510) THEN
C...Set jet flavor.
            RNA = XRUN(13)
            Q2 = GVAL(4)
            CALL GRCQJET(1,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2

C...Set flavor for Z decay.
            RNA = XRUN(12)
            CALL GRCZDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C Z + 3 jets.
C#############
         ELSE IF (IGSUB.GT.509.AND.IGSUB.LT.518) THEN
C...Set jet flavor.
            RNA = XRUN(16)
            Q2 = GVAL(5)
            CALL GRCQJET(1,3,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            KFCODE(7) = JOUT(3)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            IF (JOUT(3).EQ.9) KFCODE(7) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS1(7) = GAMS(IABS(JOUT(3)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
            AMASS2(7) = AMASS1(7)**2

C...Set flavor for Z decay.
            RNA = XRUN(15)
            CALL GRCZDCY(3,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
C#############
C Z + 4 jets.
C#############
         ELSE IF (IGSUB.GT.517.AND.IGSUB.LT.526) THEN
C...Set jet flavor.
            RNA = XRUN(19)
            Q2 =  GPRI(3)-GVAL(9)
            CALL GRCQJET(1,4,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(5) = JOUT(1)
            KFCODE(6) = JOUT(2)
            KFCODE(7) = JOUT(3)
            KFCODE(8) = JOUT(4)
            IF (JOUT(1).EQ.9) KFCODE(5) = 21
            IF (JOUT(2).EQ.9) KFCODE(6) = 21
            IF (JOUT(3).EQ.9) KFCODE(7) = 21
            IF (JOUT(4).EQ.9) KFCODE(8) = 21
            AMASS1(5) = GAMS(IABS(JOUT(1)))
            AMASS1(6) = GAMS(IABS(JOUT(2)))
            AMASS1(7) = GAMS(IABS(JOUT(3)))
            AMASS1(8) = GAMS(IABS(JOUT(4)))
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Set flavor for Z decay.
            RNA = XRUN(18)
            CALL GRCZDCY(4,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
         ENDIF

         IF (IGSUB.EQ.504.OR.IGSUB.EQ.512.OR.IGSUB.EQ.520) THEN
            AMASS1(1) = GAMS(IABS(KFCODE(1)))
            AMASS1(2) = GAMS(IABS(KFCODE(2)))
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...ZZ + jets.
      IF (IGSUB.EQ.550.OR.IGSUB.EQ.553.OR.IGSUB.EQ.554
     $     .OR.IGSUB.EQ.559.OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $     .OR.IGSUB.EQ.562.OR.IGSUB.EQ.563.OR.IGSUB.EQ.564
     $     .OR.IGSUB.EQ.565) THEN
C   Reduction for the identical-particle pair production.
C   The main processes for the ME calculation do not have such reduction 
C   factor because the processes are defined by taking into account Z decays, 
C   where one of the Zs decays to e+e- and the other to mu+mu-.
C   However, the reduction has to be taken into account once the processes 
C   are considered as ZZ productions where the second Z may also decays 
C   to e+e-.
         YACOB= YACOB/2.d0
C#############
C ZZ + 0 jet.
C#############
         IF (IGSUB.EQ.550) THEN
C...Set flavor for Z decay.
            RNA = XRUN(12)
            CALL GRCZZDCY(0,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C#############
C ZZ + 1 jet.
C#############
         ELSE IF (IGSUB.GT.552.AND.IGSUB.LT.555) THEN
C...Set jet flavor.
            IF (MAX(KFCODE(1),KFCODE(2)).EQ.21) THEN
               KFCODE(7) = MIN(KFCODE(1),KFCODE(2))
               AMASS1(7) = GAMS(IABS(KFCODE(7)))
            ELSE
               KFCODE(7) = 21
               AMASS1(7) = 0.0D0
            ENDIF
            AMASS2(7) = AMASS1(7)**2

C...Set flavor for ZZ decay.
            RNA = XRUN(15)
            CALL GRCZZDCY(1,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C##############
C ZZ + 2 jets.
C##############
         ELSE IF (IGSUB.GT.558.AND.IGSUB.LT.566) THEN
C...Set jet flavor.
            RNA = XRUN(19)
            Q2 =  GPRI(3)-GVAL(9)-GVAL(10)
            CALL GRCQJET(2,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(7) = JOUT(1)
            KFCODE(8) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(7) = 21
            IF (JOUT(2).EQ.9) KFCODE(8) = 21
            AMASS1(7) = GAMS(IABS(JOUT(1)))
            AMASS1(8) = GAMS(IABS(JOUT(2)))
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Set flavor for ZZ decay.
            RNA = XRUN(18)
            CALL GRCZZDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
         ENDIF

         IF (IGSUB.EQ.560) THEN
            AMASS1(1) = GAMS(IABS(KFCODE(1)))
            AMASS1(2) = GAMS(IABS(KFCODE(2)))
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...WW + jets.
      IF (IGSUB.EQ.551.OR.IGSUB.EQ.555.OR.IGSUB.EQ.556
     $     .OR.IGSUB.EQ.566.OR.IGSUB.EQ.567.OR.IGSUB.EQ.568
     $     .OR.IGSUB.EQ.569.OR.IGSUB.EQ.570.OR.IGSUB.EQ.571
     $     .OR.IGSUB.EQ.572) THEN
C#############
C WW + 0 jet.
C#############
         IF (IGSUB.EQ.551) THEN
C...Set flavor for WW decay.
            RNA = XRUN(12)
            CALL GRCWWDCY(0,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C#############
C WW + 1 jet.
C#############
         ELSE IF (IGSUB.GT.554.AND.IGSUB.LT.557) THEN
C...Set jet flavor.
            RNA = XRUN(16)
            Q2 = GVAL(5)
            CALL GRCWWJT(2,1,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(7) = JOUT(1)
            IF (JOUT(1).EQ.9) KFCODE(7) = 21
            AMASS1(7) = GAMS(IABS(JOUT(1)))
            AMASS2(7) = AMASS1(7)**2

C...Set flavor for WW decay.
            RNA = XRUN(15)
            CALL GRCWWDCY(1,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C##############
C WW + 2 jets.
C##############
         ELSE IF (IGSUB.GT.565.AND.IGSUB.LT.573) THEN
C...Set jet flavor.
            RNA = XRUN(19)
            Q2 =  GPRI(3)-GVAL(9)-GVAL(10)
            CALL GRCWWJT(2,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(7) = JOUT(1)
            KFCODE(8) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(7) = 21
            IF (JOUT(2).EQ.9) KFCODE(8) = 21
            AMASS1(7) = GAMS(IABS(JOUT(1)))
            AMASS1(8) = GAMS(IABS(JOUT(2)))
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Set flavor for WW decay.
            RNA = XRUN(18)
            CALL GRCWWDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
         ENDIF

         IF (IGSUB.EQ.567) THEN
            IF (GRCFLAV.GT.132) THEN
               AMASS1(1) = GAMS(IABS(KFCODE(1)))
               AMASS1(2) = GAMS(IABS(KFCODE(2)))
               IF (KFCODE(1).LT.0) THEN
                  TMPMAS = AMASS1(1)
                  AMASS1(1) = AMASS1(2)
                  AMASS1(2) = TMPMAS
               ENDIF
            ENDIF
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...ZW + jets.
      IF (IGSUB.EQ.552.OR.IGSUB.EQ.557.OR.IGSUB.EQ.558
     $     .OR.IGSUB.EQ.573.OR.IGSUB.EQ.574.OR.IGSUB.EQ.575
     $     .OR.IGSUB.EQ.576.OR.IGSUB.EQ.577.OR.IGSUB.EQ.578
     $     .OR.IGSUB.EQ.579) THEN
C#############
C ZW + 0 jet.
C#############
         IF (IGSUB.EQ.552) THEN
            IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
               ID1 = IABS(KFCODE(1))
               ID2 = IABS(KFCODE(2))
            ELSE
               ID1 = IABS(KFCODE(2))
               ID2 = IABS(KFCODE(1))
            ENDIF
            YACOB = YACOB*GRCCKM(IGENE(ID1),IGENE(ID2))

C...Set flavor for ZW decay.
            RNA = XRUN(12)
            CALL GRCZWDCY(0,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C#############
C ZW + 1 jet.
C#############
         ELSE IF (IGSUB.GT.556.AND.IGSUB.LT.559) THEN
C...Set jet flavor.
            RNA = XRUN(16)
            Q2 = GVAL(5)
            CALL GRCWJET(2,1,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(7) = JOUT(1)
            IF (JOUT(1).EQ.9) KFCODE(7) = 21
            AMASS1(7) = GAMS(IABS(JOUT(1)))
            AMASS2(7) = AMASS1(7)**2

C...Set flavor for ZW decay.
            RNA = XRUN(15)
            CALL GRCZWDCY(1,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
C##############
C ZW + 2 jets.
C##############
         ELSE IF (IGSUB.GT.572.AND.IGSUB.LT.580) THEN
C...Set jet flavor.
            RNA = XRUN(19)
            Q2 =  GPRI(3)-GVAL(9)-GVAL(10)
            CALL GRCWJET(2,2,JOUT,RNA,Q2,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(7) = JOUT(1)
            KFCODE(8) = JOUT(2)
            IF (JOUT(1).EQ.9) KFCODE(7) = 21
            IF (JOUT(2).EQ.9) KFCODE(8) = 21
            AMASS1(7) = GAMS(IABS(JOUT(1)))
            AMASS1(8) = GAMS(IABS(JOUT(2)))
            AMASS2(7) = AMASS1(7)**2
            AMASS2(8) = AMASS1(8)**2

C...Set flavor for ZW decay.
            RNA = XRUN(18)
            CALL GRCZWDCY(2,KOUT,RNA,AJACOB)
            YACOB = AJACOB*YACOB
            KFCODE(3) = KOUT(1)
            KFCODE(4) = KOUT(2)
            KFCODE(5) = KOUT(3)
            KFCODE(6) = KOUT(4)
            AMASS1(3) = GAMS(IABS(KFCODE(3)))
            AMASS1(4) = GAMS(IABS(KFCODE(4)))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
            AMASS1(6) = GAMS(IABS(KFCODE(6)))
            AMASS2(3) = AMASS1(3)**2
            AMASS2(4) = AMASS1(4)**2
            AMASS2(5) = AMASS1(5)**2
            AMASS2(6) = AMASS1(6)**2
         ENDIF

         IF (IGSUB.EQ.577) THEN
            IF (GRCFLAV.GT.132) THEN
               AMASS1(1) = GAMS(IABS(KFCODE(1)))
               AMASS1(2) = GAMS(IABS(KFCODE(2)))
               IF (KFCODE(1).LT.0) THEN
                  TMPMAS = AMASS1(1)
                  AMASS1(1) = AMASS1(2)
                  AMASS1(2) = TMPMAS
               ENDIF
            ENDIF
            IF (KFCODE(5).GT.0) THEN
               TMPMAS = AMASS1(1)
               AMASS1(1) = AMASS1(2)
               AMASS1(2) = TMPMAS
            ENDIF
            AMASS2(1) = AMASS1(1)**2
            AMASS2(2) = AMASS1(2)**2
         ENDIF
      ENDIF

C...AA + jets.
C##############
C AA + 0 jet.
C##############
      IF (IGSUB.EQ.600) THEN
         KFCODE(3) = 22 
         KFCODE(4) = 22
         AMASS1(3) = AMA 
         AMASS1(4) = AMA 
         AMASS2(3) = AMASS1(3)**2
         AMASS2(4) = AMASS1(4)**2
C##############
C AA + 1 jet.
C##############
      ELSE IF (IGSUB.GT.600.AND.IGSUB.LT.603) THEN
         KFCODE(3) = 22 
         KFCODE(4) = 22
         AMASS1(3) = AMA 
         AMASS1(4) = AMA 
         AMASS2(3) = AMASS1(3)**2
         AMASS2(4) = AMASS1(4)**2
C...Set jet flavor.
         IF (MAX(KFCODE(1),KFCODE(2)).EQ.21) THEN
            KFCODE(5) = MIN(KFCODE(1),KFCODE(2))
            AMASS1(5) = GAMS(IABS(KFCODE(5)))
         ELSE
            KFCODE(5) = 21
            AMASS1(5) = AMG
         ENDIF
         AMASS2(5) = AMASS1(5)**2
      ENDIF

      RETURN
 999  CONTINUE
      IREJ = 3
      YACOB = 0.0D0
      RETURN
      END

C----------------------------------------------------------
C Set jet flavors in final state partons of QCD jets proc.
C----------------------------------------------------------
      SUBROUTINE GRCQJET(NRES,NJET,JOUT,RNA,Q2,SUMJBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,J,ID1,ID2,ILP1,ILP2
      INTEGER IJBR,IDCYST,NRES,NJET,NGLU,ICHIN,ICHOUT
      INTEGER JOUT(4)
      INTEGER IDJBR1(40),IDJBR2(40),IDJBR3(40),IDJBR4(40)
      DOUBLE PRECISION RNA,Q2,QJ,SUMJBR,RANSUM
      DOUBLE PRECISION TMPCKMR(40)
      DOUBLE PRECISION XSCALE

C...Initialize
      IJBR = 0
      SUMJBR = 0.0D0
      DO I = 1,40
         IDJBR1(I) = 9
         IDJBR2(I) = 9
         IDJBR3(I) = 9
         IDJBR4(I) = 9
         TMPCKMR(I) = 0.0D0
      ENDDO
      JOUT(1) = 0
      JOUT(2) = 0
      JOUT(3) = 0
      JOUT(4) = 0

C...Sum of intial and final state charges
      ICHIN  = IABS(KCHARG(1)+KCHARG(2))
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      DO I = 1,NJET
         IF (KCHARG(I+2+2*NRES).EQ.0) NGLU = NGLU + 1
         ICHOUT = KCHARG(I+2+2*NRES) + ICHOUT
      ENDDO
      ICHOUT = IABS(ICHOUT)

C...Include heavy flavor jets or not.
      ILP1 = 4
      ILP2 = 3
      IF (IGJFLV(5).EQ.1) ILP2 = 5
      IF (IGJFLV(6).EQ.1) THEN
         ILP1 = 6
         ILP2 = 5
      ENDIF

C...Scale factor for SUMJBR.
      XSCALE = 1.0D0

C...Applied jet mass.
      QJ = SQRT(Q2)

C...Initial partons.
      ID1 = IABS(KFCODE(1))
      ID2 = IABS(KFCODE(2))

C####################
C gg -> n gluons + X
C####################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         IJBR = 1
         TMPCKMR(1) = 1.0D0
         SUMJBR = 1.0D0
      ENDIF
C#####################################
C qq'~/gg -> q"q"~ + (n-2) gluons + X
C#####################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
C...s-channel proc. in 2-body.
         IF (ID1.EQ.ID2) THEN
            DO I = 1,MAX(ILP1,ILP2)
               IF (IGJFLV(I).EQ.1.AND.QJ.GT.2D0*GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                  IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
C...t-channel proc. in 2-body.
         ELSE
            IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1) THEN
               IF (QJ.GT.GAMS(ID1)+GAMS(ID2)) THEN
                  IJBR = 1
                  IDJBR1(1) = KFCODE(1)
                  IDJBR2(1) = KFCODE(2)
                  TMPCKMR(1) = 1.0D0
                  SUMJBR = 1.0D0
               ENDIF
            ENDIF
         ENDIF
      ENDIF
C############################################
C qq'~/gg -> q"q"~q"'q"'~ + (n-4) gluons + X
C############################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.4) THEN
C...s-channel proc. in 2-body.
         IF (ID1.EQ.ID2) THEN
            DO I = 1,MAX(ILP1,ILP2)
               DO J = I,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.2.0D0*(GAMS(I)+GAMS(J))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                        IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                        IDJBR3(IJBR) =  ISIGN(J,KFCODE(1))
                        IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                        TMPCKMR(IJBR) = 1.0D0
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
C...t-channel proc. in 2-body.
         ELSE
            DO I = 1,MAX(ILP1,ILP2)
               IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1
     $              .OR.IGJFLV(I).EQ.1) THEN
                  IF (QJ.GT.GAMS(ID1)+GAMS(ID2)+2.0D0*GAMS(I)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) =  KFCODE(1)
                     IDJBR2(IJBR) =  KFCODE(2)
                     IDJBR3(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR4(IJBR) = -ISIGN(I,KFCODE(1))
                     TMPCKMR(IJBR) = 1.0D0
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDIF
C#############################
C ud -> ud + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1) THEN
            IF (QJ.GT.GAMS(ID1)+GAMS(ID2)) THEN
               IJBR = 1
               IF (IGPCOMB(GRCFLAV+1).EQ.0) THEN
                  IDJBR1(1) = KFCODE(1)
                  IDJBR2(1) = KFCODE(2)
               ELSE
                  IDJBR1(1) = KFCODE(2)
                  IDJBR2(1) = KFCODE(1)
               ENDIF
               TMPCKMR(1) = 1.0D0
               SUMJBR = 1.0D0
            ENDIF
         ENDIF
      ENDIF
C###################################
C ud -> ud + qq~ + (n-4) gluons + X
C###################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.4) THEN
         DO I = 1,MAX(ILP1,ILP2)
            IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1
     $           .OR.IGJFLV(I).EQ.1) THEN
               IF (QJ.GT.GAMS(ID1)+GAMS(ID2)+2.0D0*GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IF (IGPCOMB(GRCFLAV+1).EQ.0) THEN
                     IDJBR1(IJBR) = KFCODE(1)
                     IDJBR2(IJBR) = KFCODE(2)
                     IDJBR3(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR4(IJBR) = -ISIGN(I,KFCODE(1))
                  ELSE
                     IDJBR1(IJBR) = KFCODE(2)
                     IDJBR2(IJBR) = KFCODE(1)
                     IDJBR3(IJBR) =  ISIGN(I,KFCODE(2))
                     IDJBR4(IJBR) = -ISIGN(I,KFCODE(2))
                  ENDIF
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDIF
         ENDDO
      ENDIF
C#############################
C qg -> qg + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.1) THEN
         IF (IGJFLV(MIN(ID1,ID2)).EQ.1
     $        .AND.QJ.GT.GAMS(MIN(ID1,ID2))) THEN
            IJBR = 1
            IDJBR1(1) = MIN(KFCODE(1),KFCODE(2))
            TMPCKMR(1) = 1.0D0
            SUMJBR = 1.0D0
         ENDIF
      ENDIF
C#############################
C qg -> qg + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.3) THEN
         DO I = 1,MAX(ILP1,ILP2)
            IF (IGJFLV(I).EQ.1
     $           .AND.QJ.GT.2D0*GAMS(I)+GAMS(MIN(ID1,ID2))) THEN
               IJBR = IJBR + 1
               IDJBR1(IJBR) = MIN(KFCODE(1),KFCODE(2))
               IDJBR2(IJBR) =  ISIGN(I,IDJBR1(IJBR))
               IDJBR3(IJBR) = -ISIGN(I,IDJBR1(IJBR))
               TMPCKMR(IJBR) = 1.0D0
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO
      ENDIF
C###############################
C ud~ -> ud~ + (n-2) gluons + X
C###############################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1) THEN
            IF (QJ.GT.GAMS(ID1)+GAMS(ID2)) THEN
               IJBR = 1
               IF (MOD(ID1,2).EQ.0) THEN
                  IDJBR1(1) = KFCODE(1)
                  IDJBR2(1) = KFCODE(2)
               ELSE
                  IDJBR1(1) = KFCODE(2)
                  IDJBR2(1) = KFCODE(1)
               ENDIF
               TMPCKMR(1) = 1.0D0
               SUMJBR = 1.0D0
            ENDIF
         ENDIF
      ENDIF
C#####################################
C ud~ -> ud~ + qq~ + (n-4) gluons + X
C#####################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.4) THEN
         DO I = 1,MAX(ILP1,ILP2)
            IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1
     $           .OR.IGJFLV(I).EQ.1) THEN
               IF (QJ.GT.GAMS(ID1)+GAMS(ID2)+2.0D0*GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IF (MOD(ID1,2).EQ.0) THEN
                     IDJBR1(IJBR) =  KFCODE(1)
                     IDJBR2(IJBR) =  KFCODE(2)
                     IDJBR3(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR4(IJBR) = -ISIGN(I,KFCODE(1))
                  ELSE
                     IDJBR1(IJBR) =  KFCODE(2)
                     IDJBR2(IJBR) =  KFCODE(1)
                     IDJBR3(IJBR) =  ISIGN(I,KFCODE(2))
                     IDJBR4(IJBR) = -ISIGN(I,KFCODE(2))
                  ENDIF
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDIF
         ENDDO
      ENDIF
C#############################
C uu -> uu + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.2) THEN
         IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1) THEN
            IF (QJ.GT.GAMS(ID1)+GAMS(ID2)) THEN
               IJBR = 1
               IF (IGPCOMB(GRCFLAV+1).EQ.0) THEN
                  IDJBR1(1) = KFCODE(1)
                  IDJBR2(1) = KFCODE(2)
               ELSE
                  IDJBR1(1) = KFCODE(2)
                  IDJBR2(1) = KFCODE(1)
               ENDIF
               TMPCKMR(1) = 1.0D0
               SUMJBR = 1.0D0
            ENDIF
         ENDIF
      ENDIF
C###################################
C uu -> uu + qq~ + (n-4) gluons + X
C###################################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.4) THEN
         DO I = 1,MAX(ILP1,ILP2)
            IF (IGJFLV(ID1).EQ.1.OR.IGJFLV(ID2).EQ.1
     $           .OR.IGJFLV(I).EQ.1) THEN
               IF (QJ.GT.GAMS(ID1)+GAMS(ID2)+2.0D0*GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IF (IGPCOMB(GRCFLAV+1).EQ.0) THEN
                     IDJBR1(IJBR) = KFCODE(1)
                     IDJBR2(IJBR) = KFCODE(2)
                  ELSE
                     IDJBR1(IJBR) = KFCODE(2)
                     IDJBR2(IJBR) = KFCODE(1)
                  ENDIF
                  IDJBR3(IJBR) =  ISIGN(I,KFCODE(1))
                  IDJBR4(IJBR) = -ISIGN(I,KFCODE(1))
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDIF
         ENDDO
      ENDIF

C...If all branch are off.
      IF (IJBR.EQ.0) GO TO 999

C...Pick up final state flavor.
      RANSUM = RNA*SUMJBR
      IDCYST = 0
      DO WHILE (RANSUM.GT.0.0D0)
         IDCYST = IDCYST + 1
         RANSUM = RANSUM - TMPCKMR(IDCYST)
      ENDDO
      JOUT(1) = IDJBR1(IDCYST)
      JOUT(2) = IDJBR2(IDCYST)
      JOUT(3) = IDJBR3(IDCYST)
      JOUT(4) = IDJBR4(IDCYST)

C...Output Jacobian factor.
      SUMJBR = XSCALE*SUMJBR

      RETURN
 999  CONTINUE
      IREJ = 3
      SUMJBR = 0.0D0
      RETURN
      END

C----------------------------------------------------------
C Set jet flavors in final state partons of W + jets proc.
C----------------------------------------------------------
      SUBROUTINE GRCWJET(NRES,NJET,JOUT,RNA,Q2,SUMJBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,J,K,ID1,ID2,ILP1,ILP2
      INTEGER IJBR,IDCYST,NRES,NJET,NGLU,ICHIN,ICHOUT
      INTEGER JOUT(4),IGENE(6),ISJ(2)
      INTEGER IDJBR1(120),IDJBR2(120),IDJBR3(120),IDJBR4(120)
      DOUBLE PRECISION RNA,Q2,QJ,SUMJBR,RANSUM
      DOUBLE PRECISION TMPCKMR(120)
      DOUBLE PRECISION XS1,XS2,XSS,XSCALE

C...Initialize
      IJBR = 0
      SUMJBR = 0.0D0
      DO I = 1,120
         IDJBR1(I) = 9
         IDJBR2(I) = 9
         IDJBR3(I) = 9
         IDJBR4(I) = 9
         TMPCKMR(I) = 0.0D0
      ENDDO
      JOUT(1) = 0
      JOUT(2) = 0
      JOUT(3) = 0
      JOUT(4) = 0

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Sum of intial and final state charges
      ICHIN  = IABS(KCHARG(1)+KCHARG(2))
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      DO I = 1,NJET
         IF (KCHARG(I+2+2*NRES).EQ.0) NGLU = NGLU + 1
         ICHOUT = KCHARG(I+2+2*NRES) + ICHOUT
      ENDDO
      ICHOUT = IABS(ICHOUT)

C...Include heavy flavor jets or not.
      ILP1 = 4
      ILP2 = 3
      IF (IGJFLV(5).EQ.1) ILP2 = 5
      IF (IGJFLV(6).EQ.1) THEN
         ILP1 = 6
         ILP2 = 5
      ENDIF

C...Scale factor for SUMJBR.
      XSCALE = 1.0D0

C...Applied jet mass.
      QJ = SQRT(Q2)

C...Initial parton flavor. ID1:up-type, ID2:down-type.
      IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
         ID1 = KFCODE(1)
         ID2 = KFCODE(2)
      ELSE
         ID1 = KFCODE(2)
         ID2 = KFCODE(1)
      ENDIF

C##########################
C ud~ -> W+ + n gluons + X
C##########################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         XSCALE = XSCALE*GRCCKM(IGENE(IABS(ID1)),IGENE(IABS(ID2)))
         IJBR = 1
         SUMJBR = 1.0D0
         TMPCKMR(IJBR) = 1.0D0
      ENDIF
C########################################
C qq'~ -> W+ + q"q"'~ + (n-2) gluons + X
C########################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
C...Set ratio for jet flavor configuration.
         IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
            XS1 = 1.0D0
            XS2 = 0.0D0
         ELSE
            XS1 = 0.8D0
            XS2 = 0.2D0
         ENDIF
         XSS = XS1/(XS1 + XS2)
C...qq'~ -> q"q"'~
         IF (RNA.LE.XSS) THEN
            RNA = RNA/XSS
            XSCALE = XSCALE/XSS
            XSCALE = XSCALE*GRCCKM(IGENE(IABS(ID1)),IGENE(IABS(ID2)))
            DO I = 1,MAX(ILP1,ILP2)
               IF (IGJFLV(I).EQ.1.AND.QJ.GT.2.D0*GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
                     IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                  ELSE
                     IDJBR1(IJBR) = -ISIGN(I,KFCODE(1))
                     IDJBR2(IJBR) =  ISIGN(I,KFCODE(1))
                  ENDIF
                  TMPCKMR(IJBR) = 1.0D0
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
C...qq'~ -> qq"~
         ELSE
            RNA = (RNA - XSS)/(1.0D0 - XSS)
            XSCALE = XSCALE/(1.0D0 - XSS)

C...up-type jet
            IF (IABS(ID1).EQ.2) THEN
               ISJ(1) = 4
               ISJ(2) = 6
            ELSE IF (IABS(ID1).EQ.4) THEN
               ISJ(1) = 2
               ISJ(2) = 6
            ELSE IF (IABS(ID1).EQ.6) THEN
               ISJ(1) = 2
               ISJ(2) = 4
            ENDIF
            DO I = 1,ILP1/2-1
               IF (IGJFLV(ISJ(I)).EQ.1
     $              .AND.QJ.GT.GAMS(ISJ(I))+GAMS(IABS(ID1))) THEN
                  IJBR = IJBR + 1
                  IDJBR1(IJBR) = ID1
                  IDJBR2(IJBR) = -ISIGN(ISJ(I),ID1)
                  TMPCKMR(IJBR)=GRCCKM(IGENE(ISJ(I)),IGENE(IABS(ID2)))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO

C...down-type jet
            IF (IABS(ID2).EQ.1) THEN
               ISJ(1) = 3
               ISJ(2) = 5
            ELSE IF (IABS(ID2).EQ.3) THEN
               ISJ(1) = 1
               ISJ(2) = 5
            ELSE IF (IABS(ID2).EQ.5) THEN
               ISJ(1) = 1
               ISJ(2) = 3
            ENDIF
            DO I = 1,(ILP2-1)/2
               IF (IGJFLV(ISJ(I)).EQ.1
     $              .AND.QJ.GT.GAMS(ISJ(I))+GAMS(IABS(ID2))) THEN
                  IJBR = IJBR + 1
                  IDJBR1(IJBR) = -ISIGN(ISJ(I),ID2)
                  IDJBR2(IJBR) = ID2
                  TMPCKMR(IJBR)=GRCCKM(IGENE(IABS(ID1)),IGENE(ISJ(I)))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
         ENDIF
      ENDIF
C##############################################
C ud~ -> W+ + uu~qq~/dd~qq~ + (n-4) gluons + X
C##############################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.4) THEN
C...Set ratio for jet flavor configuration.
         IF (GRCCKM(1,1).EQ.1.0D0.AND.GRCCKM(1,2).EQ.0.0D0) THEN
            XS1 = 1.0D0
            XS2 = 0.0D0
         ELSE
            XS1 = 0.8D0
            XS2 = 0.2D0
         ENDIF
         XSS = XS1/(XS1 + XS2)
C...qq'~ -> q"q"'~
         IF (RNA.LE.XSS) THEN
            RNA = RNA/XSS
            XSCALE = XSCALE/XSS
            XSCALE = XSCALE*GRCCKM(IGENE(IABS(ID1)),IGENE(IABS(ID2)))
            DO I = 1,MAX(ILP1,ILP2)
               DO J = I,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.2.0D0*(GAMS(I)+GAMS(J))) THEN
                        IJBR = IJBR + 1
                        IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
                           IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                           IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                           IDJBR3(IJBR) =  ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                        ELSE
                           IDJBR1(IJBR) = -ISIGN(I,KFCODE(1))
                           IDJBR2(IJBR) =  ISIGN(I,KFCODE(1))
                           IDJBR3(IJBR) = -ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) =  ISIGN(J,KFCODE(1))
                        ENDIF
                        TMPCKMR(IJBR) = 1.0D0
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
C...qq'~ -> qq"~
         ELSE
            RNA = (RNA - XSS)/(1.0D0 - XSS)
            XSCALE = XSCALE/(1.0D0 - XSS)

C...up-type jet
            IF (IABS(ID1).EQ.2) THEN
               ISJ(1) = 4
               ISJ(2) = 6
            ELSE IF (IABS(ID1).EQ.4) THEN
               ISJ(1) = 2
               ISJ(2) = 6
            ELSE IF (IABS(ID1).EQ.6) THEN
               ISJ(1) = 2
               ISJ(2) = 4
            ENDIF
            DO I = 1,ILP1/2-1
               DO J = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(ISJ(I)).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.GAMS(IABS(ID1))+GAMS(ISJ(I))
     $                    +2.0D0*GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = ID1
                        IDJBR2(IJBR) = -ISIGN(ISJ(I),ID1)
                        IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
                           IDJBR3(IJBR) =  ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                        ELSE
                           IDJBR3(IJBR) = -ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) =  ISIGN(J,KFCODE(1))
                        ENDIF
                        TMPCKMR(IJBR) = GRCCKM(IGENE(ISJ(I)),
     $                       IGENE(IABS(ID2)))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO

C...down-type jet
            IF (IABS(ID2).EQ.1) THEN
               ISJ(1) = 3
               ISJ(2) = 5
            ELSE IF (IABS(ID2).EQ.3) THEN
               ISJ(1) = 1
               ISJ(2) = 5
            ELSE IF (IABS(ID2).EQ.5) THEN
               ISJ(1) = 1
               ISJ(2) = 3
            ENDIF
            DO I = 1,(ILP2-1)/2
               DO J = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(ISJ(I)).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.GAMS(IABS(ID2))+GAMS(ISJ(I))
     $                    +2.0D0*GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = -ISIGN(ISJ(I),ID2)
                        IDJBR2(IJBR) = ID2
                        IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
                           IDJBR3(IJBR) =  ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                        ELSE
                           IDJBR3(IJBR) = -ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) =  ISIGN(J,KFCODE(1))
                        ENDIF
                        TMPCKMR(IJBR) = GRCCKM(IGENE(IABS(ID1)),
     $                       IGENE(ISJ(I)))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF
      ENDIF
C###################################
C qg -> W+ + q'g + (n-2) gluons + X
C###################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.1) THEN
         ID1 = MIN(KFCODE(1),KFCODE(2))
C...down-type jet
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            DO I = 1,ILP2,2
               IF (IGJFLV(I).EQ.1.AND.QJ.GT.GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IDJBR1(IJBR) = ISIGN(I,ID1)
                  TMPCKMR(IJBR) = GRCCKM(IGENE(IABS(ID1)),IGENE(I))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
C...up-type jet
         ELSE
            DO I = 2,ILP1,2
               IF (IGJFLV(I).EQ.1.AND.QJ.GT.GAMS(I)) THEN
                  IJBR = IJBR + 1
                  IDJBR1(IJBR) = ISIGN(I,ID1)
                  TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(IABS(ID1)))
                  SUMJBR = TMPCKMR(IJBR) + SUMJBR
               ENDIF
            ENDDO
         ENDIF
      ENDIF
C################################################
C uu'/d~d'~ -> W+ + d"u/d~u"~ + (n-2) gluons + X
C################################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         IF (KFCODE(1).EQ.KFCODE(2)) THEN
            ID1 = IABS(KFCODE(1))
C...down-type jet
            IF (MOD(ID1,2).EQ.0) THEN
               DO I = 1,ILP2,2
                  IF (IGJFLV(I).EQ.1
     $                 .AND.QJ.GT.GAMS(I)+GAMS(ID1)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = ISIGN(I,KFCODE(1))
                     IDJBR2(IJBR) = KFCODE(1)
                     TMPCKMR(IJBR) = GRCCKM(IGENE(ID1),IGENE(I))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDDO
C...up-type jet
            ELSE
               DO I = 2,ILP1,2
                  IF (IGJFLV(I).EQ.1
     $                 .AND.QJ.GT.GAMS(I)+GAMS(ID1)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = KFCODE(1)
                     IDJBR2(IJBR) = ISIGN(I,KFCODE(1))
                     TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(ID1))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDDO
            ENDIF
         ELSE
C...down-type jet
            IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
               DO I = 1,2
                  ID1 = IABS(KFCODE(I))
                  DO J = 1,ILP2,2
                     IF (IGJFLV(J).EQ.1
     $                    .AND.QJ.GT.GAMS(ID1)+GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = ISIGN(J,KFCODE(I))
                        IDJBR2(IJBR) = KFCODE(I)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(ID1),IGENE(J))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO
               ENDDO
C...up-type jet
            ELSE
               DO I = 1,2
                  ID1 = IABS(KFCODE(I))
                  DO J = 2,ILP1,2
                     IF (IGJFLV(J).EQ.1
     $                    .AND.QJ.GT.GAMS(ID1)+GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = KFCODE(I)
                        IDJBR2(IJBR) = ISIGN(J,KFCODE(I))
                        TMPCKMR(IJBR) = GRCCKM(IGENE(J),IGENE(ID1))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ENDIF
      ENDIF
C################################################
C qg  -> W+ + q'q"q"~/q'q'q'~ + (n-3) gluons + X
C q~g -> W+ + q~q~q'          + (n-3) gluons + X
C################################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.3) THEN
         ID1 = IABS(MIN(KFCODE(1),KFCODE(2)))
         ID2 = MIN(KFCODE(1),KFCODE(2))
C...Initial up-type quark.
         IF (MOD(ID1,2).EQ.0) THEN
C...Scan d/u/s/c/b
C...Decide Down-type jet.
            DO I = 1,ILP2,2
               DO J = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.GAMS(I)+2.0D0*GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(I,ID2)
                        IDJBR2(IJBR) =  ISIGN(J,ID2)
                        IDJBR3(IJBR) = -ISIGN(J,ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(ID1),IGENE(I))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
               IF (ID1.EQ.2) THEN
                  ISJ(1) = 4
                  ISJ(2) = 6
               ELSE IF (ID1.EQ.4) THEN
                  ISJ(1) = 2
                  ISJ(2) = 6
               ENDIF
               DO J = 1,ILP1/2-1
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(ISJ(J)).EQ.1) THEN
                     IF (QJ.GT.GAMS(I)+GAMS(ID1)+GAMS(ISJ(J))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(I,ID2)
                        IDJBR2(IJBR) =  ISIGN(ID1,ID2)
                        IDJBR3(IJBR) = -ISIGN(ISJ(J),ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(ISJ(J)),IGENE(I))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
C...Decide Up-type jet.
            DO J = 2,ILP1,2
               DO I = 1,ILP2,2
                  IF (IGJFLV(J).EQ.1.OR.IGJFLV(I).EQ.1) THEN
                     IF (QJ.GT.GAMS(ID1)+GAMS(I)+GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ID2
                        IDJBR2(IJBR) =  ISIGN(J,ID2)
                        IDJBR3(IJBR) = -ISIGN(I,ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(J),IGENE(I))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
C...Initial down-type quark.
         ELSE
C...Decide Up-type jet.
            DO I = 2,ILP1,2
               DO J = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.GAMS(I)+2.0D0*GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(I,ID2)
                        IDJBR2(IJBR) =  ISIGN(J,ID2)
                        IDJBR3(IJBR) = -ISIGN(J,ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(ID1))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
               IF (ID1.EQ.1) THEN
                  ISJ(1) = 3
                  ISJ(2) = 5
               ELSE IF (ID1.EQ.3) THEN
                  ISJ(1) = 1
                  ISJ(2) = 5
               ELSE IF (ID1.EQ.5) THEN
                  ISJ(1) = 1
                  ISJ(2) = 3
               ENDIF
               DO J = 1,(ILP2-1)/2
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(ISJ(J)).EQ.1) THEN
                     IF (QJ.GT.GAMS(I)+GAMS(ID1)+GAMS(ISJ(J))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(I,ID2)
                        IDJBR2(IJBR) =  ISIGN(ID1,ID2)
                        IDJBR3(IJBR) = -ISIGN(ISJ(J),ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(ISJ(J)))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
C...Decide Down-type jet.
            DO J = 1,ILP2,2
               DO I = 2,ILP1,2
                  IF (IGJFLV(J).EQ.1.OR.IGJFLV(I).EQ.1) THEN
                     IF (QJ.GT.GAMS(ID1)+GAMS(I)+GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ID2
                        IDJBR2(IJBR) =  ISIGN(J,ID2)
                        IDJBR3(IJBR) = -ISIGN(I,ID2)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(J))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF
      ENDIF
C###########################################
C qq' -> W+ + q"qq"'q"'~ + (n-4) gluons + X
C###########################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.4) THEN
         IF (KFCODE(1).EQ.KFCODE(2)) THEN
            ID1 = IABS(KFCODE(1))
C...down-type jet
            IF (MOD(ID1,2).EQ.0) THEN
               DO I = 1,ILP2,2
                  DO J = 1,MAX(ILP1,ILP2)
                     IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                        IF (QJ.GT.GAMS(ID1)+GAMS(I)+2D0*GAMS(J)) THEN
                           IJBR = IJBR + 1
                           IDJBR1(IJBR) = ISIGN(I,KFCODE(1))
                           IDJBR2(IJBR) = KFCODE(1)
                           IDJBR3(IJBR) = ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                           TMPCKMR(IJBR) = GRCCKM(IGENE(ID1),IGENE(I))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDIF
                  ENDDO
               ENDDO
C...up-type jet
            ELSE
               DO I = 2,ILP1,2
                  DO J = 1,MAX(ILP1,ILP2)
                     IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                        IF (QJ.GT.GAMS(ID1)+GAMS(I)+2D0*GAMS(J)) THEN
                           IJBR = IJBR + 1
                           IDJBR1(IJBR) = KFCODE(1)
                           IDJBR2(IJBR) = ISIGN(I,KFCODE(1))
                           IDJBR3(IJBR) = ISIGN(J,KFCODE(1))
                           IDJBR4(IJBR) = -ISIGN(J,KFCODE(1))
                           TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(ID1))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ELSE
C...down-type jet
            IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
               DO I = 1,2
                  ID1 = IABS(KFCODE(I))
                  DO J = 1,ILP2,2
                     DO K = 1,MAX(ILP1,ILP2)
                        IF (IGJFLV(J).EQ.1.OR.IGJFLV(K).EQ.1) THEN
                           IF (QJ.GT.GAMS(ID1)+GAMS(J)+2D0*GAMS(K)) THEN
                              IJBR = IJBR + 1
                              IDJBR1(IJBR) = ISIGN(J,KFCODE(I))
                              IDJBR2(IJBR) = KFCODE(I)
                              IDJBR3(IJBR) = ISIGN(K,KFCODE(I))
                              IDJBR4(IJBR) = -ISIGN(K,KFCODE(I))
                              TMPCKMR(IJBR)= GRCCKM(IGENE(ID1),IGENE(J))
                              SUMJBR = TMPCKMR(IJBR) + SUMJBR
                           ENDIF
                        ENDIF
                     ENDDO
                  ENDDO
               ENDDO
C...up-type jet
            ELSE
               DO I = 1,2
                  ID1 = IABS(KFCODE(I))
                  DO J = 2,ILP1,2
                     DO K = 1,MAX(ILP1,ILP2)
                        IF (IGJFLV(J).EQ.1.OR.IGJFLV(K).EQ.1) THEN
                           IF (QJ.GT.GAMS(ID1)+GAMS(J)+2D0*GAMS(K)) THEN
                              IJBR = IJBR + 1
                              IDJBR1(IJBR) = KFCODE(I)
                              IDJBR2(IJBR) = ISIGN(J,KFCODE(I))
                              IDJBR3(IJBR) = ISIGN(K,KFCODE(I))
                              IDJBR4(IJBR) = -ISIGN(K,KFCODE(I))
                              TMPCKMR(IJBR)=GRCCKM(IGENE(J),IGENE(ID1))
                              SUMJBR = TMPCKMR(IJBR) + SUMJBR
                           ENDIF
                        ENDIF
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF
         ENDIF
      ENDIF
C#############################################
C ud/u~d~ -> W+ + d'd/u~u' + (n-2) gluons + X
C#############################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.2) THEN
C...down-type jet
         DO I = 1,ILP2,2
            IF (IGJFLV(I).EQ.1
     $           .AND.QJ.GT.GAMS(ID2)+GAMS(I)) THEN
               IJBR = IJBR + 1
               IDJBR1(IJBR) = ISIGN(I,ID2)
               IDJBR2(IJBR) = ID2
               TMPCKMR(IJBR) = GRCCKM(IGENE(IABS(ID1)),IGENE(I))
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO
C...up-type jet
         DO I = 2,ILP1,2
            IF (IGJFLV(I).EQ.1
     $           .AND.QJ.GT.GAMS(ID1)+GAMS(I)) THEN
               IJBR = IJBR + 1
               IDJBR1(IJBR) = ID1
               IDJBR2(IJBR) = ISIGN(I,ID1)
               TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(IABS(ID2)))
               SUMJBR = TMPCKMR(IJBR) + SUMJBR
            ENDIF
         ENDDO
      ENDIF
C#################################################
C ud/u~d~ -> W+ + d'd/u~u'qq'~ + (n-4) gluons + X
C#################################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.4) THEN
C...down-type jet
         DO I = 1,ILP2,2
            DO J = 1,MAX(ILP1,ILP2)
               IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                  IF (QJ.GT.GAMS(IABS(ID2))+GAMS(I)+2D0*GAMS(J)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = ID2
                     IDJBR2(IJBR) = ISIGN(I,ID2)
                     IDJBR3(IJBR) = ISIGN(J,ID2)
                     IDJBR4(IJBR) = -ISIGN(J,ID2)
                     TMPCKMR(IJBR) = GRCCKM(IGENE(IABS(ID1)),IGENE(I))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
C...up-type jet
         DO I = 2,ILP1,2
            DO J = 1,MAX(ILP1,ILP2)
               IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                  IF (QJ.GT.GAMS(IABS(ID1))+GAMS(I)+2D0*GAMS(J)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = ID1
                     IDJBR2(IJBR) = ISIGN(I,ID1)
                     IDJBR3(IJBR) = ISIGN(J,ID1)
                     IDJBR4(IJBR) = -ISIGN(J,ID1)
                     TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(IABS(ID2)))
                     SUMJBR = TMPCKMR(IJBR) + SUMJBR
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDIF
C###############################################
C qq~/gg  -> W+ + q'q"~      + (n-2) gluons + X
C qq'~    -> W+ + qq"~/q"q'~ + (n-2) gluons + X
C###############################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         ISJ(1) = IABS(KFCODE(1))
         ISJ(2) = IABS(KFCODE(2))
         IF (ISJ(1).EQ.ISJ(2)) THEN
            DO I = 2,ILP1,2
               DO J = 1,ILP2,2
                  IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                     IF (QJ.GT.GAMS(I)+GAMS(J)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = J
                        IDJBR2(IJBR) = -I
                        TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(J))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
            DO I = IJBR+1,2*IJBR
               IDJBR1(I) = -IDJBR1(I-IJBR)
               IDJBR2(I) = -IDJBR2(I-IJBR)
               TMPCKMR(I) = GRCCKM(IGENE(IABS(IDJBR2(I))),
     $              IGENE(IABS(IDJBR1(I))))
               SUMJBR = TMPCKMR(I) + SUMJBR
            ENDDO
            IJBR = 2*IJBR
         ELSE
C...down-type jet
            IF (MOD(ISJ(1),2).EQ.0) THEN
               DO I = 1,2
                  DO J = 1,ILP2,2
                     IF (IGJFLV(J).EQ.1
     $                    .AND.QJ.GT.GAMS(J)+GAMS(ISJ(I))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = -ISIGN(J,KFCODE(I))
                        IDJBR2(IJBR) = KFCODE(I)
                        TMPCKMR(IJBR) = GRCCKM(IGENE(ISJ(I)),IGENE(J))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO
               ENDDO
C...up-type jet
            ELSE
               DO I = 1,2
                  DO J = 2,ILP1,2
                     IF (IGJFLV(J).EQ.1
     $                    .AND.QJ.GT.GAMS(J)+GAMS(ISJ(I))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = KFCODE(I)
                        IDJBR2(IJBR) = -ISIGN(J,KFCODE(I))
                        TMPCKMR(IJBR) = GRCCKM(IGENE(J),IGENE(ISJ(I)))
                        SUMJBR = TMPCKMR(IJBR) + SUMJBR
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ENDIF
      ENDIF
C######################################################
C qq~/gg  -> W+ + q'q"~q"'q"'~      + (n-4) gluons + X
C qq'~    -> W+ + qq"~/q"q'~q"'q"'~ + (n-4) gluons + X
C######################################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.4) THEN
         ISJ(1) = IABS(KFCODE(1))
         ISJ(2) = IABS(KFCODE(2))
         IF (ISJ(1).EQ.ISJ(2)) THEN
            DO I = 2,ILP1,2
               DO J = 1,ILP2,2
                  DO K = 1,MAX(ILP1,ILP2)
                     IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1
     $                    .OR.IGJFLV(K).EQ.1) THEN
                        IF (QJ.GT.GAMS(I)+GAMS(J)+2D0*GAMS(K)) THEN
                           IJBR = IJBR + 1
                           IDJBR1(IJBR) = J
                           IDJBR2(IJBR) = -I
                           IDJBR3(IJBR) = K
                           IDJBR4(IJBR) = -K
                           TMPCKMR(IJBR) = GRCCKM(IGENE(I),IGENE(J))
                           SUMJBR = TMPCKMR(IJBR) + SUMJBR
                        ENDIF
                     ENDIF
                  ENDDO
               ENDDO
            ENDDO
            DO I = IJBR+1,2*IJBR
               IDJBR1(I) = -IDJBR1(I-IJBR)
               IDJBR2(I) = -IDJBR2(I-IJBR)
               IDJBR3(I) = -IDJBR3(I-IJBR)
               IDJBR4(I) = -IDJBR4(I-IJBR)
               TMPCKMR(I) = GRCCKM(IGENE(IABS(IDJBR2(I))),
     $              IGENE(IABS(IDJBR1(I))))
               SUMJBR = TMPCKMR(I) + SUMJBR
            ENDDO
            IJBR = 2*IJBR
         ELSE
C...down-type jet
            IF (MOD(ISJ(1),2).EQ.0) THEN
               DO I = 1,2
                  DO J = 1,ILP2,2
                     DO K = 1,MAX(ILP1,ILP2)
                        IF (IGJFLV(J).EQ.1.OR.IGJFLV(K).EQ.1) THEN
                           IF (QJ.GT.GAMS(J)+2D0*GAMS(K)) THEN
                              IJBR = IJBR + 1
                              IDJBR1(IJBR) = -ISIGN(J,KFCODE(I))
                              IDJBR2(IJBR) = KFCODE(I)
                              IDJBR3(IJBR) = -ISIGN(K,KFCODE(I))
                              IDJBR4(IJBR) = ISIGN(K,KFCODE(I))
                              TMPCKMR(IJBR)=
     $                             GRCCKM(IGENE(ISJ(I)),IGENE(J))
                              SUMJBR = TMPCKMR(IJBR) + SUMJBR
                           ENDIF
                        ENDIF
                     ENDDO
                  ENDDO
               ENDDO
C...up-type jet
            ELSE
               DO I = 1,2
                  DO J = 2,ILP1,2
                     DO K = 1,MAX(ILP1,ILP2)
                        IF (IGJFLV(J).EQ.1.OR.IGJFLV(K).EQ.1) THEN
                           IF (QJ.GT.GAMS(J)+2D0*GAMS(K)) THEN
                              IJBR = IJBR + 1
                              IDJBR1(IJBR) = KFCODE(I)
                              IDJBR2(IJBR) = -ISIGN(J,KFCODE(I))
                              IDJBR3(IJBR) = ISIGN(K,KFCODE(I))
                              IDJBR4(IJBR) = -ISIGN(K,KFCODE(I))
                              TMPCKMR(IJBR) = 
     $                             GRCCKM(IGENE(J),IGENE(ISJ(I)))
                              SUMJBR = TMPCKMR(IJBR) + SUMJBR
                           ENDIF
                        ENDIF
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF
         ENDIF
      ENDIF

C...If all branch are off.
      IF (IJBR.EQ.0) GO TO 999

C...Pick up final state flavor.
      RANSUM = RNA*SUMJBR
      IDCYST = 0
      DO WHILE (RANSUM.GT.0.0D0)
         IDCYST = IDCYST + 1
         RANSUM = RANSUM - TMPCKMR(IDCYST)
      ENDDO
      JOUT(1) = IDJBR1(IDCYST)
      JOUT(2) = IDJBR2(IDCYST)
      JOUT(3) = IDJBR3(IDCYST)
      JOUT(4) = IDJBR4(IDCYST)

C...Output Jacobian factor.
      SUMJBR = XSCALE*SUMJBR

      RETURN
 999  CONTINUE
      IREJ = 3
      SUMJBR = 0.0D0
      RETURN
      END

C-----------------------------------------------------------
C Set jet flavors in final state partons of WW + jets proc.
C-----------------------------------------------------------
      SUBROUTINE GRCWWJT(NRES,NJET,JOUT,RNA,Q2,SUMWWBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,J,K,ID1,ID2,ILP1,ILP2
      INTEGER IJBR,IDCYST,NRES,NJET,NGLU,ICHIN,ICHOUT
      INTEGER JOUT(4),IGENE(6),ISJ(2),IJ1(6),IJ2(6)
      INTEGER IDJBR1(120),IDJBR2(120),IDJBR3(120),IDJBR4(120)
      DOUBLE PRECISION RNA,Q2,QJ,SUMWWBR,RANSUM
      DOUBLE PRECISION TMPCKMR(120)
      DOUBLE PRECISION XS1,XS2,XSS,CKMRA,CKMRB,CKMRC,XSCALE

C...Initialize
      IJBR = 0
      SUMWWBR = 0.0D0
      DO I = 1,120
         IDJBR1(I) = 9
         IDJBR2(I) = 9
         IDJBR3(I) = 9
         IDJBR4(I) = 9
         TMPCKMR(I) = 0.0D0
      ENDDO
      JOUT(1) = 0
      JOUT(2) = 0
      JOUT(3) = 0
      JOUT(4) = 0

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Sum of intial and final state charges
      ICHIN  = IABS(KCHARG(1)+KCHARG(2))
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      DO I = 1,NJET
         IF (KCHARG(I+2+2*NRES).EQ.0) NGLU = NGLU + 1
         ICHOUT = KCHARG(I+2+2*NRES) + ICHOUT
      ENDDO
      ICHOUT = IABS(ICHOUT)

C...Include heavy flavor jets or not.
      ILP1 = 4
      ILP2 = 3
      IF (IGJFLV(5).EQ.1) ILP2 = 5
      IF (IGJFLV(6).EQ.1) THEN
         ILP1 = 6
         ILP2 = 5
      ENDIF

C...Scale factor for SUMWWBR.
      XSCALE = 1.0D0

C...Applied jet mass.
      QJ = SQRT(Q2)

C...Initial parton flavor. ID1:up-type, ID2:down-type.
      IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
         ID1 = KFCODE(1)
         ID2 = KFCODE(2)
      ELSE
         ID1 = KFCODE(2)
         ID2 = KFCODE(1)
      ENDIF

C...Mass of propagator is assumed to be zero.
      AMDQ(1) = 0.0D0

C###########################
C qq~ -> WW + n gluons
C###########################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         IJBR = 1
         SUMWWBR = 1.0D0
         TMPCKMR(IJBR) = 1.0D0
      ENDIF
C###############################################
C qq'~/gg -> WW + q"q"~/q"q' + (n-2) gluons
C###############################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         IF (IABS(ID1).EQ.IABS(ID2)) THEN
C...qq~ -> q'q'~
            IF (RNA.LT.1.0D0/2.0D0) THEN
               RNA = RNA*2.0D0
               XSCALE = XSCALE*2.0D0
               DO I = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.AND.QJ.GT.2D0*GAMS(I)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                     TMPCKMR(IJBR) = 1.0D0
                     SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                  ENDIF
               ENDDO
C...Jets: different flavors.
            ELSE
               RNA = RNA*2.0D0-1.0D0
               XSCALE = XSCALE*2.0D0
C...(2,4) (2,6) (4,6)
C...(1,3) (1,5) (3,5)
               IJ1(1) = 2
               IJ2(1) = 4
               IJ1(2) = 1
               IJ2(2) = 3
               IJ1(3) = 1
               IJ2(3) = 5
               IJ1(4) = 3
               IJ2(4) = 5
               IJ1(5) = 2
               IJ2(5) = 6
               IJ1(6) = 4
               IJ2(6) = 6
               DO I = 1,2*(MAX(ILP1,ILP2)-3)
                  IF (IGJFLV(IJ1(I)).EQ.1.OR.IGJFLV(IJ2(I)).EQ.1) THEN
                     IF (QJ.GT.GAMS(IJ1(I))+GAMS(IJ2(I))) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) =  ISIGN(IJ1(I),KFCODE(1))
                        IDJBR2(IJBR) = -ISIGN(IJ2(I),KFCODE(1))
                        DO J = 1,3
                           IF (MOD(IJ1(I),2).EQ.0) THEN
                              TMPCKMR(IJBR) = TMPCKMR(IJBR)
     $                             +SQRT(GRCCKM(IGENE(IJ1(I)),J))
     $                             *SQRT(GRCCKM(IGENE(IJ2(I)),J))
                           ELSE
                              TMPCKMR(IJBR) = TMPCKMR(IJBR)
     $                             +SQRT(GRCCKM(J,IGENE(IJ1(I))))
     $                             *SQRT(GRCCKM(J,IGENE(IJ2(I))))
                           ENDIF
                        ENDDO
                        TMPCKMR(IJBR) = TMPCKMR(IJBR)**2
                        SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                     ENDIF
                  ENDIF
               ENDDO
               DO I = IJBR+1,2*IJBR
                  IDJBR1(I) = -IDJBR1(I-IJBR)
                  IDJBR2(I) = -IDJBR2(I-IJBR)
                  TMPCKMR(I) = TMPCKMR(I-IJBR)
                  SUMWWBR = TMPCKMR(I) + SUMWWBR
               ENDDO
               IJBR = 2*IJBR
            ENDIF
         ELSE
C...Jets: same flavor & same type of initial quarks.
            IF (RNA.LT.1.0D0/3.0D0) THEN
               RNA = RNA*3.0D0
               XSCALE = XSCALE*3.0D0
               DO I = 1,2
                  IF (IGJFLV(IABS(KFCODE(I))).EQ.1) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) =  ISIGN(IABS(KFCODE(I)),KFCODE(1))
                     IDJBR2(IJBR) = -ISIGN(IABS(KFCODE(I)),KFCODE(1))
                     DO J = 1,3
                        IF (MOD(IABS(ID1),2).EQ.0) THEN
                           TMPCKMR(IJBR) = TMPCKMR(IJBR) 
     $                          + SQRT(GRCCKM(IGENE(IABS(ID2)),J))
     $                          *SQRT(GRCCKM(IGENE(IABS(ID1)),J))
                        ELSE
                           TMPCKMR(IJBR) = TMPCKMR(IJBR) 
     $                          + SQRT(GRCCKM(J,IGENE(IABS(ID2))))
     $                          *SQRT(GRCCKM(J,IGENE(IABS(ID1))))
                        ENDIF
                     ENDDO
                     TMPCKMR(IJBR) = TMPCKMR(IJBR)**2
                     SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                  ENDIF
               ENDDO
C...Jets: same flavor & different type of initial quarks.
            ELSE IF (RNA.LT.2.0D0/3.0D0) THEN
               RNA = RNA*3.0D0-1.0D0
               XSCALE = XSCALE*3.0D0
               DO I = 1,MAX(ILP1,ILP2)
                  IF (IGJFLV(I).EQ.1.AND.QJ.GT.2D0*GAMS(I)
     $                 .AND.IABS(ID1-I).GT.0.AND.IABS(ID2-I).GT.0) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) =  ISIGN(I,KFCODE(1))
                     IDJBR2(IJBR) = -ISIGN(I,KFCODE(1))
                     TMPCKMR(IJBR) = 1.0D0
                     SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                  ENDIF
               ENDDO
C...Jets: same flavor with initial quarks.
            ELSE
               RNA = RNA*3.0D0-2.0D0
               XSCALE = XSCALE*3.0D0
               IJBR = 1
               SUMWWBR = 1.0D0
               TMPCKMR(IJBR) = 1.0D0
               IDJBR1(IJBR) = KFCODE(1)
               IDJBR2(IJBR) = KFCODE(2)
            ENDIF
         ENDIF
      ENDIF
C####################################
C ud -> WW + u'd' + (n-2) gluons
C####################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         DO I = 2,ILP1,2
            DO J = 1,ILP2,2
               IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                  IF (QJ.GT.GAMS(I)+GAMS(J)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = ISIGN(I,ID1)
                     IDJBR2(IJBR) = ISIGN(J,ID2)
                     CKMRA = 0.0D0
                     CKMRB = 0.0D0
                     DO K = 1,3
                        CKMRA = CKMRA+SQRT(GRCCKM(IGENE(IABS(ID1)),K))
     $                       *SQRT(GRCCKM(IGENE(I),K))
                        CKMRB = CKMRB+SQRT(GRCCKM(K,IGENE(IABS(ID2))))
     $                       *SQRT(GRCCKM(K,IGENE(J)))
                     ENDDO
                     CKMRC = SQRT(GRCCKM(IGENE(IABS(ID1)),IGENE(J)))
     $                    *SQRT(GRCCKM(IGENE(I),IGENE(IABS(ID2))))
                     TMPCKMR(IJBR) = (CKMRA**2+CKMRB**2+CKMRC**2)/3.0D0
                     SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDIF
C###################################
C qg -> WW + q + (n-1) gluons
C###################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.1) THEN
         ID1 = MIN(KFCODE(1),KFCODE(2))
         IJBR = 1
         IDJBR1(IJBR) = ID1
         SUMWWBR = 1.0D0
         TMPCKMR(IJBR) = 1.0D0
      ENDIF
C########################################
C qq'~ -> WW + q"q"'~ + (n-2) gluons + X
C########################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         DO I = 2,ILP1,2
            DO J = 1,ILP2,2
               IF (IGJFLV(I).EQ.1.OR.IGJFLV(J).EQ.1) THEN
                  IF (QJ.GT.GAMS(I)+GAMS(J)) THEN
                     IJBR = IJBR + 1
                     IDJBR1(IJBR) = ISIGN(I,ID1)
                     IDJBR2(IJBR) = ISIGN(J,ID2)
                     CKMRA = 0.0D0
                     CKMRB = 0.0D0
                     DO K = 1,3
                        CKMRA = CKMRA+SQRT(GRCCKM(IGENE(IABS(ID1)),K))
     $                       *SQRT(GRCCKM(IGENE(I),K))
                        CKMRB = CKMRB+SQRT(GRCCKM(K,IGENE(IABS(ID2))))
     $                       *SQRT(GRCCKM(K,IGENE(J)))
                     ENDDO
                     CKMRC = SQRT(GRCCKM(IGENE(IABS(ID1)),IGENE(J)))
     $                    *SQRT(GRCCKM(IGENE(I),IGENE(IABS(ID2))))
                     TMPCKMR(IJBR) = (CKMRA**2+CKMRB**2+CKMRC**2)/3.0D0
                     SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDIF
C####################################
C qq -> WW + q'q' + (n-2) gluons + X
C####################################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.2) THEN
         ID1 = ISIGN(MIN(IABS(KFCODE(1)),IABS(KFCODE(2))),KFCODE(1))
         ID2 = ISIGN(MAX(IABS(KFCODE(1)),IABS(KFCODE(2))),KFCODE(1))
C...qq' -> qq'
         IF (RNA.LT.1.0D0/2.0D0) THEN
            RNA = RNA*2.0D0
            XSCALE = XSCALE*2.0D0
            IF (IGJFLV(IABS(ID1)).EQ.1.OR.IGJFLV(IABS(ID2)).EQ.1) THEN
               IF (QJ.GT.GAMS(IABS(ID1))+GAMS(IABS(ID2))) THEN
                  IJBR = 1
                  SUMWWBR = 1.0D0
                  TMPCKMR(IJBR) = 1.0D0
                  IDJBR1(IJBR) = ID1
                  IDJBR2(IJBR) = ID2
               ENDIF
            ENDIF
         ELSE
            RNA = RNA*2.0D0-1.0D0
            XSCALE = XSCALE*2.0D0
C...uu->uc,uc->uu,cu->uu,cc->cu
            IF (MOD(IABS(ID1),2).EQ.0) THEN
               DO I = 2,ILP1,2
                  IF (IABS(ID2-I).GT.0.AND.IGJFLV(I).EQ.1) THEN
                     IF (QJ.GT.GAMS(IABS(ID1))+GAMS(I)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = ID1
                        IDJBR2(IJBR) = ISIGN(I,ID2)
                        DO J = 1,3
                           TMPCKMR(IJBR) = TMPCKMR(IJBR) 
     $                          + SQRT(GRCCKM(IGENE(IABS(ID2)),J))
     $                          *SQRT(GRCCKM(IGENE(I),J))
                        ENDDO
                        TMPCKMR(IJBR) = TMPCKMR(IJBR)**2
                        SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                     ENDIF
                  ENDIF
               ENDDO
            ELSE
C...dd->(ds,db),ds->(dd,db),sd->(dd,db),db->(dd,ds),bd->(dd,ds)
C...ss->(sd,sb),sb->(sd,ss),bs->(sd,ss)
C...bb->(bd,bs)
               DO I = 1,ILP2,2
                  IF (IABS(ID2-I).GT.0.AND.IGJFLV(I).EQ.1) THEN
                     IF (QJ.GT.GAMS(IABS(ID1))+GAMS(I)) THEN
                        IJBR = IJBR + 1
                        IDJBR1(IJBR) = ID1
                        IDJBR2(IJBR) = ISIGN(I,ID2)
                        DO J = 1,3
                           TMPCKMR(IJBR) = TMPCKMR(IJBR) 
     $                          + SQRT(GRCCKM(J,IGENE(IABS(ID2))))
     $                          *SQRT(GRCCKM(J,IGENE(I)))
                        ENDDO
                        TMPCKMR(IJBR) = TMPCKMR(IJBR)**2
                        SUMWWBR = TMPCKMR(IJBR) + SUMWWBR
                     ENDIF
                  ENDIF
               ENDDO
            ENDIF
         ENDIF
      ENDIF

C...If all branch are off.
      IF (IJBR.EQ.0) GO TO 999

C...Pick up final state flavor.
      RANSUM = RNA*SUMWWBR
      IDCYST = 0
      DO WHILE (RANSUM.GT.0.0D0)
         IDCYST = IDCYST + 1
         RANSUM = RANSUM - TMPCKMR(IDCYST)
      ENDDO
      JOUT(1) = IDJBR1(IDCYST)
      JOUT(2) = IDJBR2(IDCYST)
      JOUT(3) = IDJBR3(IDCYST)
      JOUT(4) = IDJBR4(IDCYST)

C...Output Jacobian factor.
      SUMWWBR = XSCALE*SUMWWBR

      RETURN
 999  CONTINUE
      IREJ = 3
      SUMWWBR = 0.0D0
      RETURN
      END

C-------------------------
C Set flavors in W decay.
C-------------------------
      SUBROUTINE GRCWDCY(NJET,KOUT,RNA,SUMWBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,ID1,ID2
      INTEGER NJET,NGLU,ICHOUT
      INTEGER KOUT(4),IGENE(6)
      DOUBLE PRECISION RNA,SUMWBR
      DOUBLE PRECISION XSND(100)
      DOUBLE PRECISION GRCCEWK,WIDTMP
      EXTERNAL GRCCEWK

C...Initialize
      SUMWBR = 1.0D0
      KOUT(1) = 0
      KOUT(2) = 0
      KOUT(3) = 0
      KOUT(4) = 0

C...Set branching ration of W.
      XSND(1) = 0.0D0
      DO I = 2,IGNDCY+1
         XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
      ENDDO

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Sum of intial and final state charges
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      IF (NJET.GT.0) THEN
         DO I = 1,NJET
            IF (KCHARG(I+4).EQ.0) NGLU = NGLU + 1
            ICHOUT = KCHARG(I+4) + ICHOUT
         ENDDO
      ENDIF
      ICHOUT = IABS(ICHOUT)

C...Make W decay.
      DO I = 1,IGNDCY
         IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
            SUMWBR = SUMWBR/(XSND(I+1)-XSND(I))
            KOUT(1) = IGWDCY(2*GDCYPR1(I)-1)
            KOUT(2) = IGWDCY(2*GDCYPR1(I))
         ENDIF
      ENDDO
      ID1 = IABS(KOUT(2))       ! up-type
      ID2 = IABS(KOUT(1))       ! down-type

C...Weight factor.
      WIDTMP = 0.0D0
      DO I = 1,20
         IF (IGWMOD(I).EQ.1) THEN
            WIDTMP = GRCCEWK(1,IGWDCY(2*I-1)) + WIDTMP
         ENDIF
      ENDDO

C...Colour freedom & CKM.
      IF (IABS(KOUT(1)).LT.7) THEN
         SUMWBR = SUMWBR*3.0D0*GRCCKM(IGENE(ID1),IGENE(ID2))
      ENDIF

C...Apply width correction.
      IF (IWIDCOR.EQ.2) THEN
         SUMWBR = SUMWBR*AGW/WIDTMP*GDCYSUM
         IF (IABS(KOUT(1)).LT.7) THEN
            SUMWBR = SUMWBR/3.0D0/GRCCKM(IGENE(ID1),IGENE(ID2))
         ENDIF
      ENDIF

C...Decide W charge according to the jet flavors.
C##########################
C ud~ -> W+ + n gluons + X
C##########################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
         ENDIF
      ENDIF
C########################################
C qq'~ -> W+ + q"q"'~ + (n-2) gluons + X
C########################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
         ENDIF
      ENDIF
C##############################################
C ud~ -> W+ + uu~qq~/dd~qq~ + (n-4) gluons + X
C##############################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.4) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
         ENDIF
      ENDIF
C###################################
C qg -> W+ + q'g + (n-2) gluons + X
C###################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.1) THEN
         ID1 = MIN(KFCODE(1),KFCODE(2))
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) = -ISIGN(IABS(KOUT(2)),ID1)
         ENDIF
      ENDIF
C################################################
C uu'/d~d'~ -> W+ + d"u/d~u"~ + (n-2) gluons + X
C################################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
         ENDIF
      ENDIF
C################################################
C qg  -> W+ + q'q"q"~/q'q'q'~ + (n-3) gluons + X
C q~g -> W+ + q~q~q'          + (n-3) gluons + X
C################################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.3) THEN
         IF (MOD(IABS(KFCODE(5)),2).EQ.0) THEN
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(5))
         ELSE
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(5))
         ENDIF
      ENDIF
C###########################################
C qq' -> W+ + q"qq"'q"'~ + (n-4) gluons + X
C###########################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.4) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
         ENDIF
      ENDIF
C#############################################
C ud/u~d~ -> W+ + d'd/u~u' + (n-2) gluons + X
C#############################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(5)),2).EQ.0) THEN
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(5))
         ELSE
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(5))
         ENDIF
      ENDIF
C#################################################
C ud/u~d~ -> W+ + d'd/u~u'qq'~ + (n-4) gluons + X
C#################################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.4) THEN
         IF (MOD(IABS(KFCODE(5)),2).EQ.0) THEN
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(5))
         ELSE
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(5))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(5))
         ENDIF
      ENDIF
C###############################################
C qq~/gg  -> W+ + q'q"~      + (n-2) gluons + X
C qq'~    -> W+ + qq"~/q"q'~ + (n-2) gluons + X
C###############################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(5))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(5))
      ENDIF
C######################################################
C qq~/gg  -> W+ + q'q"~q"'q"'~      + (n-4) gluons + X
C qq'~    -> W+ + qq"~/q"q'~q"'q"'~ + (n-4) gluons + X
C######################################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.4) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(5))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(5))
      ENDIF

      RETURN
      END

C-------------------------
C Set flavors in Z decay.
C-------------------------
      SUBROUTINE GRCZDCY(NJET,KOUT,RNA,SUMZBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,ID1,ID2
      INTEGER NJET,NGLU,ICHOUT
      INTEGER KOUT(4)
      DOUBLE PRECISION RNA,SUMZBR
      DOUBLE PRECISION XSND(100)
      DOUBLE PRECISION GRCCEWK,WIDTMP
      EXTERNAL GRCCEWK

C...Initialize
      SUMZBR = 1.0D0
      KOUT(1) = 0
      KOUT(2) = 0
      KOUT(3) = 0
      KOUT(4) = 0

C...Set branching ration of W.
      XSND(1) = 0.0D0
      DO I = 2,IGNDCY+1
         XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
      ENDDO

C...Sum of intial and final state charges
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      IF (NJET.GT.0) THEN
         DO I = 1,NJET
            IF (KCHARG(I+4).EQ.0) NGLU = NGLU + 1
            ICHOUT = KCHARG(I+4) + ICHOUT
         ENDDO
      ENDIF
      ICHOUT = IABS(ICHOUT)

C...Make Z decay.
      DO I = 1,IGNDCY
         IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
            SUMZBR = SUMZBR/(XSND(I+1)-XSND(I))
            KOUT(1) = IGZDCY(2*GDCYPR1(I)-1)
            KOUT(2) = IGZDCY(2*GDCYPR1(I))
         ENDIF
      ENDDO

C...Weight factor.
      WIDTMP = 0.0D0
      DO I = 1,16
         IF (IGZMOD(I).EQ.1) THEN
            WIDTMP = GRCCEWK(2,IGZDCY(2*I-1)) + WIDTMP
         ENDIF
      ENDDO

C...Colour freedom & CKM.
      IF (IABS(KOUT(1)).LT.7) THEN
         SUMZBR = SUMZBR*3.0D0
      ENDIF

C...Apply width correction.
      IF (IWIDCOR.EQ.2) THEN
         SUMZBR = SUMZBR*AGZ/WIDTMP*GDCYSUM
         IF (IABS(KOUT(1)).LT.7) THEN
            SUMZBR = SUMZBR/3.0D0
         ENDIF
      ENDIF

C...Decide Z charge according to the jet flavors.
C####################
C gg -> n gluons + X
C####################
      ID1 = KFCODE(1)
      ID2 = KFCODE(2)
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
      ENDIF
C#####################################
C qq'~/gg -> q"q"~ + (n-2) gluons + X
C#####################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
      ENDIF
C#############################
C ud -> ud + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
      ENDIF
C#############################
C qg -> qg + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.1) THEN
         IF (MOD(IABS(MIN(ID1,ID2)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
         ENDIF
      ENDIF
C#############################
C qg -> qg + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.3) THEN
         IF (MOD(IABS(MIN(ID1,ID2)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
         ENDIF
      ENDIF
C###############################
C ud~ -> ud~ + (n-2) gluons + X
C###############################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) = -ISIGN(IABS(KOUT(2)),ID1)
         ENDIF
      ENDIF
C#############################
C uu -> uu + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
      ENDIF

      RETURN
      END

C-------------------------
C Set flavors in ZZ decay.
C-------------------------
      SUBROUTINE GRCZZDCY(NJET,KOUT,RNA,SUMZZBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,ID1,ID2
      INTEGER NJET,NGLU,ICHOUT
      INTEGER KOUT(4)
      DOUBLE PRECISION RNA,SUMZZBR
      DOUBLE PRECISION XSND(100)
      DOUBLE PRECISION GRCCEWK,WIDTMP
      EXTERNAL GRCCEWK

C...Initialize
      SUMZZBR = 1.0D0
      KOUT(1) = 0
      KOUT(2) = 0
      KOUT(3) = 0
      KOUT(4) = 0

C...Set branching ration of W.
      XSND(1) = 0.0D0
      DO I = 2,IGNDCY+1
         XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
      ENDDO

C...Sum of intial and final state charges
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      IF (NJET.GT.0) THEN
         DO I = 1,NJET
            IF (KCHARG(I+6).EQ.0) NGLU = NGLU + 1
            ICHOUT = KCHARG(I+6) + ICHOUT
         ENDDO
      ENDIF
      ICHOUT = IABS(ICHOUT)

C...Make Z decay.
      DO I = 1,IGNDCY
         IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
            SUMZZBR = SUMZZBR/(XSND(I+1)-XSND(I))
            KOUT(1) = IGZDCY(2*GDCYPR1(I)-1)
            KOUT(2) = IGZDCY(2*GDCYPR1(I))
            KOUT(3) = IGZDCY(2*GDCYPR2(I)-1)
            KOUT(4) = IGZDCY(2*GDCYPR2(I))
         ENDIF
      ENDDO

C...Weight factor.
      WIDTMP = 0.0D0
      DO I = 1,16
         IF (IGZMOD(I).EQ.1) THEN
            WIDTMP = GRCCEWK(2,IGZDCY(2*I-1)) + WIDTMP
         ENDIF
         WIDTMP = WIDTMP**2
      ENDDO

C...Colour freedom & CKM.
      IF (IABS(KOUT(1)).LT.7) SUMZZBR = SUMZZBR*3.0D0
      IF (IABS(KOUT(3)).LT.7) SUMZZBR = SUMZZBR*3.0D0

C...Apply width correction.
      IF (IWIDCOR.EQ.2) THEN
         SUMZZBR = SUMZZBR*GDCYSUM*AGZ**2/WIDTMP
         IF (IABS(KOUT(1)).LT.7) SUMZZBR = SUMZZBR/3.0D0
         IF (IABS(KOUT(3)).LT.7) SUMZZBR = SUMZZBR/3.0D0
      ENDIF

C...Decide Z charge according to the jet flavors.
C####################
C gg -> n gluons + X
C####################
      ID1 = KFCODE(1)
      ID2 = KFCODE(2)
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
         KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
      ENDIF
C#####################################
C qq'~/gg -> q"q"~ + (n-2) gluons + X
C#####################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
         KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
      ENDIF
C#############################
C ud -> ud + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
         KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
      ENDIF
C#############################
C qg -> qg + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.1) THEN
         IF (MOD(IABS(MIN(ID1,ID2)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
            KOUT(3) = -ISIGN(IABS(KOUT(3)),MIN(ID1,ID2))
            KOUT(4) =  ISIGN(IABS(KOUT(4)),MIN(ID1,ID2))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),MIN(ID1,ID2))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),MIN(ID1,ID2))
            KOUT(3) =  ISIGN(IABS(KOUT(3)),MIN(ID1,ID2))
            KOUT(4) = -ISIGN(IABS(KOUT(4)),MIN(ID1,ID2))
         ENDIF
      ENDIF
C###############################
C ud~ -> ud~ + (n-2) gluons + X
C###############################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) = -ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) =  ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) = -ISIGN(IABS(KOUT(4)),ID1)
         ENDIF
      ENDIF
C#############################
C uu -> uu + (n-2) gluons + X
C#############################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
         KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
         KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
         KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
      ENDIF

      RETURN
      END

C-------------------------
C Set flavors in WW decay.
C-------------------------
      SUBROUTINE GRCWWDCY(NJET,KOUT,RNA,SUMWWBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,ID1,ID2
      INTEGER NJET,NGLU,ICHOUT
      INTEGER KOUT(4),IGENE(6)
      DOUBLE PRECISION RNA,SUMWWBR
      DOUBLE PRECISION XSND(100)
      DOUBLE PRECISION GRCCEWK,WIDTMP,WIDTMP1,WIDTMP2
      EXTERNAL GRCCEWK

C...Initialize
      SUMWWBR = 1.0D0
      KOUT(1) = 0
      KOUT(2) = 0
      KOUT(3) = 0
      KOUT(4) = 0

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Set branching ration of W.
      XSND(1) = 0.0D0
      DO I = 2,IGNDCY+1
         XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
      ENDDO

C...Sum of intial and final state charges
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      IF (NJET.GT.0) THEN
         DO I = 1,NJET
            IF (KCHARG(I+6).EQ.0) NGLU = NGLU + 1
            ICHOUT = KCHARG(I+6) + ICHOUT
         ENDDO
      ENDIF
      ICHOUT = IABS(ICHOUT)

C...Make WW decay.
      DO I = 1,IGNDCY
         IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
            SUMWWBR = SUMWWBR/(XSND(I+1)-XSND(I))
            KOUT(1) = IGWDCY(2*GDCYPR1(I)-1)
            KOUT(2) = IGWDCY(2*GDCYPR1(I))
            KOUT(3) = -IGWDCY(2*GDCYPR2(I)-1)
            KOUT(4) = -IGWDCY(2*GDCYPR2(I))
         ENDIF
      ENDDO

C...Weight factor.
      WIDTMP = 0.0D0
      DO I = 1,20
         IF (IGWMOD(I).EQ.1) THEN
            WIDTMP = GRCCEWK(1,IGWDCY(2*I-1)) + WIDTMP
         ENDIF
      ENDDO
      WIDTMP = WIDTMP**2

C...Colour freedom & CKM.
      IF (IABS(KOUT(1)).LT.7) THEN
         ID1 = IABS(KOUT(2))    ! up-type
         ID2 = IABS(KOUT(1))    ! down-type
         SUMWWBR = SUMWWBR*3.0D0*GRCCKM(IGENE(ID1),IGENE(ID2))
      ENDIF
      IF (IABS(KOUT(3)).LT.7) THEN
         ID1 = IABS(KOUT(4))    ! up-type
         ID2 = IABS(KOUT(3))    ! down-type
         SUMWWBR = SUMWWBR*3.0D0*GRCCKM(IGENE(ID1),IGENE(ID2))
      ENDIF

C...Apply width correction.
      IF (IWIDCOR.EQ.2) THEN
         SUMWWBR = SUMWWBR*GDCYSUM*AGW**2/WIDTMP
         IF (IABS(KOUT(1)).LT.7) THEN
            ID1 = IABS(KOUT(2)) ! up-type
            ID2 = IABS(KOUT(1)) ! down-type
            SUMWWBR = SUMWWBR/3.0D0/GRCCKM(IGENE(ID1),IGENE(ID2))
         ENDIF
         IF (IABS(KOUT(3)).LT.7) THEN
            ID1 = IABS(KOUT(4)) ! up-type
            ID2 = IABS(KOUT(3)) ! down-type
            SUMWWBR = SUMWWBR/3.0D0/GRCCKM(IGENE(ID1),IGENE(ID2))
         ENDIF
      ENDIF

C...Decide WW charge according to the jet flavors.
C###########################
C qq'~ -> WW + n gluons + X
C###########################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
         KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
      ENDIF
C###############################################
C qq'~/gg -> WW + q"q"~/q"q' + (n-2) gluons + X
C###############################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
         KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
      ENDIF
C####################################
C ud -> WW + u'd' + (n-2) gluons + X
C####################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
         KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
      ENDIF
C###################################
C qg -> WW + q'g + (n-2) gluons + X
C###################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.1) THEN
         ID1 = MIN(KFCODE(1),KFCODE(2))
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) =  ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) = -ISIGN(IABS(KOUT(4)),ID1)
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) = -ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
         ENDIF
      ENDIF
C########################################
C qq'~ -> WW + q"q"'~ + (n-2) gluons + X
C########################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
         KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
      ENDIF
C####################################
C qq -> WW + q'q' + (n-2) gluons + X
C####################################
      IF (ICHOUT.EQ.4.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
         KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
         KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
      ENDIF

      RETURN
      END

C-------------------------
C Set flavors in ZW decay.
C-------------------------
      SUBROUTINE GRCZWDCY(NJET,KOUT,RNA,SUMZWBR)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,ID1,ID2
      INTEGER NJET,NGLU,ICHOUT
      INTEGER KOUT(4),IGENE(6)
      DOUBLE PRECISION RNA,SUMZWBR
      DOUBLE PRECISION XSND(100)
      DOUBLE PRECISION GRCCEWK,WIDTMP,WIDTMP1,WIDTMP2
      EXTERNAL GRCCEWK

C...Initialize
      SUMZWBR = 1.0D0
      KOUT(1) = 0
      KOUT(2) = 0
      KOUT(3) = 0
      KOUT(4) = 0

C...quark generation.
      DO I = 1,5,2
         IGENE(I)   = (I+1)/2
         IGENE(I+1) = (I+1)/2
      ENDDO

C...Set branching ration of W.
      XSND(1) = 0.0D0
      DO I = 2,IGNDCY+1
         XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
      ENDDO

C...Sum of intial and final state charges
      ICHOUT = 0
      NGLU = 0                  ! number of gluons.
      IF (NJET.GT.0) THEN
         DO I = 1,NJET
            IF (KCHARG(I+6).EQ.0) NGLU = NGLU + 1
            ICHOUT = KCHARG(I+6) + ICHOUT
         ENDDO
      ENDIF
      ICHOUT = IABS(ICHOUT)

C...Make ZW decay.
      DO I = 1,IGNDCY
         IF (RNA.GT.XSND(I).AND.RNA.LT.XSND(I+1)) THEN
            SUMZWBR = SUMZWBR/(XSND(I+1)-XSND(I))
            KOUT(1) = IGZDCY(2*GDCYPR1(I)-1)
            KOUT(2) = IGZDCY(2*GDCYPR1(I))
            KOUT(3) = IGWDCY(2*GDCYPR2(I)-1)
            KOUT(4) = IGWDCY(2*GDCYPR2(I))
         ENDIF
      ENDDO
      ID1 = IABS(KOUT(4))       ! up-type
      ID2 = IABS(KOUT(3))       ! down-type

C...Weight factor.
      WIDTMP1 = 0.0D0
      DO I = 1,16
         IF (IGZMOD(I).EQ.1) THEN
            WIDTMP1 = GRCCEWK(2,IGZDCY(2*I-1)) + WIDTMP1
         ENDIF
      ENDDO
      WIDTMP2 = 0.0D0
      DO I = 1,20
         IF (IGWMOD(I).EQ.1) THEN
            WIDTMP2 = GRCCEWK(1,IGWDCY(2*I-1)) + WIDTMP2
         ENDIF
      ENDDO
      WIDTMP = WIDTMP1*WIDTMP2

C...Colour freedom & CKM.
      IF (IABS(KOUT(1)).LT.7) THEN
         SUMZWBR = SUMZWBR*3.0D0
      ENDIF
      IF (IABS(KOUT(3)).LT.7) THEN
         SUMZWBR = SUMZWBR*3.0D0*GRCCKM(IGENE(ID1),IGENE(ID2))
      ENDIF

C...Apply width correction.
      IF (IWIDCOR.EQ.2) THEN
         SUMZWBR = SUMZWBR*GDCYSUM*AGZ*AGW/WIDTMP
         IF (IABS(KOUT(1)).LT.7) THEN
            SUMZWBR = SUMZWBR/3.0D0
         ENDIF
         IF (IABS(KOUT(3)).LT.7) THEN
            SUMZWBR = SUMZWBR/3.0D0/GRCCKM(IGENE(ID1),IGENE(ID2))
         ENDIF
      ENDIF

C...Decide ZW charge according to the jet flavors.
C##########################
C ud~ -> W+ + n gluons + X
C##########################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.0) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) = -ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) =  ISIGN(IABS(KOUT(4)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
         ENDIF
      ENDIF
C########################################
C qq'~ -> W+ + q"q"'~ + (n-2) gluons + X
C########################################
      IF (ICHOUT.EQ.0.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) = -ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) =  ISIGN(IABS(KOUT(4)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
         ENDIF
      ENDIF
C###################################
C qg -> W+ + q'g + (n-2) gluons + X
C###################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.1) THEN
         ID1 = MIN(KFCODE(1),KFCODE(2))
         IF (MOD(IABS(ID1),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) =  ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) = -ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) =  ISIGN(IABS(KOUT(4)),ID1)
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),ID1)
            KOUT(2) = -ISIGN(IABS(KOUT(2)),ID1)
            KOUT(3) =  ISIGN(IABS(KOUT(3)),ID1)
            KOUT(4) = -ISIGN(IABS(KOUT(4)),ID1)
         ENDIF
      ENDIF
C################################################
C uu'/d~d'~ -> W+ + d"u/d~u"~ + (n-2) gluons + X
C################################################
      IF (ICHOUT.EQ.1.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(1)),2).EQ.0) THEN
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) = -ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) =  ISIGN(IABS(KOUT(4)),KFCODE(1))
         ELSE
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(1))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(1))
            KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(1))
            KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(1))
         ENDIF
      ENDIF
C#############################################
C ud/u~d~ -> W+ + d'd/u~u' + (n-2) gluons + X
C#############################################
      IF (ICHOUT.EQ.2.AND.NJET-NGLU.EQ.2) THEN
         IF (MOD(IABS(KFCODE(7)),2).EQ.0) THEN
            KOUT(1) =  ISIGN(IABS(KOUT(1)),KFCODE(7))
            KOUT(2) = -ISIGN(IABS(KOUT(2)),KFCODE(7))
            KOUT(3) =  ISIGN(IABS(KOUT(3)),KFCODE(7))
            KOUT(4) = -ISIGN(IABS(KOUT(4)),KFCODE(7))
         ELSE
            KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(7))
            KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(7))
            KOUT(3) = -ISIGN(IABS(KOUT(3)),KFCODE(7))
            KOUT(4) =  ISIGN(IABS(KOUT(4)),KFCODE(7))
         ENDIF
      ENDIF
C###############################################
C qq~/gg  -> W+ + q'q"~      + (n-2) gluons + X
C qq'~    -> W+ + qq"~/q"q'~ + (n-2) gluons + X
C###############################################
      IF (ICHOUT.EQ.3.AND.NJET-NGLU.EQ.2) THEN
         KOUT(1) = -ISIGN(IABS(KOUT(1)),KFCODE(7))
         KOUT(2) =  ISIGN(IABS(KOUT(2)),KFCODE(7))
         KOUT(3) = -ISIGN(IABS(KOUT(3)),KFCODE(7))
         KOUT(4) =  ISIGN(IABS(KOUT(4)),KFCODE(7))
      ENDIF

      RETURN
      END

C-----------------------------------
C Set integration/event generation.
C-----------------------------------
      SUBROUTINE GRCPYGEN(PRT,ISUB,INIT,SIGEV)
      IMPLICIT REAL*8(A-H,O-Z)
      INCLUDE 'grchad.inc'
      common/PSKfail/ ifail_pskini,nfail_pskini
      character PRT*3

C  Make sure to have a positive "signed resust" for processes in which 
C  "matching" is not supported.
      signedres= 1.d0

      nfail= 0
 1    continue
      ifail_pskini= 0
      call GRCPYGEN1(PRT,ISUB,INIT,SIGEV)

C  Handle the error return from "pskini".
      if( INIT.eq.0 .and. ifail_pskini.ne.0 ) then
         nfail= nfail +1
         if( nfail.ge.10 ) then
            print *,'GRCPYGEN: PSKINI failed 10 times.'
            stop
         endif
         nfail_pskini= nfail_pskini +1
         goto 1
      endif

      return
      end

      SUBROUTINE GRCPYGEN1(PRT,ISUB,INIT,SIGEV)
      IMPLICIT REAL*8(A-H,O-Z)
CCC      IMPLICIT NONE

#include "define.h"

      INCLUDE 'grchad.inc'
      common/QPlock/xpdf_lock(2),iqplock
      common/PSKfail/ ifail_pskini,nfail_pskini

C+------------------------------------------------+
#ifdef VBASESMPI
      INCLUDE 'inclbp.h'
#endif
C+------------------------------------------------+

C...Include HEP common parameter.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/

      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      INTEGER JFLAG,IBASES
      COMMON /BASE0/ JFLAG,IBASES
      INTEGER MXDIM,NDMX,NDIM,NWILD,IG,NCALL,ITMX1,ITMX2
      DOUBLE PRECISION XL,XU,ACC1,ACC2
      PARAMETER (MXDIM = 50, NDMX = 50 )
      COMMON /BPARM1/ XL(MXDIM),XU(MXDIM),NDIM,NWILD,
     $     IG(MXDIM),NCALL
      COMMON /BPARM2/ ACC1,ACC2,ITMX1,ITMX2

      INTEGER IT1,IT2

      INTEGER JPROC
      COMMON /AMJPRC/JPROC

      CHARACTER GRCPROC*40
      COMMON/GRCCHAR/GRCPROC

      DOUBLE PRECISION FUNC,X(50)
      EXTERNAL FUNC
      INTEGER GRCCDCY
      EXTERNAL GRCCDCY
 
      CHARACTER XSTR*14
      INTEGER ISUB,INIT
      INTEGER I,J,NI,NJ,MXTRY
      DOUBLE PRECISION SIGEV,ESTIM,SIGMA,CTIME
      CHARACTER GRCFILE*60,PRT*3
      CHARACTER GRESO*60,GDATA*60,RESO*7,BDAT*5,BLANK*1
      CHARACTER*60 GRCBSFILE(500)
      SAVE GRCBSFILE
      LOGICAL FOREVER
      PARAMETER (FOREVER=.TRUE.)
      DATA BLANK/' '/, RESO/'.result'/, BDAT/'.data'/
      INTEGER ICHCOUP,ICHFAQ,ICHSUB
      INTEGER IDCY,ICHIDCY,ICHNDCY(40),ICHMDCY(40),IDCYTMP
      CHARACTER TMPA*40,TMPB*6,TMPC*2,TMPD*60,TMPE*20,OPTF*60,COLP*30
      DOUBLE PRECISION CHECM,CHPTCUT(8),CHETACUT(8),CHRCONCUT(8)
      DOUBLE PRECISION CHMETCUT,CHBDCY(40)
      DOUBLE PRECISION CRGET,CHQ2,CHFAQ2,DUMMY,XDUMMY

C...Set Subprocess ID
      IGSUB = ISUB
C...Initialization of BASES.
      IBSINT = INIT

#ifndef GRCDEBUG
      CALL BSINIT
      CALL GRCPYPRO(ISUB,GRCFILE)
      CALL GRCBSIT
#endif

C...Integration (ibsint=1)
      IF (IBSINT.EQ.1) THEN

C...Set Collision Particles
         IF (PRT(1:2).EQ.'PP') THEN
            IGPP = 0
         ELSE IF (PRT(1:2).EQ.'PA') THEN
            IGPP = 1
         ENDIF

C...Making Title.
C+------------------------------------------------+
#ifdef VBASESMPI
         IF (MYPID.EQ.0) THEN
#endif
C+------------------------------------------------+
         CALL GRCLOGO(6)
         WRITE(*,*) 'Accepted CM Energy :', GRCECM, 'GeV'
         IF (IGPP.EQ.0) THEN
            WRITE(*,*) '         Beam type : ',
     $           'Proton-Proton Collision\n'
         ELSE IF (IGPP.EQ.1) THEN
            WRITE(*,*) '         Beam type : ',
     $           'Proton-Anti-Proton Collision\n'
         ELSE
            WRITE(*,*) 'Error::GRCPYGEN: Wrong parameter. Put PP/PAP.'
            GO TO 999
         ENDIF
C+------------------------------------------------+
#ifdef VBASESMPI
         ENDIF
#endif
C+------------------------------------------------+
 
C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
         CALL BSINIT
         CALL GRCPYPRO(ISUB,GRCFILE)
         CALL GRCBSIT
#endif
C+-------------------------------------------------+

C...Make Decay Table.
         AMX = 1.0D5
         AMY = 1.0D5
         CALL SETMAS
         CALL GRCSDCY

C...Init ICHMDCY(I)
         IF (GRCCDCY(IGSUB).GT.0) THEN
            ICHIDCY = 0
            DO I = 1,40
               ICHNDCY(I) = 0
               ICHMDCY(I) = 10
               CHBDCY(I) = 0
            ENDDO
         ENDIF

C...Set BASES Output File Name
         J = 0
 100     J = J + 1
         IF (GRCFILE(J+1:J+1).NE.BLANK) GO TO 100
         GRESO = GRCFILE(1:J)//RESO
         GDATA = GRCFILE(1:J)//BDAT
         GRCBSFILE(JPROC) = GDATA
 
C...Make BASES File (ibswrt=0)
         IF (IBSWRT.EQ.0) THEN
C...Initialize kinematics.
            CALL GRCIKIN
            CALL GRCSPID
C+------------------------------------------------+
#ifdef VBASESMPI
            IF (MYPID.EQ.0) THEN
#endif
C+------------------------------------------------+
            WRITE(*,*) 'Process : [',IGSUB,'] ',GRCPROC,'\n'
            WRITE(*,*) 'Set BASES file'
            WRITE(*,*) '   Filename : ',GRESO
            WRITE(*,*) '              ',GDATA,'\n' 
            WRITE(*,*) NNF-2,' body final state'
            DO I=1,NNF-2
               IF (GRCPTCUT(I).GT.0.0) THEN
                  WRITE(*,*) '\tPt Cut of Particle',I,
     $                 ' : ',GRCPTCUT(I),' GeV'
               ELSE
                  WRITE(*,*) '\tPt Cut of Particle',I,' : none'
               ENDIF
            ENDDO
            DO I=1,NNF-2
               IF (GRCETACUT(I).LT.100.0) THEN
                  WRITE(*,*) '\tEta Cut of Particle',I,
     $                 ' : ',GRCETACUT(I)
               ELSE
                  WRITE(*,*) '\tEta Cut of Particle',I,' : none'
               ENDIF
            ENDDO
            DO I=1,NNF-2
               IF (GRCRCONCUT(I).GT.0.0) THEN
                  WRITE(*,*) '\tRcone Cut of Particle',I,
     $                 ' : ',GRCRCONCUT(I)
               ELSE
                  WRITE(*,*) '\tRcone Cut of Particle',I,' : none'
               ENDIF
            ENDDO
            IF (GRCMETCUT.GT.0.0) THEN
               WRITE(*,*) '\tMissing Pt(Et) Cut : ',GRCMETCUT,' GeV\n'
            ELSE
               WRITE(*,*) '\tMissing Pt(Et) Cut : none\n'
            ENDIF
            IF (ICOUP.EQ.5) THEN
               WRITE(*,*) '\tOption of Renormalization Scale :',
     $              ICOUP,GRCQ,'GeV'
            ELSE
               WRITE(*,*) '\tOption of Renormalization Scale :',ICOUP
            ENDIF
            IF (IFACT.EQ.0) THEN
               WRITE(*,*) '\tOption of   Factorization Scale :',IFACT
            ELSE IF (IFACT.EQ.5) THEN
               WRITE(*,*) '\tOption of   Factorization Scale :',
     $              IFACT,GRCFAQ,'GeV'
            ELSE
               WRITE(*,*) '\tOption of   Factorization Scale :',IFACT
            ENDIF
C...Check W-decay.
            IF (GRCCDCY(IGSUB).EQ.1.OR.GRCCDCY(IGSUB).EQ.3) THEN
               WRITE(*,*)
               DO I = 1,20
                  IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.000) THEN
                     CALL GRCRDCY(1,I,TMPE)
                     WRITE(*,FMT='(a24,i5,f8.3,a20)') 
     $                    'W Forced Decay :',I,GRCWBR(I),TMPE
                  ENDIF
               ENDDO
            ENDIF
C...Check Z-decay.
            IF (GRCCDCY(IGSUB).EQ.2.OR.GRCCDCY(IGSUB).EQ.3) THEN
               WRITE(*,*)
               DO I = 1,16
                  IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.000) THEN
                     CALL GRCRDCY(2,I,TMPE)
                     WRITE(*,FMT='(a24,i5,f8.3,a20)') 
     $                    'Z Forced Decay :',I,GRCZBR(I),TMPE
                  ENDIF
               ENDDO
            ENDIF

C...Open bases.result file. 
            OPEN(11,FILE = GRESO,
     $           STATUS='unknown',FORM='formatted')
 
C...BASES Integration
            WRITE(*,*)
            WRITE(*,*) 'Start BASES Integration!!'

C+------------------------------------------------+
#ifdef VBASESMPI
            ENDIF
#endif
C+------------------------------------------------+

            DO I = 1, NDIM
               WRITE(XSTR, 120) I
 120           FORMAT('x(',I2,') spectrum')
               CALL XHINIT(I, XL(I), XU(I), 50, XSTR)
            ENDDO

C+------------------------------------------------+
#ifndef VBASESMPI
            CALL BASES( FUNC, ESTIM, SIGMA, CTIME, IT1, IT2 )
#endif
C+------------------------------------------------+

C+------------------------------------------------+
#ifdef VBASESMPI
            CALL VBASES( FUNC, ESTIM, SIGMA, CTIME, IT1, IT2 )
#endif
C+------------------------------------------------+

            GRCSIG(JPROC) = ESTIM

C+------------------------------------------------+
#ifdef VBASESMPI
            IF (MYPID.EQ.0) THEN
#endif
C+------------------------------------------------+
            CALL BSINFO(11)
            CALL BHPLOT(11)
            WRITE(11,*) 'Summary Table'
            WRITE(11,*) '============='
            CALL GRCBSRS(11)
            WRITE(11,*) ' Num. of Mapping for Integration : ', INBS
            WRITE(11,*) ' Num. of Variable Rejection      : ', INKV
            WRITE(11,*) ' Num. of Cut Rejection           : ', INKC
            WRITE(11,*) ' Num. of Kinematical Rejection   : ', IN4M
            WRITE(11,*) ' Num. of Additional Rejection    : ', INAC
            WRITE(11,*) '==========================================='
            WRITE(11,FMT='(a24,d15.6,a5)') ' Total Cross Section : ',
     $           GRCSIG(JPROC),' pb'
            CLOSE(11)

            OPEN(23,FILE = GDATA,
     $           STATUS='unknown',FORM='unformatted')
            CALL BSWRIT(23)
            CLOSE (23)

C+------------------------------------------------+
#ifdef VBASESMPI
            ENDIF
#endif
C+------------------------------------------------+
 
C...Read BASES File (ibswrt=1)
         ELSE IF (IBSWRT.EQ.1) THEN
C...Initialize of Kinematics
            CALL GRCIKIN
            CALL GRCSPID

C...Check forced decay
C...Check W-decay.
            IF (GRCCDCY(IGSUB).EQ.1.OR.GRCCDCY(IGSUB).EQ.3) THEN
               IDCYTMP = 0
               DO I = 1,20
                  IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
                     IDCYTMP = IDCYTMP + 1
                  ENDIF
               ENDDO
            ENDIF
 
C...Check Z-decay.
            IF (GRCCDCY(IGSUB).EQ.2.OR.GRCCDCY(IGSUB).EQ.3) THEN
               IDCYTMP = 0
               DO I = 1,16
                  IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.0D0) THEN
                     IDCYTMP = IDCYTMP + 1
                  ENDIF
               ENDDO
            ENDIF

            WRITE(*,*) 'Read from BASES file : ',GRESO,'\n'
            OPEN(11,FILE = GRESO,
     $           STATUS='old',FORM='formatted')

            DO WHILE (FOREVER)
               READ(11,'(a)',END=1000) OPTF
 
               IF (OPTF(2:8).EQ.'Summary') THEN
                  READ(11,'(a)',END=1000)
                  READ(11,'(a14,f10.3,a4)',END=1000)
     $                 TMPA,CHECM,TMPB
                  READ(11,'(a14,a30)',END=1000)
     $                 TMPA,COLP
                  READ(11,'(a)',END=1000)
                  READ(11,'(a14,i3,a2,a40)',END=1000)
     $                 TMPA,ICHSUB,TMPC,TMPD
                  READ(11,'(a)',END=1000)
                  READ(11,'(i4,a17)',END=1000) NI,TMPA
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3,a4)',END=1000)
     $                    TMPA,NJ,TMPC,CHPTCUT(I),TMPB
                  ENDDO
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3)',END=1000)
     $                    TMPA,NJ,TMPC,CHETACUT(I)
                  ENDDO
                  DO I=1,NI
                     READ(11,'(a25,i2,a2,f7.3)',END=1000)
     $                    TMPA,NJ,TMPC,CHRCONCUT(I)
                  ENDDO
                  READ(11,'(a29,f7.3,a4)',END=1000)
     $                 TMPA,CHMETCUT,TMPB
                  READ(11,'(f3.1)',END=1000) DUMMY
                  IF (ICOUP.EQ.5) THEN
                     READ(11,'(a25,i2,f10.3,a2,a41)',END=1000)
     $                    TMPA,ICHCOUP,CHQ2,TMPB,TMPD
                  ELSE
                     READ(11,'(a25,i2,a41)',END=1000) 
     $                    TMPA,ICHCOUP,TMPD
                  ENDIF
                  IF (IFACT.EQ.0) THEN
                     READ(11,'(a25,i2,a41)',END=1000)
     $                    TMPA,ICHFAQ,TMPD
                  ELSE IF (IFACT.EQ.5) THEN
                     READ(11,'(a25,i2,f10.3,a2,a41)',END=1000)
     $                    TMPA,ICHFAQ,CHFAQ2,TMPB,TMPD
                  ELSE
                     READ(11,'(a25,i2,a41)',END=1000)
     $                    TMPA,ICHFAQ,TMPD
                  ENDIF

C...Check all-decay.
                  IF (GRCCDCY(IGSUB).GT.0) THEN
                     IF (IDCYTMP.GT.0) THEN
                        READ(11,'(f3.1)',END=1000) DUMMY
                        READ(11,'(a20,i5)',END=1000) TMPA,ICHIDCY
                        DO I = 1,ICHIDCY
                           READ(11,'(a20,i5,f8.3,a20)',END=1000) 
     $                          TMPA,ICHNDCY(I),CHBDCY(I),TMPE
                           IF (TMPE(3:3).EQ.'W') ICHMDCY(I) = 0
                           IF (TMPE(3:3).EQ.'Z') ICHMDCY(I) = 1
                        ENDDO
                     ENDIF
                  ENDIF

C...Check Parameter for Event Generation
C...Check Ecm
                  IF (ABS(CHECM-GRCECM).GT.0.0001) THEN
                     WRITE(*,*) 'Different CM Energy :',
     $                    '(BASES file):',CHECM,
     $                    'GeV and (PROGRAM):',GRCECM,'GeV'
                     GO TO 999
                  ENDIF
C...Check Collision Particles
                  IF (IGPP.EQ.0
     $                 .AND.COLP(8:20).EQ.'Proton-Proton') THEN
                  ELSE IF (IGPP.EQ.1
     $                    .AND.COLP(3:15).EQ.'Proton-Anti-P') THEN
                  ELSE
                     WRITE(*,*) COLP
                     WRITE(*,*) 'Mismatch in the Collision Particles.'
                     GO TO 999
                  ENDIF
C...Check Process
                  IF (ICHSUB.EQ.IGSUB) THEN
                     WRITE(*,*) '  Process : [',IGSUB,'] ',GRCPROC
                     WRITE(*,*)
                  ELSE
                     WRITE(*,*) 'GRC::Error: Different subprocess',
     $                    ' (BASES file):',ICHSUB,' and (PROGRAM):',
     $                    IGSUB
                  ENDIF
C...Check Pt Cut
                  DO I=1,NI
                     IF (ABS(CHPTCUT(I)-GRCPTCUT(I)).LT.0.001) THEN
                        IF (GRCPTCUT(I).GT.0.0) THEN
                           WRITE(*,*) '\tPt Cut of Particle',I,
     $                          ' : ',GRCPTCUT(I),' GeV'
                        ELSE
                           WRITE(*,*) '\tPt Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Pt Cut for Particle',
     $                       I,' : (BASES file):',CHPTCUT(I),
     $                       'GeV and (PROGRAM):',GRCPTCUT(I),'GeV'
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Eta Cut
                  DO I=1,NI
                     IF (ABS(CHETACUT(I)-GRCETACUT(I)).LT.0.001) THEN
                        IF (GRCETACUT(I).LT.100.0) THEN
                           WRITE(*,*) '\tEta Cut of Particle',I,
     $                          ' : ',GRCETACUT(I)
                        ELSE
                           WRITE(*,*) '\tEta Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Eta Cut for Particle',
     $                       I,' : (BASES file):',CHETACUT(I),
     $                       ' and (PROGRAM):',GRCETACUT(I)
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Rcon Cut
                  DO I=1,NI
                     IF (ABS(CHRCONCUT(I)-GRCRCONCUT(I)).LT.0.001) THEN
                        IF (GRCRCONCUT(I).GT.0.0) THEN
                           WRITE(*,*) '\tRcone Cut of Particle',I,
     $                          ' : ',GRCRCONCUT(I)
                        ELSE
                           WRITE(*,*) '\tRcone Cut of Particle',I,
     $                          ' : none'
                        ENDIF
                     ELSE
                        WRITE(*,*) 'Different Rcone Cut for Particle',
     $                       I,' : (BASES file):',CHRCONCUT(I),
     $                       ' and (PROGRAM):',GRCRCONCUT(I)
                        GO TO 999
                     ENDIF
                  ENDDO
C...Check Missing Et(Pt) Cut
                  IF (ABS(CHMETCUT-GRCMETCUT).LT.0.001) THEN
                     IF (GRCMETCUT.GT.0.0) THEN
                        WRITE(*,*) '\tMissing Pt(Et) Cut : ',
     $                       GRCMETCUT,' GeV\n'
                     ELSE
                        WRITE(*,*) '\tMissing Pt(Et) Cut : none\n'
                     ENDIF
                  ELSE
                     WRITE(*,*) 'Different Missing Et(Pt) Cut :',
     $                    '(BASES file):',CHMETCUT,
     $                    'GeV and (PROGRAM):',GRCMETCUT,'GeV'
                     GO TO 999
                  ENDIF
 
                  WRITE(*,*)
C...Check Option of Renormalization Scale (GeV)
                  IF (ICHCOUP.EQ.ICOUP) THEN
                     WRITE(*,*) '\tOption of Renormalization Scale : ',
     $                    ICOUP
                  ELSE
                     WRITE(*,*)
     $                    'Different Option of Renormalization Scale :',
     $                    '(BASES file):',ICHCOUP,
     $                    ' and (PROGRAM):',ICOUP
                     GO TO 999
                  ENDIF
                  IF (ICHCOUP.EQ.5) THEN
                     IF (ABS(CHQ2-GRCQ).LT.0.001) THEN
                        WRITE(*,*) '\tFixed Renormalization Scale : ',
     $                       GRCQ,' GeV'
                     ELSE
                        WRITE(*,*)
     $                       'Different Renormalization Scale (GeV)',
     $                       ' or Option :','(BASES file):',CHQ2,
     $                       'GeV and (PROGRAM):',GRCQ,'GeV'
                        GO TO 999
                     ENDIF
                  ENDIF
 
C...Check Option of Factorization Scale (GeV)
                  IF (ICHFAQ.EQ.IFACT) THEN
                     WRITE(*,*) '\tOption of Factorization Scale : ',
     $                    IFACT
                  ELSE
                     WRITE(*,*)
     $                    'Different Option of Factorization Scale :',
     $                    '(BASES file):',ICHFAQ,
     $                    ' and (PROGRAM):',IFACT
                     GO TO 999
                  ENDIF
                  IF (ICHFAQ.EQ.0) THEN
                     WRITE(*,*) '\t    (taken as same as ',
     $                    'the renormalization scale.)'
                  ELSE IF (ICHFAQ.EQ.5) THEN
                     IF (ABS(CHFAQ2-GRCFAQ).LT.0.001) THEN
                        WRITE(*,*) '\tFixed Factorization Scale : ',
     $                       GRCFAQ,' GeV'
                     ELSE
                        WRITE(*,*)
     $                       'Different Factorization Scale (GeV)',
     $                       ' or Option :','(BASES file):',CHFAQ2,
     $                       'GeV and (PROGRAM):',GRCFAQ,'GeV'
                        GO TO 999
                     ENDIF
                  ENDIF

C...Check Forced Decay Option
                  IF (GRCCDCY(IGSUB).GT.0) THEN
                     IF (IDCYTMP.GT.0) THEN
                        WRITE(*,*)
                        DO I = 1,IDCYTMP
C...Check W Decay
                           IF (ICHMDCY(I).EQ.0
     $                          .AND.IGWMOD(ICHNDCY(I)).EQ.1
     $                          .AND.ABS(GRCWBR(ICHNDCY(I))-CHBDCY(I))
     $                          .LT.0.001) THEN
                              WRITE(*,*) '\tW decay :',ICHNDCY(I),
     $                             ' Br. ',GRCWBR(ICHNDCY(I)),' ON'
C...Check Z Decay
                           ELSE IF (ICHMDCY(I).EQ.1
     $                          .AND.IGZMOD(ICHNDCY(I)).EQ.1
     $                          .AND.ABS(GRCZBR(ICHNDCY(I))-CHBDCY(I))
     $                          .LT.0.001) THEN
                              WRITE(*,*) '\tZ decay :',ICHNDCY(I),
     $                             ' Br. ',GRCZBR(ICHNDCY(I)),' ON'
                           ELSE
                              WRITE(*,*) 'GRC::Error:',
     $                             ' different decay combination',
     $                             ' or different Br..'
                              GO TO 999
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDIF
               ENDIF 
               IF (OPTF(15:25).EQ.'===========') THEN
                  READ(11,'(a24,d15.6,a5)',END=1000) TMPA,CRGET,TMPB
                  WRITE(*,*)
                  WRITE(*,*) TMPA,CRGET,TMPB
                  GO TO 1000
               ENDIF
            ENDDO
 1000       CONTINUE
            CLOSE(11)
 
            GRCSIG(JPROC) = CRGET

         ELSE IF (IBSWRT.EQ.2) THEN

C...Check process.
            IF (NPRUP.GT.1) THEN
               WRITE(*,*) 'GRC::Error: Option IBSWRT=2',
     $              ' is only available for single process.'
               GO TO 999
            ENDIF

            WRITE(*,*) 'Read from ASCII file : ',GDATA,'\n'
            OPEN(14,FILE = GDATA,STATUS='old',FORM='formatted')

            DO WHILE (FOREVER)
               READ(14,'(a)',END=222) OPTF
               WRITE(*,*) OPTF
 
               IF (OPTF(2:8).EQ.'Process') THEN
                  READ(14,'(a4,a6,f10.3)',END=222) TMPA,TMPC,CHECM
                  WRITE(*,*) ' ECM     :',CHECM
               ELSE IF (OPTF(2:8).EQ.'StrFun1') THEN
                  IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5L') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 46
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5D') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 47
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5M') THEN
                     PDFGUP(1) = 4
                     PDFSUP(1) = 48
                  ENDIF

                  IF (OPTF(24:27).EQ.'prot') THEN
                     IDBMUP(1) = 2212
                  ELSE IF (OPTF(24:27).EQ.'anti') THEN
                     IDBMUP(1) = -2212
                  ELSE
                     WRITE(*,*) 'GRC::Error: Strange Col. particle 1.'
                     GO TO 999
                  ENDIF
               ELSE IF (OPTF(2:8).EQ.'StrFun2') THEN
                  IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5L') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 46
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5D') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 47
                  ELSE IF (OPTF(16:18).EQ.'CTE'
     $                 .AND.OPTF(20:21).EQ.'5M') THEN
                     PDFGUP(2) = 4
                     PDFSUP(2) = 48
                  ENDIF

                  IF (OPTF(24:27).EQ.'prot') THEN
                     IDBMUP(2) = 2212
                  ELSE IF (OPTF(24:27).EQ.'anti') THEN
                     IDBMUP(2) = -2212
                  ELSE
                     WRITE(*,*) 'GRC::Error: Strange Col. particle 2.'
                     GO TO 999
                  ENDIF

                  READ(14,'(a16,f12.5,a6)',END=222) TMPA,SIGEV,TMPB
                  WRITE(*,*) ' Cross Section :',SIGEV,' (pb)'

                  READ(14,'(a10,i3)',END=222) TMPA,NUP
                  WRITE(*,*) ' NUP     :',NUP

                  WRITE(*,*)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'IDBMUP(1)=',IDBMUP(1),'IDBMUP(2)=',IDBMUP(2)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'PDFGUP(1)=',PDFGUP(1),'PDFGUP(2)=',PDFGUP(2)
                  WRITE(*,FMT='(a11,i6,a15,i6)') 
     $                 'PDFSUP(1)=',PDFSUP(1),'PDFSUP(2)=',PDFSUP(2)

C...Check Parameter for Event Generation
C...Check Ecm
                  IF (ABS(CHECM-GRCECM).GT.0.0001) THEN
                     WRITE(*,*) 'Different CM Energy :',
     $                    '(Ascii file):',CHECM,
     $                    'GeV and (PROGRAM):',GRCECM,'GeV'
                     GO TO 999
                  ENDIF
               ENDIF

               IF (OPTF(2:3).EQ.'#') GO TO 222
            ENDDO
 222        CONTINUE

            WRITE(*,*) 'Read Ascii file!!'
            GRCSIG(JPROC) = SIGEV
         ELSE
            WRITE(*,*) 'GRC::Error: Wrong IBSWRT=',IBSWRT,'.'
            WRITE(*,*) '            Set IBSWRT <= 2 .'
            GO TO 999
         ENDIF
 
#if PSMODEL == 61
         CALL PYUPIN(ISUB,GRCPROC,GRCSIG(JPROC)*1E-9)
#endif
 
C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
         OPEN(23,FILE = GRCBSFILE(JPROC),
     $        STATUS='old',FORM='unformatted')
         CALL BSREAD(23)
         CLOSE(23)
#endif
C+-------------------------------------------------+

C...Event Generation (ibsint=0)
      ELSE IF (IBSINT.EQ.0) THEN

         IF (IBSWRT.LT.2) THEN

#ifndef GRCDEBUG
            OPEN(23,FILE = GRCBSFILE(JPROC),
     $           STATUS='old',FORM='unformatted')
            CALL BSREAD(23)
            CLOSE(23)
#endif

#if PSMODEL == 0
C...Set Seed for random number
            IRSEED=48828125*IRSEED
            IRSEED=IAND(IRSEED,NZZ)
            CALL DRNSET( IRSEED )
#endif

C...SPRING Event Generation
            MXTRY = 100000
            CALL SPRING( FUNC, MXTRY )

C  QCDPS result has to be locked, in order to avoid the re-execution 
C  by the following FUNC call.
            iqplock= 1

C...OK, accept
            IBSINT = 2
            YACOB = 1.0D0
            XDUMMY = FUNC(X)

C  Unlock QCDPS.
            iqplock= 0

C...For Debug
C+-------------------------------------------------+
#ifdef GRCDEBUG
            DO I = 1, NDIM
               CALL XHFILL(I,XRUN(I),XDUMMY)
            ENDDO
#endif
C+-------------------------------------------------+

C...Smear to PYTHIA/HERWIG
C            CALL GRCSSMR
            if( labcut.eq.0 ) CALL GRCSSMR
         ELSE IF (IBSWRT.EQ.2) THEN

C...Smear to PYTHIA
            CALL GRCSSMR
 
         ELSE
            WRITE(*,*) 'GRC::Error: Wrong ibswrt'
            GO TO 999
         ENDIF

         if( labcut.eq.0 ) then

C  Fix the particle ordering in the LHA common (HEPEUP).
C  This must be called before adding PS radiations.
            call lhaPartFix

C  PS radiation.
            if( ishower.eq.1 ) then
C  Determine the 3-dim. kinematics.
               ldebug= 0
               call pskini( ldebug, ifail )
               ifail_pskini= ifail
C  Add the radiation information to HEPEUP.
               if( ifail.eq.0 ) call ps2lha
            endif
 
         endif
 
      ENDIF

C...Set Cross Section
      IF (IBSINT.EQ.1.OR.IREJ.LT.1) THEN
#if PSMODEL == 61
         SIGEV = GRCSIG(JPROC)*1E-9
#else
         SIGEV = GRCSIG(JPROC)
#endif

      ELSE
         SIGEV = 0.0D0
      ENDIF
 
      RETURN
 999  CONTINUE
      STOP
      END

      subroutine lhaPartFix
C  Fix the particle ordering in HEPEUP, according to the requirement 
C  of PYTHIA.
C  PYTHIA requires the children from an identical parent(s) to be 
C  placed contiguously.
C  For instance, jets in W/Z + jets events must be placed just after 
C  or before the W/Z (before the decay products of W/Z), since their 
C  parents are the initial-state partons as the W/Z.
      IMPLICIT REAL*8 (A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      if( NUP.lt.1 ) return

C  W/Z + jets
      if( (IDPRUP.ge.101 .and. IDPRUP.le.104 )
     >     .or. (IDPRUP.ge.422 .and. IDPRUP.le.451 )
     >     .or. (IDPRUP.ge.111 .and. IDPRUP.le.114 )
     >     .or. (IDPRUP.ge.501 .and. IDPRUP.le.525 ) ) then
         call lhaPartCopy( 4, NUP +1 )
         call lhaPartCopy( 5, NUP +2 )
         do i=6,NUP
            idest= i -2
            call lhaPartCopy( i, idest )
         enddo
         call lhaPartCopy( NUP +1, NUP -1 )
         call lhaPartCopy( NUP +2, NUP )
      endif

C  WW/WZ/ZZ + jets
      if( (IDPRUP.ge.121 .and. IDPRUP.le.122 )
     >     .or. (IDPRUP.ge.131 .and. IDPRUP.le.132 )
     >     .or. (IDPRUP.ge.141 .and. IDPRUP.le.142 )
     >     .or. (IDPRUP.ge.553 .and. IDPRUP.le.579 ) ) then
         call lhaPartCopy( 5, NUP +1 )
         call lhaPartCopy( 6, NUP +2 )
         call lhaPartCopy( 7, NUP +3 )
         call lhaPartCopy( 8, NUP +4 )
         idest= 4
         do i=9,NUP
            idest= idest +1
            call lhaPartCopy( i, idest )
         enddo
         call lhaPartCopy( NUP +1, NUP -3 )
         call lhaPartCopy( NUP +2, NUP -2 )
         call lhaPartCopy( NUP +3, NUP -1 )
         call lhaPartCopy( NUP +4, NUP )
      endif

      return
      end

      subroutine lhaPartCopy( isrc, idst )
      IMPLICIT REAL*8 (A-H,O-Z)

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/ 

      IDUP(idst)= IDUP(isrc)
      ISTUP(idst)= ISTUP(isrc)
      MOTHUP(1,idst)= MOTHUP(1,isrc)
      MOTHUP(2,idst)= MOTHUP(2,isrc)
      ICOLUP(1,idst)= ICOLUP(1,isrc)
      ICOLUP(2,idst)= ICOLUP(2,isrc)
      do i=1,5
         PUP(i,idst)= PUP(i,isrc)
      enddo
      VTIMUP(idst)= VTIMUP(isrc)
      SPINUP(idst)= SPINUP(isrc)

      return
      end

C----------------
C Write Summary.
C----------------
      SUBROUTINE GRCBSRS(IO)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'

      INTEGER JPROC
      COMMON /AMJPRC/JPROC
      CHARACTER GRCPROC*40
      COMMON/GRCCHAR/GRCPROC

      INTEGER GRCCDCY
      EXTERNAL GRCCDCY
      INTEGER IO,I,IDCY
      CHARACTER RSCL*60,FSCL*60,CMODE*20

      WRITE(IO,FMT='(a14,f10.3,a4)') 'CM Energy :',GRCECM,'GeV'
      IF (IGPP.EQ.0) THEN
         WRITE(IO,FMT='(a14,a30)') 'Beam type :',
     $        'Proton-Proton Collision'
      ELSE IF (IGPP.EQ.1) THEN
         WRITE(IO,FMT='(a14,a30)') 'Beam type :',
     $        'Proton-Anti-Proton Collision'
      ENDIF
      WRITE(IO,*)
      WRITE(IO,FMT='(a14,i3,a2,a40)') 'Process : [',IGSUB,'] ',GRCPROC
      WRITE(IO,*)
      WRITE(IO,FMT='(i4,a17)') NNF-2,'body final state'
      DO I=1,NNF-2
         WRITE(IO,FMT='(a25,i2,a2,f7.3,a4)') 'Pt Cut of Particle',
     $        I,':',GRCPTCUT(I),'GeV'
      ENDDO
      DO I=1,NNF-2
         WRITE(IO,FMT='(a25,i2,a2,f7.3)') 'Eta Cut of Particle',
     $        I,':',GRCETACUT(I)
      ENDDO
      DO I=1,NNF-2
         WRITE(IO,FMT='(a25,i2,a2,f7.3)') 'Rcone Cut of Particle',
     $        I,':',GRCRCONCUT(I)
      ENDDO
      WRITE(IO,FMT='(a29,f7.3,a4)') 'Missing Pt(Et) Cut :',
     $     GRCMETCUT,'GeV'
      WRITE(IO,*)

      IF (ICOUP.EQ.1) RSCL = ' (s-hat of the hard interaction (GeV^2).)'
      IF (ICOUP.EQ.2) RSCL = ' (Average mT^2  (<mT^2>) (GeV^2).)'
      IF (ICOUP.EQ.3) RSCL = ' (Total mT^2  (sum mT^2) (GeV^2).)'
      IF (ICOUP.EQ.4) RSCL = ' (Maximum mT^2  (max mT^2) (GeV^2).)'
      IF (ICOUP.EQ.5) RSCL = ' (Fixed Q^2 (GeV^2).)'
      IF (ICOUP.EQ.6) RSCL = ' (user-defined Q^2 (GeV^2).)'
      IF (ICOUP.EQ.5) THEN
         WRITE(IO,FMT='(a25,i2,f10.3,a2,a41)') 
     $        ' Renormalization scale : ',icoup,GRCQ,'^2',RSCL
      ELSE
         WRITE(IO,FMT='(a25,i2,a41)') 
     $        ' Renormalization scale : ',icoup,RSCL
      ENDIF
      IF (IFACT.EQ.0) FSCL = ' (Same as renormalization scale.)'
      IF (IFACT.EQ.1) FSCL = ' (s-hat of the hard interaction (GeV^2).)'
      IF (IFACT.EQ.2) FSCL = ' (Average mT^2  (<mT^2>) (GeV^2).)'
      IF (IFACT.EQ.3) FSCL = ' (Total mT^2  (sum mT^2) (GeV^2).)'
      IF (IFACT.EQ.4) FSCL = ' (Maximum mT^2  (max mT^2) (GeV^2).)'
      IF (IFACT.EQ.5) FSCL = ' (Fixed Q^2 (GeV^2).)'
      IF (IFACT.EQ.6) FSCL = ' (user-defined Q^2 (GeV^2).)'
      IF (IFACT.EQ.5) THEN
         WRITE(IO,FMT='(a25,i2,f10.3,a2,a41)') 
     $        ' Factorization scale   : ',IFACT,GRCFAQ,'^2',FSCL
      ELSE
         WRITE(IO,FMT='(a25,i2,a41)') 
     $        ' Factorization scale   : ',IFACT,FSCL
      ENDIF

C...Check W-decay.
      IF (GRCCDCY(IGSUB).EQ.1) THEN
         IDCY = 0
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               IDCY = IDCY + 1
            ENDIF
         ENDDO
         WRITE(IO,*)
         WRITE(IO,FMT='(a20,i5)') 'W Forced decay :',IDCY
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               CALL GRCRDCY(1,I,CMODE)
               WRITE(IO,FMT='(a20,i5,f8.3,a20)') ' :',I,GRCWBR(I),CMODE
            ENDIF
         ENDDO
      ENDIF
C...Check Z-decay.
      IF (GRCCDCY(IGSUB).EQ.2) THEN
         IDCY = 0
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.0D0) THEN
               IDCY = IDCY + 1
            ENDIF
         ENDDO
         WRITE(IO,*)
         WRITE(IO,FMT='(a20,i5)') 'Z Forced decay :',IDCY
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.0D0) THEN
               CALL GRCRDCY(2,I,CMODE)
               WRITE(IO,FMT='(a20,i5,f8.3,a20)') ' :',I,GRCZBR(I),CMODE
            ENDIF
         ENDDO
      ENDIF
C...Check W & Z-decay.
      IF (GRCCDCY(IGSUB).EQ.3) THEN
         IDCY = 0
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.0D0) THEN
               IDCY = IDCY + 1
            ENDIF
         ENDDO
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               IDCY = IDCY + 1
            ENDIF
         ENDDO
         WRITE(IO,*)
         WRITE(IO,FMT='(a20,i5)') 'Z and W Forced decay :',IDCY
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.0.0D0) THEN
               CALL GRCRDCY(2,I,CMODE)
               WRITE(IO,FMT='(a20,i5,f8.3,a20)') ' :',I,GRCZBR(I),CMODE
            ENDIF
         ENDDO
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               CALL GRCRDCY(1,I,CMODE)
               WRITE(IO,FMT='(a20,i5,f8.3,a20)') ' :',I,GRCWBR(I),CMODE
            ENDIF
         ENDDO
      ENDIF
      WRITE(IO,*)

      RETURN
      END

C----------------------------------------------------------
C Output character for decay mode.
C IMODE : = 1 W Boson.
C           2 Z Boson.
C----------------------------------------------------------
      SUBROUTINE GRCRDCY(IMODE,IDCY,CMODE)
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INTEGER IMODE,IDCY,J
      CHARACTER CMODE*20
      CHARACTER CWDCY(20)*20,CZDCY(16)*20
      DATA (CWDCY(J),J=1,20)/
     $     '  W -> d u          ','  W -> d c          ',
     $     '  W -> d t          ','  W -> d Y          ',
     $     '  W -> s u          ','  W -> s c          ',
     $     '  W -> s t          ','  W -> s Y          ',
     $     '  W -> b u          ','  W -> b c          ',
     $     '  W -> b t          ','  W -> b Y          ',
     $     '  W -> X u          ','  W -> X c          ',
     $     '  W -> X t          ','  W -> X Y          ',
     $     '  W -> e nu_e       ','  W -> mu nu_mu     ',
     $     '  W -> tau nu_tau   ','  W -> lX lY        '/
      DATA (CZDCY(J),J=1,16)/
     $     '  Z -> d d          ','  Z -> u u          ',
     $     '  Z -> s s          ','  Z -> c c          ',
     $     '  Z -> b b          ','  Z -> t t          ',
     $     '  Z -> X X          ','  Z -> Y Y          ',
     $     '  Z -> e e          ','  Z -> nu_e nu_e    ',
     $     '  Z -> mu mu        ','  Z -> nu_mu nu_mu  ',
     $     '  Z -> tau tau      ','  Z -> nu_tau nu_tau',
     $     '  Z -> lX lX        ','  Z -> lY lY        '/
C W decay mode.
      IF (IMODE.EQ.1) THEN
         CMODE = CWDCY(IDCY)
C Z decay mode.
      ELSE IF (IMODE.EQ.2) THEN
         CMODE = CZDCY(IDCY)
      ENDIF
      RETURN
      END

C-------------
C Write logo.
C-------------
      SUBROUTINE GRCLOGO(IO)
      IMPLICIT NONE
 
      INTEGER IO,I,J
      CHARACTER LOGO(13)*48, LINE*79
 
      DATA (LOGO(J),J=1,13)/
     $     '      GGG  RRRR          PPPP  PPPP   AAA       ',
     $     '     G   G R   R   @@@@  P   P P   P A   A      ',
     $     '     G     R   R  @ @@ @ P   P P   P A   A      ',
     $     '     G GGG RRRR  @ @ @ @ PPPP  PPPP  AAAAA      ',
     $     '     G   G R  R  @ @@ @  P     P     A   A      ',
     $     '      GGG  R   R  @@@@@@ P     P     A   A      ',
     $     '    =======================================     ',
     $     '      GRace At Proton-Proton/Anti-proton        ',
     $     '      This is GR@PPA version 2.76               ',
     $     '      coded by S.Tsuno (tsuno@fnal.gov)         ',
     $     '      with Minami Tateya Collab. and ATLAS-J.   ',
     $     '      URL :  http://atlas.kek.jp/physics/nlo-wg/',
     $     '      Referances, hep-ph/0204222                '/
 
      DO I = 1,22
         LINE = ' '
         LINE(2:3) = '**'
         LINE(78:79) = '**'
 
         IF (I.LT.3) THEN
            LINE(4:77) = '**************************************'//
     $           '************************************'
         ELSE IF (I.EQ.3) THEN
            LINE(6:15) = 'Welcome to'
         ELSE IF (I.GT.4.AND.I.LT.13) THEN
            LINE(5:53) = LOGO(I-4)
         ELSE IF (I.GT.13.AND.I.LT.17) THEN
            LINE(5:53) = LOGO(I-5)
         ELSE IF (I.GT.17.AND.I.LT.20) THEN
            LINE(5:53) = LOGO(I-6)
         ELSE IF (I.GT.20) THEN
            LINE(4:77) = '**************************************'//
     $           '************************************'
         ENDIF
         WRITE(IO,'(a79)') LINE
      ENDDO
 
      RETURN
      END

C-------------------
C Make Decay Table.
C-------------------
      SUBROUTINE GRCSDCY
      IMPLICIT NONE
      INCLUDE 'grchad.inc'
      INCLUDE 'inclk.h'

      INTEGER I,J
      DOUBLE PRECISION TMPZBR(16),TMPWBR(20),TMPCKM(4,4)

C...Particle properties + some flavour parameters.
      GAMS(1)  = AMDQ(1)     ! d
      GAMS(2)  = AMUQ(1)     ! u
      GAMS(3)  = AMDQ(2)     ! s
      GAMS(4)  = AMUQ(2)     ! c
      GAMS(5)  = AMDQ(3)     ! b
      GAMS(6)  = AMUQ(3)     ! t
      GAMS(7)  = AMX         ! unknown
      GAMS(8)  = AMY         ! unknown
      GAMS(9)  = 0.0D0       ! gluon 1
      GAMS(10) = 0.0D0       ! gluon 2
      GAMS(11) = AMLP(1)     ! electron
      GAMS(12) = AMNU(1)     ! nu_e
      GAMS(13) = AMLP(2)     ! muon
      GAMS(14) = AMNU(2)     ! nu_mu
      GAMS(15) = AMLP(3)     ! tau
      GAMS(16) = AMNU(3)     ! nu_tau

      IF (IGSUB.EQ.408.OR.IGSUB.EQ.409.OR.IGSUB.EQ.410
     $     .OR.IGSUB.EQ.411.OR.IGSUB.EQ.412.OR.IGSUB.EQ.551
     $     .OR.IGSUB.EQ.555.OR.IGSUB.EQ.556.OR.IGSUB.EQ.566
     $     .OR.IGSUB.EQ.567.OR.IGSUB.EQ.568.OR.IGSUB.EQ.569
     $     .OR.IGSUB.EQ.570.OR.IGSUB.EQ.571.OR.IGSUB.EQ.572
     $     .OR.IGSUB.EQ.120.OR.IGSUB.EQ.121.OR.IGSUB.EQ.122
     $     .OR.IGSUB.EQ.170.OR.IGSUB.EQ.171) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.0.0D0) THEN
               DO J = 1,20
                  IF (IGWMOD(J).EQ.1.AND.GRCWBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCWBR(I)*GRCWBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.412.OR.IGSUB.EQ.421
     $        .OR.IGSUB.EQ.422.OR.IGSUB.EQ.423.OR.IGSUB.EQ.424
     $        .OR.IGSUB.EQ.425.OR.IGSUB.EQ.426.OR.IGSUB.EQ.427
     $        .OR.IGSUB.EQ.428.OR.IGSUB.EQ.429.OR.IGSUB.EQ.430
     $        .OR.IGSUB.EQ.431.OR.IGSUB.EQ.432.OR.IGSUB.EQ.433
     $        .OR.IGSUB.EQ.434.OR.IGSUB.EQ.435.OR.IGSUB.EQ.436
     $        .OR.IGSUB.EQ.437.OR.IGSUB.EQ.438.OR.IGSUB.EQ.439
     $        .OR.IGSUB.EQ.440.OR.IGSUB.EQ.441.OR.IGSUB.EQ.442
     $        .OR.IGSUB.EQ.443.OR.IGSUB.EQ.444.OR.IGSUB.EQ.445
     $        .OR.IGSUB.EQ.446.OR.IGSUB.EQ.447.OR.IGSUB.EQ.448
     $        .OR.IGSUB.EQ.449.OR.IGSUB.EQ.450.OR.IGSUB.EQ.451
     $        .OR.IGSUB.EQ.100.OR.IGSUB.EQ.101.OR.IGSUB.EQ.102
     $        .OR.IGSUB.EQ.103.OR.IGSUB.EQ.104.OR.IGSUB.EQ.109) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,20
            IF (IGWMOD(I).EQ.1.AND.GRCWBR(I).GT.1.0D-4) THEN
               IGNDCY = IGNDCY + 1
               GDCYPR1(IGNDCY) = I
               GDCYRAT(IGNDCY) = GRCWBR(I)
               GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.500.OR.IGSUB.EQ.501.OR.IGSUB.EQ.502
     $        .OR.IGSUB.EQ.503.OR.IGSUB.EQ.504.OR.IGSUB.EQ.505
     $        .OR.IGSUB.EQ.506.OR.IGSUB.EQ.507.OR.IGSUB.EQ.508
     $        .OR.IGSUB.EQ.509.OR.IGSUB.EQ.510.OR.IGSUB.EQ.511
     $        .OR.IGSUB.EQ.512.OR.IGSUB.EQ.513.OR.IGSUB.EQ.514
     $        .OR.IGSUB.EQ.515.OR.IGSUB.EQ.516.OR.IGSUB.EQ.517
     $        .OR.IGSUB.EQ.518.OR.IGSUB.EQ.519.OR.IGSUB.EQ.520
     $        .OR.IGSUB.EQ.521.OR.IGSUB.EQ.522.OR.IGSUB.EQ.523
     $        .OR.IGSUB.EQ.524.OR.IGSUB.EQ.525
     $        .OR.IGSUB.EQ.110.OR.IGSUB.EQ.111.OR.IGSUB.EQ.112
     $        .OR.IGSUB.EQ.113.OR.IGSUB.EQ.114) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               IGNDCY = IGNDCY + 1
               GDCYPR1(IGNDCY) = I
               GDCYRAT(IGNDCY) = GRCZBR(I)
               GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.550.OR.IGSUB.EQ.553.OR.IGSUB.EQ.554
     $        .OR.IGSUB.EQ.559.OR.IGSUB.EQ.560.OR.IGSUB.EQ.561
     $        .OR.IGSUB.EQ.562.OR.IGSUB.EQ.563.OR.IGSUB.EQ.564
     $        .OR.IGSUB.EQ.565.OR.IGSUB.EQ.140.OR.IGSUB.EQ.141
     $        .OR.IGSUB.EQ.142) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               DO J = 1,16
                  IF (IGZMOD(J).EQ.1.AND.GRCZBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCZBR(I)*GRCZBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ELSE IF (IGSUB.EQ.552.OR.IGSUB.EQ.557.OR.IGSUB.EQ.558
     $        .OR.IGSUB.EQ.573.OR.IGSUB.EQ.574.OR.IGSUB.EQ.575
     $        .OR.IGSUB.EQ.576.OR.IGSUB.EQ.577.OR.IGSUB.EQ.578
     $        .OR.IGSUB.EQ.579.OR.IGSUB.EQ.130.OR.IGSUB.EQ.131
     $        .OR.IGSUB.EQ.132) THEN
         IGNDCY = 0
         GDCYSUM = 0.0d0
         DO I = 1,16
            IF (IGZMOD(I).EQ.1.AND.GRCZBR(I).GT.1.0D-4) THEN
               DO J = 1,20
                  IF (IGWMOD(J).EQ.1.AND.GRCWBR(J).GT.1.0D-4) THEN
                     IGNDCY = IGNDCY + 1
                     GDCYPR1(IGNDCY) = I
                     GDCYPR2(IGNDCY) = J
                     GDCYRAT(IGNDCY) = GRCZBR(I)*GRCWBR(J)
                     GDCYSUM = GDCYSUM + GDCYRAT(IGNDCY)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END

C---------------------------------------------------
C Return decay flag: none:0,W-decay:1,Z-decay:2,ZW-decay:3
C---------------------------------------------------
      INTEGER FUNCTION GRCCDCY(ISUB)
      IMPLICIT NONE
      INTEGER ISUB
      GRCCDCY = 0
C...top productions.
      IF (ISUB.GT.407.AND.ISUB.LT.413) GRCCDCY = 1
      IF (ISUB.GT.169.AND.ISUB.LT.172) GRCCDCY = 1
C...W+jets productions.
      IF (ISUB.GT.420.AND.ISUB.LT.500) GRCCDCY = 1
      IF (ISUB.GT.99.AND.ISUB.LT.110)  GRCCDCY = 1
C...Z+jets productions.
      IF (ISUB.GT.499.AND.ISUB.LT.550) GRCCDCY = 2
      IF (ISUB.GT.109.AND.ISUB.LT.120) GRCCDCY = 2
C...ZZ+jets productions.
      IF (ISUB.EQ.550)                 GRCCDCY = 2
      IF (ISUB.GT.552.AND.ISUB.LT.555) GRCCDCY = 2
      IF (ISUB.GT.558.AND.ISUB.LT.566) GRCCDCY = 2
      IF (ISUB.EQ.140)                 GRCCDCY = 2
      IF (ISUB.EQ.141)                 GRCCDCY = 2
      IF (ISUB.EQ.142)                 GRCCDCY = 2
C...WW+jets productions.
      IF (ISUB.EQ.551)                 GRCCDCY = 1
      IF (ISUB.GT.554.AND.ISUB.LT.557) GRCCDCY = 1
      IF (ISUB.GT.565.AND.ISUB.LT.573) GRCCDCY = 1
      IF (ISUB.EQ.120)                 GRCCDCY = 1
      IF (ISUB.EQ.121)                 GRCCDCY = 1
      IF (ISUB.EQ.122)                 GRCCDCY = 1
C...ZW+jets productions.
      IF (ISUB.EQ.552)                 GRCCDCY = 3
      IF (ISUB.GT.556.AND.ISUB.LT.559) GRCCDCY = 3
      IF (ISUB.GT.572.AND.ISUB.LT.580) GRCCDCY = 3
      IF (ISUB.EQ.130)                 GRCCDCY = 3
      IF (ISUB.EQ.131)                 GRCCDCY = 3
      IF (ISUB.EQ.132)                 GRCCDCY = 3
      RETURN
      END

C--------------------------------------
C Make kinematics for decay particles.
C--------------------------------------
      SUBROUTINE GRCKDCY
      IMPLICIT NONE

C...Include GR@PPA common parameter.
      INCLUDE 'grchad.inc'
C      INCLUDE 'inclk.h'

C...User process event common block.
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500) 
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP  
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,IDUP(MAXNUP),
     &ISTUP(MAXNUP),MOTHUP(2,MAXNUP),ICOLUP(2,MAXNUP),PUP(5,MAXNUP),
     &VTIMUP(MAXNUP),SPINUP(MAXNUP)  
      SAVE /HEPEUP/

      DOUBLE PRECISION E3,P3,E4,COST,SINT,PHI,PA(4),PB(4),PX(4)
      DOUBLE PRECISION AM1,AM2,AM3,RMAS,RWID,XSND(30)
      DOUBLE PRECISION QMIN,QMAX,THEMIN,THEMAX,THE,QM,YCB
      INTEGER I,J,ICMX,IDUMMY,IPART,INPARNT(3)
      DOUBLE PRECISION BETA,GAMMA,BOSTZ,BOSTE
      DOUBLE PRECISION DRN
      EXTERNAL DRN

      IF (IREJ.GT.0) GO TO 999

C...Look for unstable particle.
      IPART = 0
      DO I = 3,NUP
         IF (IABS(IDUP(I)).EQ.6.OR.IABS(IDUP(I)).EQ.23
     $        .OR.IABS(IDUP(I)).EQ.24.OR.IABS(IDUP(I)).EQ.25) THEN
            IPART = IPART + 1
            INPARNT(IPART) = I
         ENDIF
      ENDDO
      IF (IPART.EQ.0) GO TO 999

      DO J = 1,IPART
C...Decide Decay products.
         XSND(1) = 0.0D0
         DO I = 2,IGNDCY+1
            XSND(I) = XSND(I-1)+GDCYRAT(I-1)/GDCYSUM
         ENDDO

         DO I = 1,IGNDCY
            IF (DRN(IDUMMY).GT.XSND(I)
     $           .AND.DRN(IDUMMY).LT.XSND(I+1)) THEN
C...W decays.
               IF (IABS(IDUP(INPARNT(J))).EQ.24) THEN
                  IDUP(NUP+1) = IGWDCY(2*GDCYPR1(I)-1)
                  IDUP(NUP+2) = IGWDCY(2*GDCYPR1(I))
                  IF (IDUP(INPARNT(J)).LT.0) THEN
                     IDUP(NUP+1) = -IDUP(NUP+1)
                     IDUP(NUP+2) = -IDUP(NUP+2)
                  ENDIF
C...Z decays.
               ELSE IF (IABS(IDUP(INPARNT(J))).EQ.23) THEN
                  IDUP(NUP+1) = IGZDCY(2*GDCYPR1(I)-1)
                  IDUP(NUP+2) = IGZDCY(2*GDCYPR1(I))
               ELSE
                  WRITE(*,*) 'TST::Error: Not support yet.',
     $                 IDUP(INPARNT(J))
               ENDIF
            ENDIF
         ENDDO
         AM1 = GAMS(IABS(IDUP(NUP+1)))
         AM2 = GAMS(IABS(IDUP(NUP+2)))

C...Boost back to CM frame.
         BETA = (PUP(3,1)+PUP(3,2))/(PUP(4,1)+PUP(4,2))
         GAMMA = 1.0D0/SQRT(1.0D0-BETA**2)
         PX(1) = PUP(1,INPARNT(J))
         PX(2) = PUP(2,INPARNT(J))
         PX(3) = GAMMA*(PUP(3,INPARNT(J))-BETA*PUP(4,INPARNT(J)))
         PX(4) = GAMMA*(PUP(4,INPARNT(J))-BETA*PUP(3,INPARNT(J)))
         AM3 = PUP(5,INPARNT(J))

C         IF (IABS(IDUP(INPARNT(J))).EQ.24
C     $        .AND.ABS(SPINUP(INPARNT(J))).GT.0
C     $        .AND.IABS(IDUP(NUP+1)).GT.6) THEN
C            SPINUP(INPARNT(J)) = -IDUP(INPARNT(J))/24
C         ENDIF

C...Set kinematics of decay products on CM frame.
 80      COST = -1.0D0 + 2.0D0*DRN(IDUMMY)

C...For gauge boson.
         IF (IABS(IDUP(INPARNT(J))).EQ.24
     $        .OR.IABS(IDUP(INPARNT(J))).EQ.23) THEN
            IF (SPINUP(INPARNT(J)).EQ.1
     $           .AND.(1.0D0+COST)**2.LT.4.0D0*DRN(IDUMMY)) GO TO 80
            IF (SPINUP(INPARNT(J)).EQ.0
     $           .AND.(1.0D0-COST**2).LT.1.0D0*DRN(IDUMMY)) GO TO 80
            IF (SPINUP(INPARNT(J)).EQ.-1
     $           .AND.(1.0D0-COST)**2.LT.4.0D0*DRN(IDUMMY)) GO TO 80
         ENDIF

         E3   = (AM3**2+AM1**2-AM2**2)/2.0D0/AM3
         P3   = SQRT((E3-AM1)*(E3+AM1))
         SINT = SQRT((1.0D0-COST)*(1.0D0+COST))
         PHI  = DRN(IDUMMY)*2.0D0*ACOS(-1.0D0)

         PA(1) = P3*SINT*COS(PHI)
         PA(2) = P3*SINT*SIN(PHI)
         PA(3) = P3*COST
         PA(4) = E3

         E4 = (AM3**2+AM2**2-AM1**2)/2.0D0/AM3
         PB(1) = -PA(1)
         PB(2) = -PA(2)
         PB(3) = -PA(3)
         PB(4) = E4

C...Boost to CM frame of parent particle.
         IF (PX(1).EQ.0.0D0.AND.PX(2).EQ.0.0D0) THEN
            BETA = PX(3)/PX(4)
            GAMMA = 1.0D0/SQRT(1.0D0-BETA**2)
            BOSTZ = GAMMA*(PA(3)-BETA*PA(4))
            BOSTE = GAMMA*(PA(4)-BETA*PA(3))
            PA(3) = BOSTZ
            PA(4) = BOSTE
            BOSTZ = GAMMA*(PB(3)-BETA*PB(4))
            BOSTE = GAMMA*(PB(4)-BETA*PB(3))
            PB(3) = BOSTZ
            PB(4) = BOSTE
            BETA = (PUP(3,1)+PUP(3,2))/(PUP(4,1)+PUP(4,2))
            GAMMA = 1.0D0/SQRT(1.0D0-BETA**2)
         ELSE
            CALL GRCW2LB(PX,PA,PA)
            CALL GRCW2LB(PX,PB,PB)
         ENDIF

C...Status codes for entries.
         ISTUP(INPARNT(J)) = 2
         ISTUP(NUP+1) = 1
         ISTUP(NUP+2) = 1

C...Position of the first and last mother particles for entries.
         MOTHUP(1,NUP+1) = INPARNT(J)
         MOTHUP(2,NUP+1) = 0
         MOTHUP(1,NUP+2) = INPARNT(J)
         MOTHUP(2,NUP+2) = 0

C...Set Kinematics.
         PUP(1,NUP+1) = PA(1)
         PUP(2,NUP+1) = PA(2)
         PUP(3,NUP+1) = GAMMA*(PA(3)+BETA*PA(4))
         PUP(4,NUP+1) = GAMMA*(PA(4)+BETA*PA(3))
         PUP(5,NUP+1) = AM1

         PUP(1,NUP+2) = PB(1)
         PUP(2,NUP+2) = PB(2)
         PUP(3,NUP+2) = GAMMA*(PB(3)+BETA*PB(4))
         PUP(4,NUP+2) = GAMMA*(PB(4)+BETA*PB(3))
         PUP(5,NUP+2) = AM2

C...Check and initialize the color string.
         DO I = NUP,NUP+2
            IF (PUP(4,I).LT.PUP(5,I)) IREJ = 3
            ICOLUP(1,I) = 0
            ICOLUP(2,I) = 0
         ENDDO

C...Look for max.Id code of color string.
         ICMX = 0
         DO I = 1,NUP
            ICMX = MAX(ICMX,ICOLUP(1,I),ICOLUP(2,I))
         ENDDO

C...Possible color strings for entries.
         IF (IABS(IDUP(NUP+1)).LT.10) THEN
            IF (IDUP(NUP+1).GT.0) THEN
               ICOLUP(1,NUP+1) = ICMX+1
               ICOLUP(2,NUP+2) = ICMX+1
            ELSE
               ICOLUP(2,NUP+1) = ICMX+1
               ICOLUP(1,NUP+2) = ICMX+1
            ENDIF
         ENDIF
         NUP = NUP + 2
      ENDDO

C...Done.
      RETURN
 999  CONTINUE
      RETURN
      END

C-------------------
C Make Decay Table.
C-------------------
      SUBROUTINE GRCINIT
      IMPLICIT NONE
      INCLUDE 'grchad.inc'

      INTEGER I,J

C...Set Branching Ratio, Decay Mode, and Decay Products of Z boson.
      DATA (GRCZBR(I),I=1,16)/0.153995D0, 0.11942D0,  0.153984D0,
     $                        0.119259D0, 0.152272D0, 0.0D0,
     $                        0.0D0,      0.0D0,      0.033576D0,
     $                        0.066806D0, 0.033576D0, 0.066806D0,
     $                        0.0335D0,   0.066806D0, 0.0D0,
     $                        0.0D0/      

      DATA (IGZMOD(I),I=1,16)/0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0/

      DATA (IGZDCY(I),I=1,32)/  1, -1,  2, -2,  3, -3,  4, -4,
     $                          5, -5,  6, -6,  7, -7,  8, -8,
     $                         11,-11, 12,-12, 13,-13, 14,-14,
     $                         15,-15, 16,-16, 17,-17, 18,-18/

C...Set Branching Ratio, Decay Mode, and Decay Products of W boson.
      DATA (GRCWBR(I),I=1,20)/0.321369D0, 0.016494D0, 0.0D0,
     $                        0.0D0,      0.016502D0, 0.320615D0,
     $                        0.0D0,      0.0D0,      0.00001D0,
     $                        0.000591D0, 0.0D0,      0.0D0,
     $                        0.0D0,      0.0D0,      0.0D0,
     $                        0.0D0,      0.108166D0, 0.108166D0,
     $                        0.108087D0, 0.0D0/

      DATA (IGWMOD(I),I=1,20)/0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0/

      DATA (IGWDCY(I),I=1,40)/ -1,  2, -1,  4, -1,  6, -1,  8,
     $                         -3,  2, -3,  4, -3,  6, -3,  8,
     $                         -5,  2, -5,  4, -5,  6, -5,  8,
     $                         -7,  2, -7,  4, -7,  6, -7,  8,
     $                        -11, 12,-13, 14,-15, 16,-17, 18/

C...Set CKM matrix.
      DATA ((GRCCKM(I,J),J=1,4),I=1,4)/
     $     0.95113D0,  0.04884D0,  0.00003D0,  0.00000D0,
     $     0.04884D0,  0.94940D0,  0.00176D0,  0.00000D0,
     $     0.00003D0,  0.00176D0,  0.99821D0,  0.00000D0,
     $     0.00000D0,  0.00000D0,  0.00000D0,  1.00000D0/

C...Set Jet Flavor.
      DATA (IGJFLV(I),I=1,8)/1,1,1,1,1,0,1,0/

C...Symmetric conversion array
      DATA IGPCOMB/0,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,1,0,0,1,1,0,0,1,1,
     $             0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,
     $             1,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     $             1,1,0,0,1,1,1,1,0,0,1,1,0,0,1,1,0,0,1,1,0,0,0,0,0,
     $             0,1,1,1,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,1,0,0,0,1,
     $             1,1,0,0,0,1,1,1,0,0,1,1,0,0,1,1,0,0,1,1,1,1,0,0,0,
     $             0,1,1,0,0,1,1/

C...Set option for ELWK parameters.
      IGAUGE = 1

C...Set option for Boson width correction.
      IWIDCOR = 1

C...Set CM energy. 
      GRCECM = 14000.0D0

C...Kinematical cuts.
      DO I = 1,8
         GRCPTCUT(I)   = 0.0D0
         GRCETACUT(I)  = 10.0D0
         GRCRCONCUT(I) = 0.0D0
      ENDDO

C...Fixed seed for BASES/SPRING random number.
      NZZ = 2147483647

      GPTCUT   = 20.0D0         ! Pt Cut for jets
      GETACUT  = 2.5D0          ! Eta Cut
      GRCONCUT = 0.4D0          ! RCone Cut

C...Top anomalous coupling factors.
      F0 = 1.0D0                ! SM
      F1 = 0.0D0
      F2 = 0.0D0

C...Set default masses.
      CALL SETMAS

C  Default setting of the advanced options.
      signedres= 1.d0
      matching= 0
      ishower= 0
      labcut= 0

      RETURN
      END
